define('smart-attachments/model/AttachmentCategory',
    [
        'jquery',
        'underscore',
        'jira/lib/class',
        'jira/util/events',
        'smart-attachments/base/DNDHelper',
        'smart-attachments/base/Functions',
        'smart-attachments/base/Operations',
        'smart-attachments/toolbox',
        'smart-attachments/model/AttachmentProgressBar',
        'smart-attachments/model/AttachmentItem'
    ], function ($, _, Class, Events, DNDHelper, Functions, Operations, Toolbox, AttachmentProgressBar, AttachmentItem) {

        var isMatchedElement = function (id) {
            return function (element) {
                return Functions.getAttachmentId(element) === id;
            }
        };

        var addDocumentDataToElement = function (element, document) {
            if (element) {
                element.dataset['document'] = 'true';
                element.dataset['documentId'] = document.id;
                element.dataset['documentName'] = document.name;
            }
        };

        var addDocumentDataToRevisionElement = function (element, document) {
            if (element) {
                element.dataset['revisionId'] = document.id;
            }
        };

        function isContainsAttachment() {
            var value = localStorage.getItem("dndItem");
            return (value && value === "attachment");
        }

        function isContainsCategory() {
            var value = localStorage.getItem("dndItem");
            return (value && value === "category");
        }

        var isObjectWithSameAttachmentId = function (attachmentId) {
            return function (obj) {
                return obj.getAttachmentId() === attachmentId;
            }
        };

        function cloneDocumentElement(element) {
            var documentElement =  element.cloneNode(true);
            var twixi = documentElement.querySelector('.twixi-block');
            if (twixi) {
                documentElement.appendChild(twixi.querySelector('.attachment-thumb'));
                var dl = twixi.querySelector('dl');
                var zipContentBlock = dl.querySelector('.zip-contents');
                zipContentBlock.parentNode.removeChild(zipContentBlock);
                documentElement.appendChild(dl);
                twixi.parentNode.removeChild(twixi);
            }
            return documentElement;
        }

        var AttachmentCategory = Class.extend({
            template: AttachCategory.category,

            init: function (attr) {
                _.extend(this, attr);
                this.getProjectKey = Toolbox.getProjectKeyFactory(this.issue.getIssueKey());
                this.makeRevisionsMap();
                this.$node = $(this.template({
                    category: this.category,
                    issueId: this.issue.getIssueId(),
                    issueKey: this.issue.getIssueKey(),
                    hasCreatePermission: this.pluginConfig.hasCreatePermission,
                    baseUrl: this.baseUrl,
                    isJSD: this.isJSDForm
                }));
                this.progressBars = [];
                this.$counterBlock = this.$node.find('.attach-cat-badge-wrapper span');
                this.itemsContainer = this.$node.find('#' + this.containerId);
                this.revisionContainer = this.$node.find('#file_attachments');
                this.$categoryTitle = $(this.$node.find('.attach-cat-title')[0]);
            },

            decorateDnD: function () {
                this.$node.on('dragover.smart drop.smart', this.onDragOver.bind(this));
                this.$node.on('dragleave.smart', this.onDragLeave.bind(this));
                this.$categoryTitle.attr('draggable', true);
                this.$categoryTitle.on('dragstart.smart', this.onDragStartCategory.bind(this));
                this.$categoryTitle.on('dragend.smart', this.onDragEndCategory.bind(this));
            },

            enableDND: function () {
                this.$categoryTitle.attr('draggable', true);
                this.$node.find('li.attachment-content:not(.revision)').attr('draggable', true);
            },

            disableDND: function () {
                this.$categoryTitle.attr('draggable', false);
                this.$node.find('li.attachment-content:not(.revision)').attr('draggable', false);
            },

            onDragStartCategory: function (event) {
                event.stopPropagation();
                var dataTransfer = event.originalEvent.dataTransfer;
                dataTransfer.effectAllowed = 'move';
                var dragPosition = DNDHelper.getDragPosition(this.$categoryTitle, event);
                dataTransfer.setDragImage(DNDHelper.createDragImage(this.$categoryTitle), dragPosition.x, dragPosition.y);
                dataTransfer.setData("text", this.category.id);
                localStorage.setItem("dndItem", "category");
                localStorage.setItem("position", this.position);
            },

            onDragEndCategory: function () {
                localStorage.removeItem("dndItem");
            },

            onDragOverAttachment: function () {
                this.$node.addClass('drop-mask-active');
            },

            onDragOverCategory: function (event) {
                var position = parseInt(localStorage.getItem("position"));
                if (this.position === position) {
                    return;
                }
                var originalEvent = event.originalEvent;
                var offset = this.$node.offset();
                var positionY = originalEvent.clientY;
                var halfHeight = this.$node.height() / 2;
                var isNeibor = Math.abs(this.position - position) === 1;
                if (positionY < offset.top + halfHeight) {
                    if (!(isNeibor && (this.position > position))) {
                        this.$node.addClass('drop-mask-active-before');
                    }
                    this.$node.removeClass('drop-mask-active-after');
                } else {
                    if (!(isNeibor && (position > this.position))) {
                        this.$node.addClass('drop-mask-active-after');
                    }
                    this.$node.removeClass('drop-mask-active-before');
                }
            },

            onDragOver: function (event) {
                var isCategory = isContainsCategory(event);
                var isAttachment = isContainsAttachment(event);
                if (!isAttachment && !isCategory) {
                    return;
                }

                event.preventDefault();
                event.stopPropagation();

                if (event.originalEvent) {
                    event.originalEvent.dataTransfer.dropEffect = "move";
                }

                if (isAttachment) {
                    this.onDragOverAttachment(event);
                }

                if (isCategory) {
                    this.onDragOverCategory(event);
                }

                if (event.type === 'drop') {
                    var text = event.originalEvent.dataTransfer.getData('text');
                    if (isAttachment) {
                        if (text !== "") {
                            var data = JSON.parse(text);
                            this.onItemMove(null, data.attachmentId, data.categoryId, this.category.id);
                        }
                    }
                    if (isCategory) {
                        if (text !== "") {
                            var categoryId = parseInt(text);
                            var isAfter = this.$node.hasClass('drop-mask-active-after');
                            var insertPosition = this.position + (isAfter ? 1 : 0);
                            this.onCategoryMove(insertPosition, categoryId);
                        }
                    }
                    this.onDragLeave(null);
                }
            },

            onDragLeave: function () {
                this.$node.removeClass('drop-mask-active');
                this.$node.removeClass('drop-mask-active-before');
                this.$node.removeClass('drop-mask-active-after');
            },

            makeRevisionsMap: function () {
                var ctx = this;
                var revisions = [];
                var documentElements = [];

                this.revisionsMap = this.category.documents
                    .reduce(function (collector, document) {
                        var attachmentIds = document.attachmentIds.slice();
                        var revisionIds = attachmentIds.reverse();
                        var lastRevisionId = revisionIds[0];
                        var foundElement = _.find(ctx.attachmentElements, isMatchedElement(lastRevisionId));
                        if (foundElement) {
                            var documentElement = cloneDocumentElement(foundElement);
                            addDocumentDataToElement(documentElement, document);
                            documentElements.push(documentElement);
                            revisions = revisions.concat(revisionIds);
                            collector[lastRevisionId] = revisionIds.map(function (id) {
                                var revisionElement = _.find(ctx.attachmentElements, isMatchedElement(id));
                                addDocumentDataToRevisionElement(revisionElement, document);
                                return revisionElement
                            });
                        }
                        return collector;
                    }, {});

                this.attachmentElements = _.filter(this.attachmentElements, function (attachmentElement) {
                    var attachmentId = Functions.getAttachmentId(attachmentElement);
                    return revisions.indexOf(attachmentId) === -1;
                });

                this.attachmentElements = this.attachmentElements.concat(documentElements);
            },

            isContainsAttachmentId: function (attachmentId) {
                return _.some(this.progressBars, isObjectWithSameAttachmentId(attachmentId))
            },

            addProgressBar: function (progressBar) {
                var delegate = new AttachmentProgressBar(progressBar);
                delegate.bind('onBeforeDestroy', function () {
                    this.progressBars = _.filter(this.progressBars, function (item) {
                        return item.delegate.file !== progressBar.file;
                    });
                    this.renderEmptyBlock();
                }.bind(this));
                this.progressBars.push(delegate);
                this.itemsContainer.append(delegate.render().$node);
                this.renderEmptyBlock();
            },

            releaseElement: function (itemView, removeElement) {

                this.attachmentElements = _.filter(this.attachmentElements, function (element) {
                    return element !== itemView.element;
                });

                this.itemsView = _.filter(this.itemsView, function (view) {
                    return view.element !== itemView.element;
                });

                itemView.remove(removeElement);
                var moveInfo = itemView.getMoveInfo();
                this.removeAttachmentInfoFromCategory(moveInfo);
                this.renderCounter(this.attachmentElements.length);
                this.renderEmptyBlock();
                return moveInfo;
            },

            onItemRemove: function (event, itemView) {
                if (this.isAgile) {
                    GH.DetailsView.reload();
                } else {
                    this.releaseElement(itemView, true);
                    this.trigger('attachment:remove', [this, itemView]);
                }
            },

            onItemMove: function (event, attachmentId, sourceCategoryId, targetCategoryId) {
                if (sourceCategoryId !== targetCategoryId) {
                    this.trigger('attachment:move', [attachmentId, sourceCategoryId, targetCategoryId]);
                }
            },

            onCategoryMove: function (position, categoryId) {
                this.trigger('category:position', [position, categoryId]);
            },

            onPrepareForRevision: function (event, versionInfo) {
                this.trigger('attachment:prepareForRevision', [versionInfo]);
            },

            createAttachmentView: function (attachmentElement) {
                var issue = this.issue;
                var canAddComment = Toolbox.isFVIModuleEnabled() && this.pluginConfig.canAddComments;
                var attachmentId = Functions.getAttachmentId(attachmentElement);
                var attachmentItemView = new AttachmentItem({
                    containerId: this.containerId,
                    element: attachmentElement,
                    hasCreatePermission: this.pluginConfig.hasCreatePermission,
                    canAddComment: canAddComment,
                    category: this.category,
                    categories: this.categories,
                    revisions: this.revisionsMap[attachmentId] || [],
                    issue: issue,
                    isRevision: false
                });
                attachmentItemView.bind('attachment:prepareForRevision', this.onPrepareForRevision.bind(this));
                attachmentItemView.bind('attachment:remove', this.onItemRemove.bind(this));
                attachmentItemView.bind('attachment:move', this.onItemMove.bind(this));
                return attachmentItemView;
            },

            addAttachmentInfoToCategory: function (attachmentInfo) {
                var category = this.category;

                if (attachmentInfo.isDocument) {
                    var document = {
                        id: attachmentInfo.documentId,
                        name: attachmentInfo.documentName,
                        attachmentIds: attachmentInfo.attachmentIds
                    };
                    category.documents.push(document);
                } else {
                    attachmentInfo.attachmentIds.forEach(function (attachmentId){
                        if (category.attachmentIds.indexOf(attachmentId) === -1) {
                            category.attachmentIds.push(attachmentId);
                        }
                    });
                }

            },

            removeAttachmentInfoFromCategory: function (attachmentInfo) {
                var category = this.category;
                category.attachmentIds = _.filter(category.attachmentIds, function (attachmentId) {
                    return attachmentInfo.attachmentIds.indexOf(attachmentId) === -1;
                });
                if (attachmentInfo.isDocument) {
                    category.documents = _.filter(category.documents, function (document) {
                        return document.id !== attachmentInfo.documentId;
                    });
                }
            },

            findProgressBar: function (attachmentId) {
                return _.find(this.progressBars, isObjectWithSameAttachmentId(attachmentId));
            },

            removeProgressBarByAttachmentId: function (attachmentId) {
                var progressBar = this.findProgressBar(attachmentId);
                var matchFn = isObjectWithSameAttachmentId(attachmentId);
                this.progressBars = _.filter(this.progressBars, function (progressBar) {
                    return !progressBar.getAttachmentId() || !matchFn(progressBar);
                });
                return progressBar;
            },

            addAttachmentElement: function (attachmentElement, opts) {
                var attachmentId = Functions.getAttachmentId(attachmentElement);
                var moveInfo = opts || {
                    attachmentIds: [attachmentId],
                    isDocument: false
                };

                this.attachmentElements.push(attachmentElement);
                this.addAttachmentInfoToCategory(moveInfo);

                var progressBar = this.removeProgressBarByAttachmentId(attachmentId);

                if (moveInfo.isDocument) {
                    this.revisionsMap[attachmentId] = opts.revisionElements;
                    this.render();
                } else {
                    var attachmentView = this.createAttachmentView(attachmentElement);
                    if (progressBar) {
                        progressBar.$node.replaceWith(attachmentView.render());
                    } else {
                        this.itemsContainer.append(attachmentView.render());
                    }
                    this.itemsView.push(attachmentView);
                    this.renderCounter(this.attachmentElements.length);
                    this.renderEmptyBlock();
                }
            },

            renderCounter: function (value) {
                this.$counterBlock.html(value);
            },

            renderEmptyBlock: function () {
                if (this.isEmpty() && (this.progressBars.length === 0)) {
                    this.$node.find('.attach-cat-empty').removeClass('hidden');
                    this.$node.find('.attach-cat-download-all').hide();
                } else {
                    this.$node.find('.attach-cat-empty').addClass('hidden');
                    this.$node.find('.attach-cat-download-all').show();
                }
            },

            render: function () {
                this.itemsContainer.empty();
                this.revisionContainer.empty();
                this.itemsView = this.attachmentElements.map(this.createAttachmentView.bind(this));

                _.each(this.itemsView, function (itemView) {
                    if (itemView.isHaveRevisions()) {
                        this.revisionContainer.append(itemView.render());
                        _.each(itemView.revisionViews, function (revisionView) {
                            this.revisionContainer.append(revisionView.render());
                        }, this);
                    } else {
                        this.itemsContainer.append(itemView.render());
                    }
                }, this);

                //todo может старый код для 6 Жиры
                var galleryImg = this.$node.find('.gallery img');
                if (galleryImg.length) {
                    galleryImg.attr('height', '');
                    galleryImg.attr('width', '');
                }

                this.renderCounter(this.attachmentElements.length);
                this.renderEmptyBlock();
                this.attachBulkModeHandlers();
                return this.$node;
            },

            collapse: function () {
                this.$node.addClass("collapsed");
            },

            toggleCollapse: function (event) {
                event.stopPropagation();
                event.preventDefault();
                this.$node.toggleClass("collapsed");
                Operations.toggleCollapsed(this.getProjectKey(), this.category.id, this.$node.hasClass('collapsed'));
            },

            attachHandlers: function () {
                var toggleCollapsedHandler = this.toggleCollapse.bind(this);
                this.$categoryTitle.click(toggleCollapsedHandler);
                this.$node.find('.section-title span.aui-icon').on('click', toggleCollapsedHandler);
                this.$node.find('a.attach-cat-download-all').on('click', this.onDownloadAll.bind(this));
                this.$node.find('a.attach-cat-add').on('click', this.onPrepareForUpload.bind(this));
                this.$node.find('.attach-cat-bulk-checkbox').on('click', this.toggleSelectionForAllAttachment.bind(this));
                this.decorateDnD();
            },

            attachBulkModeHandlers: function () {
                this.$node.find('ol.item-attachments').on('click', '.sa-invisible-panel-wrap', this.toggleSelectCheckbox.bind(this));
            },

            toggleSelectionForAllAttachment: function () {
                var $li = this.$node.find('li.attachment-content:not(.revision)');
                if (event.target.checked) {
                    $li.addClass('selected-attach');
                } else {
                    $li.removeClass('selected-attach');
                }
            },

            toggleSelectCheckbox: function () {
                var allCount = this.$node.find('li.attachment-content:not(.revision)').length;
                var selectedCount = this.$node.find('li.selected-attach').length;
                if (selectedCount === allCount) {
                    this.$node.find('.attach-cat-bulk-checkbox').attr('checked', true);
                } else {
                    this.$node.find('.attach-cat-bulk-checkbox').removeAttr('checked');
                }
            },

            onDownloadAll: function (event) {
                event.stopPropagation();
                event.preventDefault();
                var href = event.delegateTarget.getAttribute('href');
                Operations.downloadFileFromPath(href);
            },

            onPrepareForUpload: function () {
                this.trigger('attachment:prepareForUpload', [this.category, this.issue]);
            },

            getAttachmentInfo: function (opts) {
                var options = opts || {};
                return this.itemsView.reduce(function (result, view) {
                    if (view.isDocument) {
                        var deep = 0;
                        if (options.withLastRevision) {
                            deep = 1;
                        }
                        if (options.withAllRevisions) {
                            deep = view.revisionViews.length;
                        }
                        for(var i = 0; i<deep; i++){
                            var lastRevision = view.revisionViews[i];
                            result.push({id: lastRevision.getItemId(), name: lastRevision.attachmentName});
                        }
                    } else {
                        result.push({id: view.getItemId(), name: view.attachmentName});
                    }
                    return result;
                },[]);
            },

            getAttachmentView: function (attachmentId) {
                return _.find(this.itemsView, function (view) {
                    return view.getItemId() === attachmentId;
                });
            },

            getSelectedItems: function () {
                return _.filter(this.itemsView, function (item) {
                    return item.isSelected();
                })
            },

            isEmpty: function () {
                return (typeof this.attachmentElements === 'undefined') || this.attachmentElements.length === 0;
            }
        });
        return AttachmentCategory;
    });