define('smart-attachments/base/AttachmentCompare', ['underscore', 'exports'], function (_, exports) {
    var Filename = function (filename, extension) {
        return {
            getFilename: function () {
                return filename;
            },

            getExtension: function () {
                return extension;
            }
        }
    };

    var isStartedFrom = function (a, b) {
        return a.getFilename().indexOf(b.getFilename()) === 0;
    };

    var isSameName = function (a, b) {
        return a.getFilename() === b.getFilename();
    };

    var isSameExtension = function (a, b) {
        return a.getExtension() === b.getExtension();
    };

    function toFilename(value) {
        var s = value || '';
        s = s.toUpperCase();
        var pos = s.lastIndexOf('.');
        var filename = s;
        var extension = '';
        if (pos > -1) {
            filename = s.substr(0, pos);
            extension = s.substr(pos);
        }
        return new Filename(filename, extension);
    }

    var isSuitable = function (a, b) {
        return isSameExtension(a, b) && isStartedFrom(a, b);
    };

    var isSameFile = function (a, b) {
        return isSameExtension(a, b) && isSameName(a, b);
    };

    var MatchMode = {
        PARTIAL: 'partial',
        FULL_MATCH: 'fullmatch'
    };

    exports.MATCH_MODE = MatchMode;

    var comparators = [];
    comparators[MatchMode.PARTIAL] = isSuitable;
    comparators[MatchMode.FULL_MATCH] = isSameFile;

    exports.isSuitableForRevision = function (matchMode) {
        return function (fileName, candidate) {
            var listFilename = toFilename(fileName);
            var candidateFilename = toFilename(candidate);
            return comparators[matchMode](candidateFilename, listFilename);
        }
    }
});