define('smart-attachments/base/Operations',
    [
        'jquery', 'underscore', 'aui/params', 'jira/ajs/ajax/smart-ajax'
    ], function ($, _, params, SmartAjax) {
        function deleteAttachment(id) {
            var result = $.Deferred();
            SmartAjax.makeRequest({
                type: 'DELETE',
                url: params.baseURL + '/rest/api/2/attachment/' + id,
                success: function (data, statusText, xhr) {
                    if (xhr.status === 204) {
                        result.resolve({});
                    }
                    if (xhr.status === 403) {
                        result.reject("No permission");
                    }

                    if (xhr.status === 404) {
                        result.reject("No permission");
                    }
                }
            });
            return result;
        }

        function deleteDocuments(issueKey, ids) {
            var result = $.Deferred();
            SmartAjax.makeRequest({
                type: 'POST',
                url: params.baseURL + '/rest/attach-cat/1.0/attachments/remove',
                data: {
                    issueKey: issueKey,
                    ids: ids
                },
                success: function () {
                    result.resolve({});
                }
            });
            return result;
        }

        function downloadFileFromPath(path) {
            var a = document.createElement('A');
            a.href = path;
            a.download = path.substr(path.lastIndexOf('/') + 1);
            document.body.appendChild(a);
            a.click();
            document.body.removeChild(a);
        }

        function downloadAttachments(issueKey, ids) {
            downloadFileFromPath(params.baseURL + '/rest/attach-cat/1.0/attachments/download-bulk?' +
                'issueKey=' + issueKey +
                "&ids=" + ids.join("&ids="));

        }

        function moveAttachment(issueKey, targetId, attachments) {
            var result = $.Deferred();
            SmartAjax.makeRequest({
                type: 'POST',
                url: params.baseURL + '/rest/attach-cat/1.0/attachments/move',
                data: {
                    issueKey: issueKey,
                    toCatId: targetId,
                    attachments: attachments
                },
                success: function () {
                    result.resolve({});
                }
            });
            return result;
        }

        function archiveToCategory(issueKey, fileName, targetCategoryId, plainAttachmentIds, documentsIds) {
            var result = $.Deferred();
            SmartAjax.makeRequest({
                type: 'POST',
                url: params.baseURL + '/rest/attach-cat/1.0/attachments/archive',
                data: {
                    issueKey: issueKey,
                    ids: plainAttachmentIds,
                    docIds: documentsIds,
                    cat: targetCategoryId,
                    name: fileName
                },
                success: function () {
                    result.resolve({});
                },
                error: function () {
                    result.reject({});
                }
            });
            return result;
        }

        function toggleCollapsed(projectKey, categoryId, state) {
            SmartAjax.makeRequest({
                type: 'POST',
                url: params.baseURL + '/rest/attach-cat/1.0/attachments/settings',
                data: {
                    projectKey: projectKey,
                    id: categoryId,
                    value: state
                }
            });
        }

        function renameAttachment(issueKey, categoryId, attachmentId, documentId, filename) {
            SmartAttachAnalytics.eventAttachmentRename();
            var data = {
                issueKey: issueKey,
                catId: categoryId,
                id: attachmentId,
                name: filename
            };
            if (documentId) {
                data.documentId = documentId;
            }
            var result = $.Deferred();
            SmartAjax.makeRequest({
                type: 'POST',
                url: params.baseURL + '/rest/attach-cat/1.0/attachments/rename',
                data: data,
                success: function () {
                    result.resolve(filename);
                }
            });
            return result;
        }

        function saveCategoriesPosition(projectKey, positions) {
            SmartAjax.makeRequest({
                type: 'POST',
                url: params.baseURL + '/rest/attach-cat/1.0/attachments/reorder',
                data: {
                    key: projectKey,
                    ids: positions
                }
            });
        }

        function saveCategorizedAttachments(issueId, issueKey, files) {
            var result = $.Deferred();
            SmartAjax.makeRequest({
                type: 'POST',
                url: params.baseURL + '/rest/attach-cat/1.0/attachments',
                data: {
                    issueKey: issueKey,
                    issueId: issueId,
                    tempAttachments: JSON.stringify(files)
                },
                success: function () {
                    result.resolve({});
                }
            });
            return result;
        }

        var categoriesRestUrl = '/rest/attach-cat/1.0/attachments/categories';

        function getCategories(id, issueType, isIssue) {
            var result = $.Deferred();
            SmartAjax.makeRequest({
                type: 'GET',
                url: params.baseURL + categoriesRestUrl,
                async: true,
                data: {
                    pid: id,
                    issueType: issueType ? issueType : -1,
                    isIssue: isIssue
                },
                success: function (data, statusText, xhr) {
                    var categories = xhr.status === 200 ? data.categories : [];
                    var fieldTypeDropZones = data ? data.dropZones : true;
                    result.resolve({
                        categories: categories,
                        fieldTypeDropZone: fieldTypeDropZones
                    })
                },
                error: function (jqXHR) {
                    console.error(categoriesRestUrl + ' ' + jqXHR.status + ' ' + jqXHR.statusText);

                    if (jqXHR.status !== 412) {
                        JIRA.Messages.showMsg(AJS.I18n.getText("attach.cat.get.error"), {
                            type: JIRA.Messages.Types.ERROR,
                            closeable: false,
                            timeout: 3
                        });
                    }
                    result.resolve({
                        categories: [],
                        fieldTypeDropZone: false
                    })
                }
            });
            return result;
        }

        return {
            getCategories:getCategories,
            deleteAttachment: deleteAttachment,
            deleteDocuments: deleteDocuments,
            downloadAttachments: downloadAttachments,
            moveAttachment: moveAttachment,
            archiveToCategory: archiveToCategory,
            downloadFileFromPath: downloadFileFromPath,
            toggleCollapsed: toggleCollapsed,
            renameAttachment: renameAttachment,
            saveCategoriesPosition: saveCategoriesPosition,
            saveCategorizedAttachments: saveCategorizedAttachments
        }
    });

