define('smart-attachments/model/AttachmentProgressBar', ['jquery', 'jira/lib/class'], function ($, Class) {

    var AttachmentProgressBar = Class.extend({
        template: AttachCategory.attachmentProgressBar,
        init: function (progressBar) {
            this.delegate = progressBar;

            this.delegate.bind('onBeforeDestroy', function () {
                this.trigger('onBeforeDestroy', [this]);
            }.bind(this));

            this.monitorUpload(progressBar.result);
        },

        getFileID: function () {
            return this.delegate.getFileID();
        },

        getAttachmentId: function () {
            return this.delegate.attachmentId;
        },

        monitorUpload: function (promise) {
            promise.progress(this.setProgress.bind(this));
            promise.fail(this.setFailed.bind(this));
        },

        setProgress: function (state) {
            this.progress = state;
            this.updateProgress();
        },

        setFailed: function (message, isPrecondition) {
            if (typeof isPrecondition === 'boolean') {
                this.$node.addClass('UploadProgressBar_uploadFailed');
            }
            var progressBar = this.$node.find('.upload-progress-bar__progress-bar');
            progressBar.find('.upload-progress-bar__bar').css('width', '100%');
            progressBar.addClass('upload-progress-bar__progress-bar-error');
        },

        updateProgress: function () {
            var progressBar = this.$node.find('.upload-progress-bar__progress-bar');
            progressBar.toggleClass('UploadProgressBar_progressUnknown', this.progress === false);
            if (this.progress >= 0) {
                progressBar.find('.upload-progress-bar__bar').css('width', 100 * this.progress + '%');
            }
            if ((100 * this.progress) > 99) {
                progressBar.addClass('upload-progress-bar__progress-bar-finished');
            }
        },

        render: function () {
            this.$node = $(this.template({
                fileName: this.delegate.getFileName()
            }));
            this.$node.find('.cancel-action').on('click', function () {
                this.delegate.destroy();
                this.$node.remove();
            }.bind(this));
            var isListView = this.delegate.$node.closest('ol').attr('id') === 'file_attachments';
            if (isListView) {
                var $span = this.delegate.$node.find('.attachment-thumb span.aui-icon');
                if ($span.length) {
                    this.$node.find('span.image-container').append($span[0]);
                }
            } else {
                var $image = this.delegate.$node.find('img');
                if (!$image.length) {
                    $image = this.delegate.$node.find('.attachment-thumbnail-icon');
                }
                this.$node.find('span.image-container').append($image[0]);
            }

            return this;
        }
    });

    return AttachmentProgressBar;
});