define('smart-attachments/base/AutoDistribution', ['underscore', 'jquery'], function (_, $) {
    var AUTO_CATEGORY = {
        id: 'auto',
        name: AJS.I18n.getText('attach.cat.drop-zone.auto-distribution.description'),
        position: 0
    };

    function matchFileNameByPattern(fileName, pattern) {
        var patternItems = pattern.split(',');

        for (var i = 0; i < patternItems.length; i++) {
            var patternItem = patternItems[i].trim();
            if (fileName.indexOf(patternItem) !== -1) {
                return true;
            }
        }

        return false;
    }

    function matchFileNameByRegexp(fileName, regexp) {
        return !!fileName.match(regexp);
    }

    function isMatchedByAutoDistributionRule(category, fileName) {
        var rule = category.autoDistributionRule;

        switch (rule.typeId) {
            case 0: {
                return matchFileNameByPattern(fileName, rule.value);
            }
            case 1: {
                return matchFileNameByRegexp(fileName, rule.value);
            }
            default: {
                return false;
            }
        }
    }

    var AutoDistribution = function () {
    };
    _.extend(AutoDistribution.prototype, {
        getAutoDistributionCategoryTemplate: function () {
            return AUTO_CATEGORY;
        },

        isNeedToShowAutoDistributionZone: function (categories) {
            return categories.length > 1 && this.hasCategoryAutoDistributionRule(categories);
        },

        hasCategoryAutoDistributionRule: function (categories) {
            for (var i = 0; i < categories.length; i++) {
                if (categories[i].autoDistributionRule) {
                    return true;
                }
            }
            return false;
        },

        bindAutoDistributionDragOverZoneForForms: function (categories, container) {
            var autoDistributionZone = container.find('[data-category-id="auto"]');

            autoDistributionZone.on('dragover', function () {
                container.find('.cat-cell-container-hover').removeClass('cat-cell-container-hover');
                autoDistributionZone.find('.cat-cell-container').addClass('cat-cell-container-hover');
            });

            autoDistributionZone.on('drop', this.onDropAutoDistributionZone.bind({
                categories: categories,
                container: container,
                matchByRuleCategoryId: this.matchByRuleCategoryId,
                createAndTriggerDropEventForZone: this.createAndTriggerDropEventForZone
            }));
        },

        onDropAutoDistributionZone: function (event) {
            event.preventDefault();
            event.stopPropagation();
            event.stopImmediatePropagation();

            if (event.originalEvent) {
                var files = event.originalEvent.dataTransfer.files;
                for (var i = 0; i < files.length; i++) {
                    var file = files[i];
                    var categoryId = this.matchByRuleCategoryId(this.categories, file);
                    var targetCategoryDragOverZone = this.container.find('[data-category-id=' + categoryId + ']');
                    this.createAndTriggerDropEventForZone(file, targetCategoryDragOverZone);
                }
            }

            return false;
        },

        createAndTriggerDropEventForZone: function (file, zone) {
            var dropEvent = $.Event('drop');
            dropEvent.originalEvent = {};
            dropEvent.originalEvent['dataTransfer'] = {
                dropEffect: 'none',
                effectAllowed: 'all',
                files: [file],
                types: ['Files']
            };
            dropEvent.originalEvent['preventDefault'] = dropEvent.preventDefault;
            dropEvent.originalEvent['stopPropagation'] = dropEvent.stopPropagation;
            dropEvent.originalEvent['stopImmediatePropagation'] = dropEvent.stopImmediatePropagation;
            zone.trigger(dropEvent);
        },

        matchByRuleCategoryId: function (categories, file) {
            var category = _.find(categories, function (category) {
                return (category.autoDistributionRule && isMatchedByAutoDistributionRule(category, file.name))
            });
            if (!category) {
                category = findDefaultCategoryOrFirstAvailable(categories);
            }
            return category.id;
        },

        createMatcher: function (categoryId, categories) {
            var matcher = function () {
                return (typeof categoryId === 'string') ? parseInt(categoryId) : categoryId;
            };

            if (categoryId === 'auto') {
                matcher = function (file) {
                    return this.matchByRuleCategoryId(categories, file)
                }.bind(this);
            }

            return {
                match: matcher
            }
        }
    });

    function findDefaultCategoryOrFirstAvailable(categories) {
        var category = _.find(categories, function (category) {
            return category.id === 0;
        });
        if (!category) {
            category = categories[0];
        }
        return category;
    }

    return new AutoDistribution();
});