define('smart-attachments/base/DNDHelper', ['jquery'], function ($) {
    function getDragPosition($node, e) {
        var offset = $node.offset();
        return {
            x: e.originalEvent.pageX - offset.left,
            y: e.originalEvent.pageY - offset.top
        }
    }

    function createDragImage($node) {
        var container = $($node.parent().get(0).cloneNode(false));
        $(document.body).append(container);

        var $img = $($node.clone());
        $img.find('.attachment-delete').remove();
        $img.addClass('draggable-ghost');
        $img.css("width", $node.width()).appendTo($(container).get(0));

        setTimeout(function () {
            container.remove();
        }, 10);
        return $img[0];
    }

    function isMicrosoftBrowser() {
        return (navigator.userAgent && navigator.userAgent.indexOf("MSIE") > -1)
            || (navigator.userAgent && navigator.userAgent.indexOf("Edge") > -1)
            || !!document.documentMode === true;
    }

    function setDataTransferAttachmentDownloadURL(e, $attachment) {
        if (e.originalEvent.dataTransfer) {
            var downloadUrl = $attachment.data('downloadurl') ? $attachment.data('downloadurl') :
                $attachment.find('a[data-downloadurl]').data('downloadurl');
            if (downloadUrl && !isMicrosoftBrowser()) {
                e.originalEvent.dataTransfer.setData("DownloadURL", downloadUrl);
            }
        }
    }

    function getDataTransferFromEvent(event) {
        if (event.dataTransfer) {
            return event.dataTransfer;
        }

        if (event.originalEvent && event.originalEvent.dataTransfer) {
            return event.originalEvent.dataTransfer;
        }
    }

    function dragEventContainsFiles(event) {
        var dataTransfer = getDataTransferFromEvent(event);

        if (dataTransfer && dataTransfer.types) {
            var types = dataTransfer.types;
            for (var i = 0; i < types.length; i++) {
                if (types[i] === "Files") {
                    return true;
                }
            }
            return false;
        }
        return true;
    }

    function isEventTargetFormOrWikiTextField(event) {
        return event.target.classList.contains('wiki-textfield') || event.target.localName === 'form';
    }

    return {
        getDragPosition: getDragPosition,
        createDragImage: createDragImage,
        isMicrosoftBrowser: isMicrosoftBrowser,
        setDataTransferAttachmentDownloadURL: setDataTransferAttachmentDownloadURL,
        getDataTransferFromEvent: getDataTransferFromEvent,
        dragEventContainsFiles: dragEventContainsFiles,
        isEventTargetFormOrWikiTextField: isEventTargetFormOrWikiTextField
    }
});