define('smart-attachments/toolbox', [
    "jquery",
    'jira/util/data/meta'
], function ($, AjsMeta) {
    var SERVICE_DESK_PROJECT_KEY = "service_desk";
    var projectsModule = require("module", "jira/api/projects");
    var JiraProjectsApi = {
        getCurrentProjectType: function() {
            return "none";
        }
    };

    if (projectsModule.exports) {
        JiraProjectsApi = projectsModule.exports;
    }

    function isSDProject() {
        var projectType = $("#dnd-metadata-webpanel").data("project-type") || AjsMeta.get("issue-project-type") || JiraProjectsApi.getCurrentProjectType();
        return projectType === SERVICE_DESK_PROJECT_KEY;
    }

    var getProjectKey = function(issueKey) {
        return issueKey.substr(0, issueKey.lastIndexOf("-"));
    };

    function isFVIModuleEnabled() {
        return !isFVIModuleDisabled();
    }

    function isFVIModuleDisabled() {
        return typeof FVI === 'undefined';
    }

    return {
        isSDProject: isSDProject,

        attachFilesToCategories: function (issueId, files) {
            return JIRA.SmartAjax.makeRequest({
                type: 'POST',
                url: AJS.params.baseURL + '/rest/attach-cat/1.0/attachments',
                data: {
                    issueId: issueId,
                    tempAttachments: JSON.stringify(files)
                }
            });
        },

        fetchCategories: function (issueId) {
            return JIRA.SmartAjax.makeRequest({
                type: 'GET',
                url: AJS.params.baseURL + '/rest/attach-cat/1.0/attachments/categories',
                data: {
                    pid: issueId,
                    issueType: -1,
                    isIssue: true
                }
            })
                .fail(function () {
                    JIRA.Messages.showMsg(AJS.I18n.getText("attach.cat.get.error"), {
                        type: JIRA.Messages.Types.ERROR,
                        closeable: false,
                        timeout: 3
                    });
                });
        },
        isFVIModuleEnabled: isFVIModuleEnabled,
        getProjectKey: getProjectKey,
        getProjectKeyFactory: function(issueKey) {
            return _.partial(getProjectKey, issueKey);
        }
    }
});