define('smart-attachments/toolbox', [
    "jquery",
    'jira/util/data/meta',
    'jira/util/events',
    'jira/util/events/types',
    'exports'
], function ($, AjsMeta, Events, Types, exports) {
    var SERVICE_DESK_PROJECT_KEY = "service_desk";
    var projectsModule = require("module", "jira/api/projects");
    var JiraProjectsApi = {
        getCurrentProjectType: function () {
            return "none";
        }
    };

    if (projectsModule.exports) {
        JiraProjectsApi = projectsModule.exports;
    }

    var isSDProject = function () {
        var projectType = $("#dnd-metadata-webpanel").data("project-type") || AjsMeta.get("issue-project-type") || JiraProjectsApi.getCurrentProjectType();
        return projectType === SERVICE_DESK_PROJECT_KEY;
    };

    var isAgile = function() {
        return !!document.querySelector('#gh');
    };

    var isIssueNavigator = function () {
        return !!document.getElementById('full-issue-navigator');
    };

    var isSearchForIssues = function () {
        return !!document.getElementById('jqlFieldz')
    };

    var isServiceDeskQueues = function () {
        return !!document.querySelector('#sd-page-panel.sd-queues');
    };

    exports.isSDProject = isSDProject;
    exports.isAgile = isAgile;
    exports.isIssueNavigator = isIssueNavigator;
    exports.isSearchForIssues = isSearchForIssues;
    exports.isServiceDeskQueues = isServiceDeskQueues;

    var getProjectKey = function (issueKey) {
        return issueKey.substr(0, issueKey.lastIndexOf("-"));
    };

    function isFVIModuleDisabled() {
        return typeof FVI === 'undefined';
    }

    exports.isFVIModuleEnabled = function () {
        return !isFVIModuleDisabled();
    };


    exports.attachFilesToCategories = function (issueId, files) {
        return JIRA.SmartAjax.makeRequest({
            type: 'POST',
            url: AJS.params.baseURL + '/rest/attach-cat/1.0/attachments',
            data: {
                issueId: issueId,
                tempAttachments: JSON.stringify(files)
            }
        });
    };

    exports.fetchCategories = function (issueId) {
        return JIRA.SmartAjax.makeRequest({
            type: 'GET',
            url: AJS.params.baseURL + '/rest/attach-cat/1.0/attachments/category/issue',
            data: {issueId: issueId}
        })
            .fail(function () {
                JIRA.Messages.showMsg(AJS.I18n.getText("attach.cat.get.error"), {
                    type: JIRA.Messages.Types.ERROR,
                    closeable: false,
                    timeout: 3
                });
            });
    };

    exports.getProjectKey = getProjectKey;

    exports.getProjectKeyFactory = function (issueKey) {
        return _.partial(getProjectKey, issueKey);
    };

    exports.createRevisionFromTempAttachments = function (issue, categoryId, parentAttachmentId, documentId, viewName, files, skipTemp) {
        console.log("Create revision from TempAttachments");
        var result = $.Deferred();
        //var skip = !!skipTemp;
        if (files.length === 0) {
            result.resolve({});
        } else {
            JIRA.SmartAjax.makeRequest({
                type: 'POST',
                async: false,
                url: AJS.params.baseURL + '/rest/attach-cat/1.0/attachments/version',
                data: {
                    issueKey: issue.getIssueKey(),
                    catId: categoryId,
                    parent: parentAttachmentId,
                    name: viewName,
                    skipTemp: !!skipTemp,
                    documentId: documentId,
                    attachments: files
                },
                success: function () {
                    result.resolve({});
                }
            });
        }

        return result;
    };

    exports.invalidateDataAndView = function (issue) {
        console.log('Invalidate data and view for issue: ' + issue.getIssueKey());
        Events.trigger('change:attachments', [issue]);
        if (isAgile()) {
            GH.DetailsView.reload();
        } else {
            Events.trigger(Types.REFRESH_ISSUE_PAGE, [issue.getIssueId()]);
        }
    }

});