define('smart-attachments/dialog/CreateIssueDecorator',
    [
        'jquery',
        'underscore',
        'jira/util/events',
        'jira/util/events/types',
        'smart-attachments/dialog/DialogDecorator',
        'smart-attachments/dropzone/FieldTypeDropZone',
        'smart-attachments/dropzone/CategoryDropZone'
    ], function ($, _, Events, Types, DialogDecorator, FieldTypeDropZone, CategoryDropzone) {
        var MutationConfig = {
            childList: true,
            attributes: false,
            characterData: false,
            subtree: true
        };

        var isExpectedNode = function (node) {
            return node.nodeName === 'FORM';
        };

        var CreateIssueDecorator = DialogDecorator.extend({
            init: function (opts) {
                this._super.apply(this, arguments);
                this.setTargetFields(opts.dialog.$popupContent);
                this.attachEventListeners();
                this.isDialog = this.$form.attr('id') !== 'issue-create';
                if (!this.isDialog) {
                    if (this.$form.find('.error, .aui-message error').length === 0) {
                        this.resetTempAttachmentsInLocalStorage();
                    }
                }
                this.invokeParentDecorate();
            },

            decorate: function () {

            },

            invokeParentDecorate: function () {
                DialogDecorator.prototype.decorate.call(this);
            },

            setTargetFields: function ($container) {
                this.$projectSelect = $($container.find('input[name="pid"]')[0]);
                this.$issueType = $($container.find('input[name="issuetype"]')[0]);
            },

            updateDropzone: function (dropzone, categoriesConfig) {
                var categories = _.filter(categoriesConfig.categories, function (category) {
                    return !category.hidden;
                });

                if (categories.length) {
                    var updatedCategoryDropzone = void 0;
                    if (!categoriesConfig.fieldTypeDropZone) {
                        var lastCategoryId = this.isDialog ? undefined : this.getLastCategoryId();
                        updatedCategoryDropzone = new FieldTypeDropZone(categoriesConfig.categories, this.$form, dropzone, lastCategoryId);
                        this.injectAnalyticsFieldTypeDropZone(updatedCategoryDropzone);
                    } else {
                        updatedCategoryDropzone = new CategoryDropzone(categoriesConfig.categories, this.$form, dropzone);
                        this.injectAnalyticsCategoryDropZones(updatedCategoryDropzone);
                    }
                    var categoryResolver = this.createCategoryResolverByTempAttachments(categories);
                    if (this.categoryDropzone) {
                        this.moveProgressBars(categoryResolver, this.categoryDropzone.getProgressBars(), updatedCategoryDropzone);
                    } else {
                        if (dropzone.progressBars.length) {
                            console.log('Restore categories for temp attachments');
                            this.moveProgressBars(categoryResolver, dropzone.progressBars, updatedCategoryDropzone);
                        }
                    }
                    this.categoryDropzone = updatedCategoryDropzone;
                    if (categoriesConfig.fieldTypeDropZone) {
                        this.clearDefaultDropZone();
                    }
                } else {
                    if (this.categoryDropzone) {
                        delete this.categoryDropzone;
                    }
                }
            },

            clearDefaultDropZone: function () {
                this.dropzone.$node.parent().empty();
            },

            moveProgressBars: function (categoryResolver, progressBars, targetCategoryDropzone) {
                var resetAttachmentBingings = this.resetAttachmentBinding;

                delete this.resetAttachmentBinding;
                console.log(progressBars);
                _.each(progressBars, function (progressBar) {
                    var dropZone;
                    var targetCategoryId = categoryResolver.resolve(progressBar);
                    if (resetAttachmentBingings || !targetCategoryId) {
                        dropZone = targetCategoryDropzone.getDefaultOrFirstCategoryDropzone();
                    } else {
                        dropZone = targetCategoryDropzone.getDropZoneByCategoryId(targetCategoryId);
                    }
                    progressBar.category = dropZone.category;
                    progressBar.$node.find('button').removeAttr("disabled");
                    if (dropZone.appendProgressBar) {
                        dropZone.appendProgressBar(progressBar);
                    }
                });
            },

            decorateDropzone: function (event, dropzone) {
                this.dropzone = dropzone;
                var resultHandler = _.partial(this.updateDropzone.bind(this), dropzone);
                this.getCategories(this.$projectSelect.val(), this.$issueType.val(), false)
                    .then(resultHandler);
            },

            onDialogUpdate: function (mutations) {
                console.log('Dialog content is changed');

                var subtreeMutation = _.find(mutations, function (mutation) {
                    var stop = false;
                    var size = mutation.addedNodes.length;
                    for (var i = 0; i < size && !stop; i++) {
                        var node = mutation.addedNodes[i];
                        stop = isExpectedNode(node);
                    }
                    return stop;
                });

                if (subtreeMutation) {
                    this.observer.disconnect();
                    //restore form link
                    this.$form = $(this.dialog.$popupContent.find('form')[0]);
                    this.setTargetFields(this.$form);
                    this.invokeParentDecorate();
                    this.attachEventListeners();
                }
            },

            setResetAttachmentBindings: function () {
                this.resetAttachmentBinding = true;
            },

            attachEventListeners: function () {
                var startTrackingDOM = function () {
                    var observer = new MutationObserver(this.onDialogUpdate.bind(this));
                    var target = this.dialog.$popupContent[0];
                    this.observer = observer;
                    observer.observe(target, MutationConfig);
                };

                var changesHandler = startTrackingDOM.bind(this);

                this.$projectSelect.on('change', this.setResetAttachmentBindings.bind(this));
                this.$projectSelect.on('change', changesHandler);
                this.$issueType.on('change', changesHandler);
                this.$form.off('before-submit.smart');
                this.$form.on('before-submit.smart', this.onBeforeSubmitHandler.bind(this));
            },

            onBeforeSubmitHandler: function () {
                if (!this.isDialog) {
                    this.storeTempAttachmentsInLocalStorage(this.categoryDropzone);
                }
                if (this.categoryDropzone) {
                    var files = this.categoryDropzone.getTempAttachments();
                    if (files.length) {
                        var projectId = this.$projectSelect.val();
                        var issueTypeId = this.$issueType.val();
                        var summary = this.$form.find('#summary').val();
                        this.saveAttachmentsCache(projectId, issueTypeId, summary, files);
                        SmartAttachmentsAnalytics.event('attachmentUpload', [files]);
                    }
                }
            },

            saveAttachmentsCache: function (pid, issueTypeId, summary, files) {
                JIRA.SmartAjax.makeRequest({
                    type: 'POST',
                    async: false,
                    url: AJS.params.baseURL + '/rest/attach-cat/1.0/attachments/cache',
                    data: {
                        pid: pid,
                        issueTypeId: issueTypeId,
                        summary: summary,
                        tempAttachments: JSON.stringify(files)
                    }
                });
            },

            injectAnalyticsFieldTypeDropZone: function (fieldTypeDropZone) {
                fieldTypeDropZone.dropzone.queueEvent = _.wrap(fieldTypeDropZone.dropzone.queueEvent, this.queueEvent.bind(this));
            },

            injectAnalyticsCategoryDropZones: function (categoryDropZones) {
                categoryDropZones.customDropZones.forEach(function (customDropZone) {
                    customDropZone.dropZone.queueEvent = _.wrap(
                        customDropZone.dropZone.queueEvent,
                        this.queueEvent.bind(customDropZone.dropZone)
                    );
                }.bind(this));
            },

            queueEvent: function (queueEventOriginal) {
                var args = Array.prototype.slice.call(arguments, 1);
                queueEventOriginal.apply(this, args);

                var eventName = args[0];
                var attachmentsCount = args[1].count;
                var categoryId = this.category ? this.category.id : this.categoryDropzone.categorySelect[0].value;
                var props = [categoryId, attachmentsCount];

                if (eventName === 'fileInput') {
                    SmartAttachmentsAnalytics.event('createIssueFormAttachmentBrowseUpload', props);
                }
                if (eventName === 'fileDrop') {
                    SmartAttachmentsAnalytics.event('createIssueFormDnDAttachment', props);
                }
            }

        });

        return CreateIssueDecorator;
    });