define('smart-attachments/dialog/DialogDecorator',
    [
        'jquery',
        'underscore',
        'jira/lib/class',
        'smart-attachments/base/Operations'
    ],
    function ($, _, Class, Operations) {

        var CategoryResolver = function (tempAttachments, availableCategories) {
            function resolve(progressBar) {
                var categoryId = void 0;
                if (progressBar.category) {
                    categoryId = progressBar.category.id;
                } else {
                    var value = _.find(tempAttachments, function (tempAttachment) {
                        return tempAttachment.fileId === progressBar.getFileID();
                    });
                    if (value) {
                        categoryId = parseInt(value.categoryId);
                    }
                }

                if (categoryId) {
                    var isAvailable = _.some(availableCategories, function (category) {
                        return category.id === categoryId;
                    });
                    if (isAvailable) {
                        return categoryId;
                    }
                }
                return void 0;
            }

            return {
                resolve: resolve
            }
        };

        var DialogDecorator = Class.extend({
            init: function (opts) {
                _.extend(this, opts);
                this.getCategories = Operations.getCategories;
                this.$form = $(this.dialog.$popupContent.find('form')[0]);
                $(this.dialog).prop('_decorator', this);
                if (this.isAgile) {
                    if (typeof this.dialog.handleSuccess === 'undefined') {
                        this.dialog.handleSuccess = function () {
                            console.log('ok');
                        }
                    }
                }
                this.decorate();
            },

            decorate: function () {
                if (this.$form.prop('_dropzone')) {
                    this.decorateDropzone(null, this.$form.prop('_dropzone'));
                } else {
                    this.$form.one('dropzone:connected', this.decorateDropzone.bind(this));
                }
            },

            decorateDropzone: function (event, dropzone) {
                //override it;
            },

            saveCategorizedAttachments: function (issue, files) {
                return Operations.saveCategorizedAttachments(issue.getIssueId(), issue.getIssueKey(), files);
            },

            onBeforeSubmitHandler: function () {
                //override it;
            },

            attachToSubmit: function () {
                this.$form.unbind('before-submit').bind('before-submit', this.onBeforeSubmitHandler.bind(this));
            },

            storeTempAttachmentsInLocalStorage: function (categoryDropzone) {
                if (typeof (Storage) === "undefined") {
                    return
                }

                var tempAttachments = categoryDropzone.getJiraTempAttachments();
                if (tempAttachments.length) {
                    localStorage.setItem("sa-temp-files", JSON.stringify(tempAttachments));
                    localStorage.setItem("sa-temp-files-create-date", Date.now());
                }
            },

            getTempAttachmentsFromLocalStorage: function () {
                if (this.isDialog) {
                    return [];
                }

                if (typeof (Storage) === "undefined") {
                    return [];
                }

                var value = localStorage.getItem("sa-temp-files");
                if (!value) {
                    return []
                }

                var tempAttachments = JSON.parse(value);
                console.log(tempAttachments);
                return tempAttachments;
            },

            resetTempAttachmentsInLocalStorage: function () {
                if (typeof (Storage) === "undefined") {
                    return [];
                }

                localStorage.removeItem("sa-temp-files");
                localStorage.removeItem("sa-temp-files-create-date");
            },

            getLastCategoryId: function () {
                var tempAttachments = this.getTempAttachmentsFromLocalStorage();
                if (tempAttachments.length) {
                    return tempAttachments[0].categoryId;
                }
            },

            createCategoryResolverByTempAttachments: function (availableCategories) {
                var tempAttachments = this.getTempAttachmentsFromLocalStorage();
                return new CategoryResolver(tempAttachments, availableCategories);
            }
        });
        return DialogDecorator;
    });