define('smart-attachments/dialog/service-desc/AttachFileDecorator',
    [
        'jquery',
        'underscore',
        'jira/lib/class'

    ], function ($, _, Class) {
        var SDAttachFileDecorator = Class.extend({
            init: function (opts) {
                _.extend(this, opts);
                this.decorate();
            },

            changeDialogTitle: function (category) {
                var title = this.dialog.content.find('h2');
                var text = "Attach files to category: " + category.name;
                title.attr('title', text);
                title.text(text);
            },

            decorate: function () {
                console.log('decorate dropzone:' + this.dialog.dropzone);
                if (this.uploadInfo) {
                    var category = this.uploadInfo.category;
                    this.changeDialogTitle(category);
                }
            }
        });

        return SDAttachFileDecorator;
    });


define('smart-attachments/dialog/AttachFileDecorator',
    [
        'jquery',
        'underscore',
        'jira/util/events',
        'jira/util/events/types',
        'smart-attachments/base/CommonClasses',
        'smart-attachments/dialog/DialogDecorator'
    ], function ($, _, Events, Types, CommonClasses, DialogDecorator) {
        var AttachFileDecorator = DialogDecorator.extend({
            init: function () {
                this._super.apply(this, arguments);
                this.attachEventListeners();
                this.$form.prop('_decoratedDropZone', this);
            },

            decorateDropzone: function (event, dropzone) {
                console.log('decorate dropzone:' + dropzone);
                if (this.uploadInfo) {
                    var category = this.uploadInfo.category;
                    this.changeDialogTitle(category);
                }
            },

            attachEventListeners: function () {
                $(this.dialog).one('Dialog.hide', function (event, $popup, reason, dialogId) {
                    console.log('dialog hide ' + dialogId + ": " + reason);
                    if (!reason) {
                        this.trigger('submit:success', this);
                    }
                }.bind(this));
                this.attachToSubmit();
            },

            changeDialogTitle: function (category) {
                var title = this.dialog.$popupHeading.find('h2');
                var text = "Attach files to category: " + category.name;
                title.attr('title', text);
                title.text(text);
            },

            getTempAttachments: function () {
                var categoryId = this.uploadInfo.category.id;
                var dropzone = this.$form.prop('_dropzone');
                if (dropzone) {
                    return dropzone.progressBars.map(function (progressBar) {
                        return new CommonClasses.TempAttachment(progressBar.getFileName(), categoryId);
                    });
                } else {
                    return [];
                }
            },

            onBeforeSubmitHandler: function () {
                SmartAttachAnalytics.eventAttachForm();
                var files = this.getTempAttachments();
                if (files.length) {
                    this.saveCategorizedAttachments(this.uploadInfo.issue, files)
                }
            }
        });

        return AttachFileDecorator;
    });

define('smart-attachments/dialog/AttachFilesDecorator',
    [
        'jquery', 'underscore',
        'jira/util/events',
        'jira/util/events/types',
        'smart-attachments/dialog/DialogDecorator',
        'smart-attachments/dropzone/FieldTypeDropZone',
        'smart-attachments/dropzone/CategoryDropZone'
    ], function ($, _, Events, Types, DialogDecorator, FieldTypeDropZone, CategoryDropzone) {
        function notHidden(category) {
            return !category.hidden;
        }

        var AttachFilesDecorator = DialogDecorator.extend({
            init: function (options) {
                var dialogId = options.dialog.options.id;
                this.isTransitionForm = dialogId && dialogId.indexOf('transition') !== -1;
                this._super.apply(this, arguments);
            },

            decorateDropzone: function (event, dropzone) {
                console.log('decorate dropzone:' + dropzone);
                dropzone.$node.hide();
                var resultHandler = _.partial(this.updateDropzone.bind(this), dropzone);
                var issueId = this.uploadInfo.issue.getIssueId();
                this.getCategories(issueId, false, true).then(resultHandler);
            },

            updateDropzone: function (dropzone, categoriesConfig) {
                var categories = _.filter(categoriesConfig.categories, notHidden);
                if (categories.length) {
                    var updatedCategoryDropzone;
                    if (!categoriesConfig.fieldTypeDropZone) {
                        var lastCategoryId = this.getLastCategoryId();
                        updatedCategoryDropzone = new FieldTypeDropZone(categoriesConfig.categories, this.$form, dropzone, lastCategoryId);
                    } else {
                        updatedCategoryDropzone = new CategoryDropzone(categoriesConfig.categories, this.$form, dropzone);
                    }
                    if (dropzone.progressBars.length) {
                        console.log('Restore categories for temp attachments');
                        var categoryResolver = this.createCategoryResolverByTempAttachments(categories);
                        this.moveProgressBars(categoryResolver, dropzone.progressBars, updatedCategoryDropzone);
                        this.resetTempAttachmentsInLocalStorage();
                    }
                    this.categoryDropzone = updatedCategoryDropzone;
                    this.attachEventListeners();
                } else {
                    dropzone.$node.show();
                }
            },

            moveProgressBars: function (categoryResolver, progressBars, targetCategoryDropzone) {
                console.log(progressBars);
                _.each(progressBars, function (progressBar) {
                    var dropZone;
                    var targetCategoryId = categoryResolver.resolve(progressBar);
                    if (!targetCategoryId) {
                        dropZone = targetCategoryDropzone.getDefaultOrFirstCategoryDropzone();
                    } else {
                        dropZone = targetCategoryDropzone.getDropZoneByCategoryId(targetCategoryId);
                    }
                    progressBar.category = dropZone.category;
                    progressBar.$node.find('button').removeAttr("disabled");
                    if (dropZone.appendProgressBar) {
                        dropZone.appendProgressBar(progressBar);
                    }
                });
            },

            attachEventListeners: function () {
                $(this.dialog).one('Dialog.hide', function (event, $popup, reason, dialogId) {
                    console.log('dialog hide ' + dialogId + ": " + reason);
                    if (!reason) {
                        this.trigger('submit:success', this);
                    }
                }.bind(this));
                this.attachToSubmit();
            },

            changeDialogTitle: function (category) {
                var title = this.dialog.$popupHeading.find('h2');
                var text = "Attach files to category: " + category.name;
                title.attr('title', text);
                title.text(text);
            },

            onBeforeSubmitHandler: function () {
                if (this.isTransitionForm) {
                    this.storeTempAttachmentsInLocalStorage(this.categoryDropzone);
                }
                SmartAttachAnalytics.eventAttachForm();
                var files = this.categoryDropzone.getTempAttachments();
                if (files.length) {
                    this.saveCategorizedAttachments(this.uploadInfo.issue, files)
                }
            }
        });

        return AttachFilesDecorator;
    });


define('smart-attachments/dialog/VersionDecorator',
    [
        'jquery',
        'underscore',
        'jira/util/events',
        'jira/util/events/types',
        'smart-attachments/dialog/DialogDecorator'
    ], function ($, _, Events, Types, DialogDecorator) {

        var VersionDecorator = DialogDecorator.extend({
            dialogTitle: AJS.I18n.getText("attach.cat.load.version"),

            init: function (opts) {
                this._super.apply(this, arguments);
            },

            decorateDropzone: function (event, dropzone) {
                this.changeDialogTitle();
                this.createInputDocumentName();
                var attachmentInfos = this.versioning.attachmentInfos;
                if (attachmentInfos.length) {
                    this.$attachmentSelectContainer = this.createDropdownWithAttachments(attachmentInfos);
                }
                this.bindAttachmentDropZoneChange(dropzone);
                this.attachToSubmit();
            },

            createInputDocumentName: function () {
                var info = this.versioning.info;
                if (!info.isDoc) {
                    this.dialog.$popupContent.find('.form-body').prepend(AttachCatDialog.versionInput({name: info.name}));
                    this.$fileNameInput = this.dialog.$popupContent.find('#attach-cat-version');
                }
            },

            changeDialogTitle: function () {
                var title = this.dialog.$popupHeading.find('h2');
                title.attr('title', this.dialogTitle);
                title.text(this.dialogTitle);
            },

            createDropdownWithAttachments: function (infos) {
                var $attachmentSelectContainer = $(AttachCatDialog.attachSelect({attachments: infos}));
                var $fileInputList = $(this.dialog.$popupContent.find('fieldset:has(*.file-input-list)')[0]);
                $fileInputList.after($attachmentSelectContainer);
                var ctx = this;
                $attachmentSelectContainer.find('select').change(function (event) {
                    var attachId = $(event.target).val();
                    if (attachId > 0) {
                        ctx.removeAttachments($fileInputList);
                        $fileInputList.hide();
                    } else {
                        $fileInputList.show();
                    }
                });

                this.bindAttachmentDropZoneChange($attachmentSelectContainer, $fileInputList.find('.file-input-list'));

                return $attachmentSelectContainer;
            },

            bindAttachmentDropZoneChange: function (dropzone) {
                var ctx = this;
                dropzone.bind('progressBarStarted', function () {
                    console.log('on progress bar add');
                    ctx.hideAttachDropdown();
                });
                dropzone.bind('progressBarDestroyed', function () {
                    console.log('on progress bar removed');
                    if (dropzone.progressBars.length === 0) {
                        ctx.showAttachDropdown();
                    }
                });
            },

            onBeforeSubmitHandler: function () {
                SmartAttachAnalytics.eventAttachForm();
                this.uploadNewRevisionInfo();
            },

            getFilesFromJIRADropZone: function ($form) {
                //todo использовать progressBars из dropzone
                var files = [];
                $.each($form.find('input.checkbox[name="filetoconvert"]:checked'), function (i, input) {
                    files.push($(input).parent().find('label').text().trim());
                });
                $.each($form.find('.upload-progress-bar__file-name'), function (i, span) {
                    files.push($(span).text().trim());
                });
                return files;
            },

            uploadNewRevisionInfo: function () {
                var versioning = this.versioning.info;
                if (!versioning.isDoc) {
                    versioning.name = this.$fileNameInput.val().trim() || versioning.name;
                }
                var result;
                if (this.isAttachDropdownSelected()) {
                    var attachmentId = this.getAttachDropdownValue();
                    var commentContent = this.getDialogCommentContent();
                    result = this.postCreateVersionByAttachId(attachmentId, commentContent, versioning);
                } else {
                    var files = this.getFilesFromJIRADropZone(this.$form);
                    result = this.postCreateVersionByAttachNames(files, versioning);
                }

                result.then(function () {
                    this.dialog.hide();
                    this.trigger('submit:success', this);
                }.bind(this))
            },

            postCreateVersionByAttachNames: function (files, versioning) {
                var result = $.Deferred();
                if (files.length === 0) {
                    result.resolve({});
                } else {
                    JIRA.SmartAjax.makeRequest({
                        type: 'POST',
                        async: false,
                        url: AJS.params.baseURL + '/rest/attach-cat/1.0/attachments/version',
                        data: {
                            issueKey: this.issue.getIssueKey(),
                            catId: versioning.catId,
                            parent: versioning.parent,
                            name: versioning.name,
                            documentId: versioning.documentId,
                            attachments: files
                        },
                        success: function () {
                            result.resolve({});
                        }
                    });
                }

                return result;
            },

            postCreateVersionByAttachId: function (attachId, commentContent, versioning) {
                var result = $.Deferred();
                JIRA.SmartAjax.makeRequest({
                    type: 'POST',
                    async: false,
                    url: AJS.params.baseURL + '/rest/attach-cat/1.0/attachments/revision-comment',
                    data: {
                        attachmentId: attachId,
                        commentContent: commentContent,
                        catId: versioning.catId,
                        parent: versioning.parent,
                        documentName: versioning.name,
                        documentId: versioning.documentId
                    },
                    success: function () {
                        result.resolve({});
                    }
                });
                return result;
            },

            removeAttachments: function (fieldset) {
                this.getAttachmentsFromDropZone(fieldset).remove();
            },

            getAttachmentsFromDropZone: function ($fileInputList) {
                return $fileInputList.find('input[name="filetoconvert"]:checked');
            },

            hideAttachDropdown: function () {
                if (this.$attachmentSelectContainer) {
                    this.$attachmentSelectContainer.find('select').val('-1');
                    this.$attachmentSelectContainer.fadeOut('fast');
                }
            },

            showAttachDropdown: function () {
                if (this.$attachmentSelectContainer) {
                    this.$attachmentSelectContainer.show();
                }
            },

            isAttachDropdownSelected: function () {
                var selectedValue = this.getAttachDropdownValue();
                return selectedValue !== undefined && parseInt(selectedValue) !== -1;
            },

            getAttachDropdownValue: function () {
                if (this.$attachmentSelectContainer) {
                    return this.$attachmentSelectContainer.find('select').val();
                }
                return void 0;
            },

            getDialogCommentContent: function () {
                return this.dialog.$popupContent.find('#comment').val();
            }
        });

        return VersionDecorator;
    });

define('smart-attachments/dialog/CtrlvDialogDecorator', [
    'jquery',
    'underscore',
    'jira/util/events',
    'jira/util/events/types',
    'smart-attachments/base/CommonClasses',
    'smart-attachments/base/AutoDistribution',
    'smart-attachments/dialog/DialogDecorator'

], function ($, _, Events, Types, CommonClasses, AutoDistribution, DialogDecorator) {
    var CtrlVDecorator = DialogDecorator.extend({
        decorate: function () {
            console.log("Decorate: " + this.dialog.options.id);
            this.$fileNameInput = this.$form.find("#attachscreenshotname");
            this.selectedCategoryId = 0;
            this.getCategories(this.uploadInfo.issue.getIssueId(), false, true)
                .then(this.finishDecorate.bind(this))
        },

        getScreenshotNameFromForm: function () {
            var screenshotName = this.$fileNameInput;
            return screenshotName.length ? screenshotName.val() + ".png" : "";
        },

        onCategorySelect: function (event) {
            event.stopPropagation();
            this.selectedCategoryId = parseInt(event.target.value);
        },

        createCategroiesDropdownList: function (categories, $form, $target) {
            var screenshotName = this.getScreenshotNameFromForm($form);
            var categoryId = AutoDistribution.matchByRuleCategoryId(categories, {name: screenshotName});
            this.selectedCategoryId = categoryId || 0;
            var $selectBlock = $(AttachCatDialog.catSelect({
                categories: categories,
                defaultId: this.selectedCategoryId
            }));
            $selectBlock.find('select').on('change', this.onCategorySelect.bind(this));
            $target.before($selectBlock);
        },

        finishDecorate: function (categoriesConfig) {
            var categories = _.filter(categoriesConfig.categories, function (category) {
                return !category.hidden;
            });
            if (categories.length) {
                var $target = $(this.$form.find('#attach-screenshot-filename-group')[0]);
                this.createCategroiesDropdownList(categories, this.$form, $target);
                this.attachEventListeners();
            }
        },

        getTempAttachments: function () {
            var screenshotName = this.$fileNameInput;
            var result = [];
            if (screenshotName.length) {
                var fullScreenshotName = screenshotName.val() + ".png";
                result.push(new CommonClasses.TempAttachment(fullScreenshotName, this.selectedCategoryId));
            }
            return result;
        },

        onBeforeSubmitHandler: function () {
            var files = this.getTempAttachments();
            if (files.length) {
                this.saveCategorizedAttachments(this.uploadInfo.issue, files)
            }
        },

        attachEventListeners: function () {
            $(this.dialog).one('Dialog.hide', function (event, $popup, reason, dialogId) {
                console.log('dialog hide ' + dialogId + ": " + reason);
                if (!reason) {
                    this.trigger('submit:success', this);
                }
            }.bind(this));
            this.attachToSubmit();
        }
    });
    return CtrlVDecorator;
});

