(function ($) {

    window.CategoryFormDropZone = {

        uploadLimit: 0,
        uploadSize: "",
        isTmpMoves: false,

        init: function (allCategories, $formFieldGroup) {
            var categories = getPermittedCategories(allCategories);

            if (!categories.length){
                return;
            }

            require(CategoryFormDropZone.getModulesPath(), function (FormDropZone, Events, DataTransfer) {
                var initializedDropZone = $formFieldGroup.find('.issue-drop-zone');
                if (initializedDropZone.length > 1 ) {
                    initializedDropZone = $(initializedDropZone.get(0));
                }
                if (initializedDropZone.length !== 1) {
                    AttachCatDialogHandler.initDropdownCategories($formFieldGroup.parents('form'), $formFieldGroup);
                    return;
                }

                var jiraAttachContainer = initializedDropZone.parent();

                CategoryFormDropZone.createDragoverMask(categories, $formFieldGroup);
                CategoryDropZoneUtils.override(FormDropZone, Events, DataTransfer);

                CategoryFormDropZone.getUploadConfig(initializedDropZone);
                var dropZoneTemplate = CategoryFormDropZone.getDropZoneTemplate();

                CategoryFormDropZone.unbindDefDropZoneDropEvents(jiraAttachContainer);
                jiraAttachContainer.hide();
                var ourContainer = CategoryFormDropZone.createNewContainer(jiraAttachContainer);

                var customDropZones = [];
                $.each(categories, function (i, category) {
                    var attachDropZone = $(dropZoneTemplate);
                    attachDropZone.attr('data-category-id', category.id).attr('data-category-name', category.name);
                    ourContainer.append(attachDropZone);
                    var newSection = ourContainer.find('[data-category-id=' + category.id + "]");
                    var dropZone = new FormDropZone(newSection);
                    CategoryFormDropZone.customizeDropZone(ourContainer);

                    customDropZones.push(new AttachCatClasses.CustomDropZone(dropZone, newSection, category.id));
                });

                CategoryFormDropZone.customDropZones = customDropZones;
                CategoryFormDropZone.insertTempFiles(jiraAttachContainer);

                CategoryFormDropZone.fixDropZonesTarget();
                CategoryFormDropZone.bindDropProgressBarIntoDropZone($formFieldGroup);
                CategoryFormDropZone.fixIESlowInitialisation(ourContainer);
            });
        },

        getModulesPath: function () {
            return ['dndattachment/dropzones/FormDropZone', 'jira/util/events', 'dndattachment/util/DataTransfer'];
        },

        createDragoverMask: function (categories, $formFieldGroup) {
            var masks = $formFieldGroup.parents('form').parent();
            masks.find('.custom-dragover-mask').remove();
            var mask = $(AttachCatDialog.dragoverMask());
            masks.append(mask);
            mask = masks.find('.custom-dragover-mask');
            mask.append(AttachCatDialog.container());
            var catContainer = mask.find('#categories-container');

            AttachDropSections.createSections(mask, catContainer, categories, AttachCatDialog.categoryInDnD);
        },

        getUploadConfig: function (initializedDropZone) {
            CategoryFormDropZone.uploadLimit = initializedDropZone.data('uploadLimit');
            CategoryFormDropZone.uploadSize = initializedDropZone.data('uploadSize');
        },

        getDropZoneTemplate: function () {
            return AttachCatDialog.attachmentDropZone({
                uploadLimit: CategoryFormDropZone.uploadLimit,
                uploadSize: CategoryFormDropZone.uploadSize
            });
        },

        createNewContainer: function (container) {
            var classes = container.attr('class');
            var newContainer = $(AttachCatDialog.fieldGroupAttachmentDropZone({classes: classes}));
            newContainer.insertAfter(container);
            return newContainer;
        },

        customizeDropZone: function ($container) {
            var dropZone = this.getNotCustomizedDropZone($container);
            dropZone.addClass('customized-drop-zone');
            var categoryTitle = dropZone.data('categoryName');
            var template = AttachCatDialog.dropZoneCategoryTitle({categoryTitle: categoryTitle});
            dropZone.prepend(template);
        },

        getNotCustomizedDropZone: function ($container) {
            return $container.find('.issue-drop-zone').not('.customized-drop-zone');
        },

        insertTempFiles: function (jiraAttachContainer) {
            var customDropZones = CategoryFormDropZone.customDropZones;
            if (!customDropZones.length) {
                return
            }

            var realFiles = CategoryFormDropZone.files;
            var storedTmpFiles = CategoryFormDropZone.jiraTempFiles;

            if (storedTmpFiles.length && realFiles.length && storedTmpFiles.length === realFiles.length) {
                $.each(customDropZones, function (i, customDropZone) {
                    var tmpFileIds = [];
                    $.each(storedTmpFiles, function (j, storedTempFile) {
                       if (storedTempFile.categoryId === customDropZone.categoryId) {
                           tmpFileIds.push(storedTempFile.fileId);
                       }
                    });

                    var files =  [];
                    $.each(realFiles, function (j, realFile) {
                        if (tmpFileIds.indexOf(realFile.id) > -1) {
                            files.push(realFile);
                        }
                    });

                    var dropZoneClass = customDropZone.dropZone;
                    if (typeof dropZoneClass.loadAttachments === "function") {
                        dropZoneClass.loadAttachments(files);
                    }

                    if (typeof dropZoneClass.loadFiles === "function") {
                        dropZoneClass.loadFiles(files);
                    }
                });
                CategoryFormDropZone.files = [];
                CategoryFormDropZone.jiraTempFiles = [];
            } else {
                catchTempFilesByMutationObserver(customDropZones[0].section, jiraAttachContainer);
                //In case, when some files are already placed in the default JIRA drop zone. They will be caught by the mutation observer.
                moveMissedAttachmentsToDropZone(customDropZones[0].section, jiraAttachContainer);
            }
        },

        fixDropZonesTarget: function () {
            setTimeout(function () {
                $('.field-group .issue-drop-zone').each(function (i, dropZone) {
                    $(dropZone).find('.issue-drop-zone__target').each(function (i, target) {
                        if (i > 0) {
                            $(target).remove();
                        }
                    })
                })
            }, 500);
        },

        removeCurrentDropZones: function ($context) {
            this.getCustomizedDropZoneContainer($context).remove();
        },

        getCustomizedDropZoneContainer: function ($context) {
            return $context.find('.customized-drop-zone').parents('.file-input-list')
        },

        getTempAttachmentsWithCategories: function ($form) {
            var tempAttachments = [];

            var dropdownWithCategories = $form.find('#attach-cat-select');
            if (dropdownWithCategories.length > 0) {
                var categoryId = dropdownWithCategories.val();
                $.each(AttachCatDialogHandler.getFilesFromJIRADropZone($form), function (i, file) {
                    tempAttachments.push(new AttachCatClasses.TempAttachment(file, categoryId));
                });
                return tempAttachments;
            }

            $form.find('.field-group .issue-drop-zone[data-category-id]').each(function (i, dropZone) {
                var $dropZone = $(dropZone);
                var categoryId = $dropZone.data('categoryId');
                $.each($dropZone.find('input.checkbox[name="filetoconvert"]:checked'), function(i, input) {
                    var attachName = $(input).parent().find('label').html().trim();
                    var tempAttachment = new AttachCatClasses.TempAttachment(attachName, categoryId);
                    tempAttachments.push(tempAttachment);
                });

                $.each($dropZone.find('.upload-progress-bar__file-name'), function(i, span) {
                    var attachName = $(span).text().trim();
                    var tempAttachment = new AttachCatClasses.TempAttachment(attachName, categoryId);
                    tempAttachments.push(tempAttachment);
                });
            });
            return tempAttachments;
        },

        getJiraTempAttachmentsWithCategories: function ($form) {
            var tempAttachments = [];
            $form.find('.field-group .issue-drop-zone[data-category-id]').each(function (i, dropZone) {
                var $dropZone = $(dropZone);
                var categoryId = $dropZone.data('categoryId');
                var fileInputs = $dropZone.find('input[name="filetoconvert"]:checked');

                $.each(fileInputs, function(i, input) {
                    var fileId = $(input).val();
                    var tempAttachment = new AttachCatClasses.JiraTempAttachment(fileId, categoryId);
                    tempAttachments.push(tempAttachment);
                });
            });
            return tempAttachments;
        },

        bindDropProgressBarIntoDropZone: function ($context) {
            var container = $context.parents('form').parent();
            var masks = container.find('.customized-drop-zone, .customized-drop-zone >*');
            masks.off('drop.sa-move').on('drop.sa-move', function (e) {
                var fileId = e.originalEvent.dataTransfer.getData('text');
                var targetContainer = $(this).closest('.issue-drop-zone');
                if (!fileId || targetContainer.find('input[value="' + fileId + '"]').length > 0) {
                    return;
                }
                targetContainer.find('input[value="' + fileId + '"]');
                var fileInputList = targetContainer.parent();
                var transferObjects = fileInputList.find('input[value="' + fileId + '"], .upload-progress-bar[data-file-id="' + fileId + '"]');
                targetContainer.append(transferObjects);
            });

            container.off('dragstart.sa-move').on('dragstart.sa-move', '.upload-progress-bar', function(e){
                e.originalEvent.dataTransfer.setData('text', $(this).data('fileId'));
            });
        },

        fixIESlowInitialisation : function ($context) {
            setTimeout(function () {
                var customDZs = $context.find('.customized-drop-zone');
                if (customDZs.length === 0) {
                    return;
                }

                var firstCustomDZ = $(customDZs.get(0));
                var doubleItems = {
                    zoneText: firstCustomDZ.find('.issue-drop-zone__text'),
                    zoneIcon: firstCustomDZ.find('.issue-drop-zone__drop-icon'),
                    zoneButton: firstCustomDZ.find('.issue-drop-zone__button'),
                    zoneInputFile: firstCustomDZ.find('input.issue-drop-zone__file')
                };

                $.each(doubleItems, function (propertyName, value) {
                    if (value.length > 1) {
                        $(value.get(1)).remove();
                    }
                });

            }, 2000);
        },

        storeTempAttachmentsInBrowserCache: function ($form) {
            if (typeof(Storage) === "undefined") {
                return
            }

            var tempAttachments = CategoryFormDropZone.getJiraTempAttachmentsWithCategories($form);
            if (!tempAttachments.length) {
                return;
            }

            localStorage.setItem("sa-temp-files", JSON.stringify(tempAttachments));
            localStorage.setItem("sa-temp-files-create-date", Date.now());
        },

        retrieveTempAttachmentsFromBrowserCache: function () {
            if (typeof(Storage) === "undefined") {
                return
            }
            var jsonFiles = localStorage.getItem("sa-temp-files");
            var createTime = localStorage.getItem("sa-temp-files-create-date");
            if (jsonFiles && createTime) {
                var oneMinute = 60 * 1000;
                var period = Date.now() - createTime;
                if (period < oneMinute){
                    return JSON.parse(jsonFiles);
                }
            }
            return [];
        },

        toPlaceInNeededSection: function(fieldSet, storedTempFiles, fileId, tempAttachment){
            if (storedTempFiles && storedTempFiles.length) {
                var tempFile = $.grep(storedTempFiles, function (file) {return file.fileId === fileId;});
                if (tempFile) {
                    var targetSection = fieldSet.find('[data-category-id=' + tempFile[0].categoryId + "]");
                    targetSection.append(tempAttachment);
                    return true;
                }
            }
            return false;
        },

        unbindDefDropZoneDropEvents: function (containerDefDropZone) {
            setTimeout(function () {
                var $issueDropZones = $(containerDefDropZone).find('.issue-drop-zone');
                var $targetDrop = $issueDropZones.find('>*').andSelf();
                $targetDrop.off('dragover dragenter drop');
            }, 2000);
        }
    };

    window.CategoryDropZoneUtils = {

        override: function (FormDropZone, Events, DataTransfer) {
            this.placeUploadProgressBar(FormDropZone);
            this.connectContainer(FormDropZone, Events, DataTransfer);
        },

        placeUploadProgressBar: function (FormDropZone) {
            FormDropZone.prototype.placeUploadProgressBar = function ($progressBar) {
                $progressBar.attr('draggable', true);
                var file = $progressBar.data('file');

                CategoryDropZoneUtils.addAttachmentNameToTitleForProgressBar($progressBar);

                if (file && file.isDefCategory) {
                    var $defCategoryDropZone = CategoryDropZoneUtils.getDefCategoryDropZone(this.$fieldNode);
                    if ($defCategoryDropZone.length) {
                        return $defCategoryDropZone.append($progressBar);
                    }
                }
                return this.$node.append($progressBar);
            }
        },

        connectContainer: function (FormDropZone, Events, DataTransfer) {
            FormDropZone.prototype.connectContainer = function () {
                var categoryId = this.$node.data('categoryId');
                if (categoryId === undefined) {
                    return;
                }
                this.$containerNode = this.$node.parents('form').parent();
                this.$dragoverMask = this.$containerNode.find('.custom-dragover-mask li[data-category-id=' + categoryId + ']');
                this.$body = $(document.body);


                this.onDragOverDropHandler = this.onDragOverDrop.bind(this);
                this.onDragLeaveHandler = this.onDragLeave.bind(this);
                this.onBodyDragOverDropHandler = this.onBodyDragOverDrop.bind(this);
                this.$containerNode.on('dragover drop', this.onDragOverDropHandler);
                this.$containerNode.on('dragleave', this.onDragLeaveHandler);
                this.$body.on('dragover drop', this.onBodyDragOverDropHandler);

                this.onDragOverMaskItemHandler = this.onDragOverMaskItem.bind(this);
                this.$dragoverMask.on('drop', this.onDragOverDropHandler);
                this.$dragoverMask.on('dragover', this.onDragOverMaskItemHandler);

                this.onBeforeSubmitHandler = this.onBeforeSubmit.bind(this);
                this.$formNode.on('before-submit', this.onBeforeSubmitHandler);

                this.onContentAddedHandler = this.onContentAdded.bind(this);
                JIRA.bind(JIRA.Events.NEW_CONTENT_ADDED, this.onContentAddedHandler);
                Events.bind("Dialog.hide", this.onDialogHide.bind(this));
                if (AJS.dialog2) {
                    AJS.dialog2.on("hide", this.onDialogHide.bind(this));
                }
                this.bind('uploadFinished', CategoryDropZoneUtils.onDropZoneAttachmentUploadFinished);
                this.bind('progressBarDestroyed', CategoryDropZoneUtils.onDestroyProgressBar);
            };

            FormDropZone.prototype.onDragOverDrop = function (event) {
                if (!this.isAttached()) {
                    this.disconnectContainer();
                    return;
                }
                if (!dragEventContainsFiles(event)) {
                    return;
                }

                event.preventDefault();
                event.stopPropagation();
                event.stopImmediatePropagation();
                if (event.originalEvent) {
                    event.originalEvent.dataTransfer.dropEffect = "copy";
                }
                this.$containerNode.addClass('attach-files-drop-zone__dragover');

                if (event.type === 'drop') {
                    this.$containerNode.removeClass('attach-files-drop-zone__dragover');
                    this.$containerNode.find('.cat-cell-container-hover').removeClass('.cat-cell-container-hover');
                    $(document).trigger("dropHandled");

                    var eventDataTransfer = getDataTransferFromEvent(event);
                    if (!eventDataTransfer) {
                        return;
                    }

                    var dataTransfer = new DataTransfer(eventDataTransfer);
                    dataTransfer.getFiles().then(function (files) {
                        files = CategoryDropZoneUtils.markFilesToDefCategory(files, event);
                        this.handleFilesReceived(files);
                        this.queueEvent('fileDrop', {count: files.length});
                    }.bind(this));
                }
            };

            FormDropZone.prototype.onDragOverMaskItem = function (event) {
                CategoryDropZoneUtils.highlightDragoveredDropZone(this.$containerNode, $(event.delegateTarget));
            };
        },

        highlightDragoveredDropZone: function ($containerNode, $eventTarget) {
            $containerNode.find('.cat-cell-container-hover').removeClass('cat-cell-container-hover');
            $eventTarget.find('.cat-cell-container').addClass('cat-cell-container-hover');
        },

        isTargetTextarea: function (event) {
            return event.target.localName === 'textarea';
        },

        markFilesToDefCategory: function (files, event) {
            if (CategoryDropZoneUtils.isTargetTextarea(event)) {
                $.each(files, function (index, file) {
                    file["isDefCategory"] = true;
                })
            }
            return files;
        },

        getDefCategoryDropZone: function ($fieldNode) {
            return $($fieldNode).find(".customized-drop-zone[data-category-id='0']");
        },
        
        onDropZoneAttachmentUploadFinished: function (event) {
            var wikiAttachmentButton = $('.wiki-edit-attachment-picker-trigger');
            var hasHiddenClass = wikiAttachmentButton.hasClass('hidden');
            if (hasHiddenClass) {
                wikiAttachmentButton.data('saIsHidden', wikiAttachmentButton.hasClass('hidden'));
            }
            wikiAttachmentButton.removeClass('hidden');
        },

        onDestroyProgressBar: function (event) {
            var form = $(event.target.$containerNode);
            var files = CategoryFormDropZone.getTempAttachmentsWithCategories(form);

            if (files.length === 0) {
                var wikiAttachmentButton = $('.wiki-edit-attachment-picker-trigger');
                var hasHiddenClass = wikiAttachmentButton.data('saIsHidden');
                if (hasHiddenClass) {
                    wikiAttachmentButton.addClass('hidden');
                }
            }
        },

        addAttachmentNameToTitleForProgressBar: function (progressBar) {
            var fileNameItem = $(progressBar).find('.upload-progress-bar__file-name');
            var fileName = fileNameItem.text();
            fileNameItem.attr('title', fileName);
        }
    };

    AJS.toInit(function () {
        catchTempFiles();
    });

    function catchTempFilesByMutationObserver(newSection, jiraAttachContainer) {
        var storedTempFiles = CategoryFormDropZone.retrieveTempAttachmentsFromBrowserCache();
        var fieldSet = $(jiraAttachContainer).parent();

        var callBack = function (mutations) {
            if (mutations.length) {
                $.each(mutations, function (i, mutation) {
                    if (mutation.addedNodes.length) {
                        var tempAttachment = $(mutation.addedNodes[0]);
                        var fileId = tempAttachment.data('fileId') || tempAttachment.val();
                        if (!fileId) {
                            return
                        }
                        var isAlreadyMoved = CategoryFormDropZone.toPlaceInNeededSection(fieldSet, storedTempFiles, fileId, tempAttachment);
                        if (!isAlreadyMoved) {
                            newSection.append(tempAttachment);
                        }
                    }
                });
            }
        };

        var disconnect = function (observer, fnCallback, itemForObservation) {
            setTimeout(function () {
                if (observer) {
                    observer.disconnect();
                } else {
                    itemForObservation.removeEventListener('DOMNodeInserted', fnCallback, false);
                    itemForObservation.removeEventListener('DOMNodeRemoved', fnCallback, false);
                }
            }, 30000);
        };

        trackDOMChanges(jiraAttachContainer.find('.issue-drop-zone').get(0), callBack, disconnect);
    }

    function trackDOMChanges(itemForObservation, fnCallback, fnDisconnect) {
        var MutationObserver = window.MutationObserver || window.WebKitMutationObserver,
            eventListenerSupported = window.addEventListener;

        if (MutationObserver) {
            var observer = new MutationObserver(function (mutations) {
                fnCallback(mutations);
            });

            observer.observe(itemForObservation, {
                attributes: true,
                childList: true,
                characterData: false
            });

            fnDisconnect(observer);
        }
        else if (eventListenerSupported) {
            itemForObservation.addEventListener('DOMNodeInserted', fnCallback, false);
            itemForObservation.addEventListener('DOMNodeRemoved', fnCallback, false);
            fnDisconnect(undefined, fnCallback, itemForObservation);
        }
    }

    function moveMissedAttachmentsToDropZone(section, jiraAttachContainer) {
        var files = jiraAttachContainer.find('input[name="filetoconvert"], div.upload-progress-bar');
        section.append(files);
    }

    function catchTempFiles() {
        var context = $('.field-group.file-input-list');
        var inputs = context.find('input[name=filetoconvert]:checked');

        CategoryFormDropZone.files = inputs.map(function (idx, el) {
            return {id: el.value, name: $(el).siblings('label').text()};
        }).toArray();

        CategoryFormDropZone.jiraTempFiles = CategoryFormDropZone.retrieveTempAttachmentsFromBrowserCache();
    }

    function dragEventContainsFiles(event) {
        var dataTransfer = getDataTransferFromEvent(event);

        if (dataTransfer && dataTransfer.types) {
            var types = dataTransfer.types;
            for (var i = 0; i < types.length; i++) {
                if (types[i] === "Files") {
                    return true;
                }
            }
            return false;
        }
        return true;
    }

    function getPermittedCategories(categories) {
        return $.grep(categories, function (category) {
            return !category.hidden;
        });
    }

    function getDataTransferFromEvent(event) {
        if (event.dataTransfer) {
            return event.dataTransfer;
        }

        if (event.originalEvent && event.originalEvent.dataTransfer) {
            return event.originalEvent.dataTransfer;
        }
    }

})(AJS.$);