(function ($) {

    window.AttachCatVersioning = {
        issueId: null,
        expandedDocs: {}
    };

    window.AttachCatDialogHandler = {

        categories: null,
        pid: null,
        issueType: null,

        init: function() {
            var t = this;

            t.bindCreateIssueDialog();

            if (this.isEmailForm()) {
                return;
            }

            var formFields = $('form .field-group.file-input-list, #attach-screenshot-filename-group');

            var formFieldGroup = $('form .field-group.file-input-list');
            var isIssueForm = formFieldGroup.length > 0;

            var screenshotFieldGroup = $('#attach-screenshot-filename-group');
            var isScreenshotForm = screenshotFieldGroup.length > 0;

            if (isIssueForm || isScreenshotForm) {
                var form = formFields.parents('form');
                var isCreate = t.isExistProjectId();

                t.pid = this.getProjectId();
                t.issueType = this.getIssueType();

                t.getCategories(t.pid, t.issueType, !isCreate);
                t.bindSaveFormEvent(form, isCreate);

                var $fileInputList = isScreenshotForm ? screenshotFieldGroup : formFieldGroup;

                isIssueForm && t.fieldTypeDropZones ? t.initFormDropZones(formFieldGroup)
                    : t.initDropdownCategories(form, $fileInputList);
            }
        },

        bindCreateIssueDialog: function() {
            var t = this;

            JIRA.bind(JIRA.Events.NEW_CONTENT_ADDED, function(e, context) {
                if (t.isServiceDeskAttachDialog(context)) {
                    t.createCategorySelectInServiceDeskDialog(context);
                    return;
                }

                if (!t.isSuitableDialogForAttachmentUpload(context)) {
                    return;
                }

                var dialog = context.parents('.jira-dialog');
                if (dialog.length === 0) return;
                var isCreateSubTask = context.parents("#create-subtask-dialog").size() > 0;
                var isCreateIssue = !isCreateSubTask ? t.isCreateIssueDialog(dialog) : false;
                t.createCategorySelectInContext(dialog, isCreateSubTask, isCreateIssue);
            });
        },

        createCategorySelectInContext: function($context, isCreateSubTask, isCreateIssue) {
            var t = this;
            var $fileInputList = $context.find('.field-group.file-input-list, #attach-screenshot-filename-group');

            if($fileInputList.size() === 0) {
                return;
            }

            var isIssueForm = $fileInputList.hasClass('file-input-list');

            if (isCreateIssue) {
                t.bindCreateIssueEvents($context, $fileInputList);
            } else {
                var versioning = null;
                var catId = 0;
                if (window.AttachCat && AttachCat.issueId == JIRA.Issue.getIssueId()) {
                    var issueTypeSelect = $context.find('#issuetype');
                    var issueIdFromDialog = $context.find('input[name=id]').val();

                    if (issueIdFromDialog && issueIdFromDialog != AttachCat.issueId) {
                        t.getCategories(issueIdFromDialog, -1, true);
                    } else {
                        t.getCategories(AttachCat.issueId, issueTypeSelect.val(), true);
                        catId = AttachCat.catId;
                        versioning = AttachCat.versioning;
                    }

                    issueTypeSelect.change(function() {
                        t.issueType = issueTypeSelect.val();
                        t.getCategories(AttachCat.issueId, t.issueType, true);
                    });

                } else {
                    t.getCategories(JIRA.Issue.getIssueId() || $('input[name="id"]').val()
                        || $('input[name="parentIssueId"]').val() || $('#test-exec-id').val(), -1, true);
                }

                if (t.categories.length < 1 && !versioning) {
                    return;
                }

                $context.find('#attach-cat-select-field-container').remove();
                $context.find('#attach-cat-version-field-container').remove();

                if (!versioning) {
                    isIssueForm && t.fieldTypeDropZones ? t.reinitFormDropZone($fileInputList)
                        : t.initDropdownCategories($context, $fileInputList, catId);

                    if ($context.find('#attach-cat-select option').length == 0) {
                        $context.find('#attach-cat-select-field-container').html('');
                        AJS.messages.warning("#attach-cat-select-field-container", {
                            title: AJS.I18n.getText("attach.cat.dialog.warn.title"),
                            body: AJS.I18n.getText("attach.cat.dialog.warn.body")
                        });
                    }
                } else {
                    SALoadVersionDialogHandler.init($context, $fileInputList, versioning);
                }

                t.bindSubmitFormEventInContext($context, versioning, isCreateSubTask, isCreateIssue);
            }
        },

        createCategorySelectInServiceDeskDialog: function (context) {
            var t = this;

            context.find('#sd-attach-cat-select-field-container').remove();

            t.getCategories(JIRA.Issue.getIssueId(), null, true);
            if (t.categories.length < 1) {
                return;
            }

            var dropDownTemplate = AttachCatDialog.serviceDeskCategorySelect({categories: t.categories, defaultId: AttachCat.catId});
            context.find('#attachment-area').before(dropDownTemplate);

            t.inlineHelpForCategorySelectInServiceDeskDialog();

            t.handleServiceDeskFormSubmit(context);
        },

        handleServiceDeskFormSubmit: function (context) {
            var t = this;

            context.find('button[type="submit"], #button-public, #button-private').mousedown(function () {
                var files = CategoryFormDropZone.getTempAttachmentsWithCategories(context);
                t.saveAttachmentInCategory(t.getIssueKey(), files);
            })
        },

        bindCreateIssueEvents: function ($context, $fileInputList) {
            var t = this;

            var projectSelect = $context.find('#project');
            var issueTypeSelect = $context.find('#issuetype');
            t.pid = projectSelect.val();
            t.issueType = issueTypeSelect.val();
            t.getCategories(t.pid, t.issueType);
            t.fieldTypeDropZones ? t.reinitFormDropZones($fileInputList) : t.initDropdownCategories($context, $fileInputList);

            projectSelect.change(function() {
                t.pid = projectSelect.val();
                t.issueType = issueTypeSelect.val();
                t.getCategories(t.pid, t.issueType);
            });

            issueTypeSelect.change(function() {
                t.pid = projectSelect.val();
                t.issueType = issueTypeSelect.val();
                t.getCategories(t.pid, t.issueType);
            });

            t.bindSaveFormEvent($context.find('form'), true, true);
        },

        bindSubmitFormEventInContext: function ($context, versioning, isCreateSubTask, isCreateIssue) {
            var t = this;
            var form = $context.find('form');

            $(form).unbind('before-submit').bind('before-submit', function() {

                if (isCreateSubTask || isCreateIssue) {
                    SmartAttachAnalytics.eventAttachCreateIssue();
                } else {
                    SmartAttachAnalytics.eventAttachForm();
                }

                if (versioning) {
                    SALoadVersionDialogHandler.getAttachmentsAndCreateRevision(form, versioning);
                } else {
                    var files = CategoryFormDropZone.getTempAttachmentsWithCategories(form);

                    t.getScreenshots(files, form);

                    if (files.length === 0) {
                        return;
                    }

                    $.each(files, function (i, file) {
                        SmartAttachAnalytics.eventUploadFile(file.name);
                    });

                    if (isCreateSubTask) {
                        var subTaskSummary = $context.find('#summary').val();
                        var projectId = $context.find('#project').val();
                        t.saveAttachmentsCache(projectId, subTaskSummary, files);
                    } else {
                        var issueKey = t.getIssueKey();
                        var issueIdFromDialog = $context.find('input[name=id]').val();

                        if (issueIdFromDialog && issueIdFromDialog != JIRA.Issue.getIssueId()) {
                            var key = $(document).find("tr[rel=" + issueIdFromDialog + "]").data("issuekey");
                            issueKey = key ? key : issueKey;
                        }
                        t.saveAttachmentInCategory(issueKey, files);
                    }
                }
            });
        },

        bindSaveFormEvent: function(form, isCreate, isDialog) {
            var t = this;

            $(form).unbind('before-submit').bind('before-submit', function() {
                var files = CategoryFormDropZone.getTempAttachmentsWithCategories(form);

                t.getScreenshots(files, form);

                if (files.length === 0) {
                    return;
                }

                $.each(files, function (i, file) {
                    SmartAttachAnalytics.eventUploadFile(file.name);
                });

                CategoryFormDropZone.storeTempAttachmentsInBrowserCache(form);

                if (isCreate) {
                    SmartAttachAnalytics.eventAttachCreateIssue();
                    t.saveAttachmentsCache(t.pid, $('#summary').val(), files);
                } else {
                    SmartAttachAnalytics.eventAttachForm();
                    t.saveAttachmentInCategory(t.getIssueKey(), files);
                }
            });
        },

        reinitFormDropZones: function ($formFieldGroup) {
            CategoryFormDropZone.removeCurrentDropZones($formFieldGroup);
            CategoryFormDropZone.init(this.categories, $formFieldGroup);
        },

        reinitFormDropZone: function ($formFieldGroup) {
            CategoryFormDropZone.removeCurrentDropZones($formFieldGroup);
            AttachCatDialogHandler.initFormDropZoneForSelectedCategory($formFieldGroup);
        },

        initFormDropZones: function ($formFieldGroup) {
            CategoryFormDropZone.init(this.categories, $formFieldGroup);
        },

        initFormDropZoneForSelectedCategory: function ($formFieldGroup) {
            if (JIRA.Dialog.current !== null && JIRA.Dialog.current.$activeTrigger !== undefined) {
                var $activeTrigger = JIRA.Dialog.current.$activeTrigger;
                var categoryId = $activeTrigger.data('cat');

                var requiredCategory = $.grep(this.categories, function (category) {
                    return category.id === categoryId;
                });

                if (requiredCategory.length) {
                    CategoryFormDropZone.init(requiredCategory, $formFieldGroup);
                    return;
                }
            }
            AttachCatDialogHandler.initFormDropZones($formFieldGroup);
        },

        initDropdownCategories: function($context, $fileInputList, catId) {
            var t = this;

            $context.find('#attach-cat-select-field-container').remove();

            if (t.categories.length < 1) {
                return;
            }

            $fileInputList.parent().before(AttachCatDialog.catSelect({categories: t.categories, defaultId: catId || 0}));

            if ($context.find('#attach-cat-select option').length == 0) {
                $context.find('#attach-cat-select-field-container').html('');
                AJS.messages.warning("#attach-cat-select-field-container", {
                    title: AJS.I18n.getText("attach.cat.dialog.warn.title"),
                    body: AJS.I18n.getText("attach.cat.dialog.warn.body")
                });
            }
        },

        inlineHelpForCategorySelectInServiceDeskDialog: function () {
            AJS.InlineDialog($('#sd-category-select-info'), 'sd-category-select-info',
                function (content, trigger, showPopup) {
                    content.html(AttachCatDialog.serviceDeskCategorySelectInlineHelp());
                    showPopup();
                    return false;
                }, {
                    offsetX: -10
                });
        },

        saveAttachmentInCategory: function(issueKey, files) {
            JIRA.SmartAjax.makeRequest({
                type: 'POST',
                async: false,
                url: AJS.params.baseURL + '/rest/attach-cat/1.0/attachments',
                data: {
                    issueKey: issueKey,
                    issueId: JIRA.Issue.getIssueId(),
                    tempAttachments: JSON.stringify(files)
                }
            });
        },

        saveAttachmentsCache: function(pid, summary, files) {
            JIRA.SmartAjax.makeRequest({
                type: 'POST',
                async: false,
                url: AJS.params.baseURL + '/rest/attach-cat/1.0/attachments/cache',
                data: {
                    pid: pid,
                    summary: summary,
                    tempAttachments: JSON.stringify(files)
                }
            });
        },

        getCategories: function(pid, issueType, isIssue) {
            var t = this;
            var categoriesRestUrl = '/rest/attach-cat/1.0/attachments/categories';

            JIRA.SmartAjax.makeRequest({
                type: 'GET',
                url: AJS.params.baseURL + categoriesRestUrl,
                async: false,
                data: {
                    pid: pid,
                    issueType: issueType ? issueType : -1,
                    isIssue: isIssue
                },
                success: function (data, statusText, xhr) {
                    t.categories = xhr.status === 200 ? data.categories : [];
                    t.fieldTypeDropZones = data ? data.dropZones : true;
                },
                error: function (jqXHR) {
                    t.categories = [];
                    console.error(categoriesRestUrl + ' ' + jqXHR.status + ' ' + jqXHR.statusText);

                    if (jqXHR.status === 412) {
                        return;
                    }

                    JIRA.Messages.showMsg(AJS.I18n.getText("attach.cat.get.error"), {
                        type: JIRA.Messages.Types.ERROR,
                        closeable: false,
                        timeout: 3
                    });
                }
            });
        },

        getScreenshots: function (files, $form) {
            var screenshotName = $form.find("#attachscreenshotname");
            if (screenshotName.length) {
                var fullScreenshotName = screenshotName.val() + ".png";
                var categoryId = $form.find('#attach-cat-select').val();
                files.push(new AttachCatClasses.TempAttachment(fullScreenshotName, categoryId));
            }
        },

        getFilesFromJIRADropZone: function ($form) {
            var files = [];
            $.each($form.find('input.checkbox[name="filetoconvert"]:checked'), function(i, input) {
                files.push($(input).parent().find('label').text().trim());
            });
            $.each($form.find('.upload-progress-bar__file-name'), function(i, span) {
                files.push($(span).text().trim());
            });
            return files;
        },

        getIssueKey: function() {
            var issueKey = JIRA.Issue.getIssueKey();
            if (issueKey === null) issueKey = AJS.Meta.get("issue-key");
            return issueKey !== null ? issueKey : "";
        },

        isEmailForm: function () {
            return $('#sendEmailForm').length !== 0;
        },

        isCreateIssueDialog: function (dialog) {
            var isCreateIssueParam = dialog.find('[name="isCreateIssue"]');
            if (isCreateIssueParam.length) {
                return isCreateIssueParam.val() === "true";
            }

            var dialogId = dialog.attr('id');
            if (dialogId) {
                return dialogId.match(/((create)(.*?)(dialog)|(dialog)(.*?)(create))/i) !== null;
            }

            var submitButtonId = dialog.find('input[type="submit"]').attr('id');
            if (submitButtonId){
                return submitButtonId.match(/(create)(.*?)(issue)/i) !== null;
            }

            return false;
        },

        isExistProjectId: function () {
            return $('form input[name="pid"]').length !== 0;
        },

        getProjectId: function () {
            var inputWithId = $('form input[name="pid"]');
            return inputWithId.length ? inputWithId.val() : $('form input[name="id"]').val();
        },

        getIssueType: function () {
            return $('form input[name="issuetype"]').val();
        },

        isServiceDeskAttachDialog: function (context) {
            return context.hasClass('servicedesk-attachments-dialog')
        },

        isSuitableDialogForAttachmentUpload: function (context) {
            if (context.find('.field-group.file-input-list, #attach-max-size').length === 0) {
                return false;
            }

            var dialogsWithoutAttachments = context
                .find('#assign-issue, #log-work, #comment-add, #comment-edit, #link-jira-issue');

            return dialogsWithoutAttachments.length === 0;
        }
    };

    window.AttachCatClasses = {

        TempAttachment: function (name, categoryId) {
            this.name = name;
            this.categoryId = categoryId;
        },

        JiraTempAttachment: function (fileId, categoryId) {
            this.fileId = fileId;
            this.categoryId = categoryId;
        },

        CustomDropZone: function (dropZone, section, categoryId) {
            this.dropZone = dropZone;
            this.section = section;
            this.categoryId = categoryId;
        }
    };

    AJS.toInit(function () {
        AttachCatDialogHandler.init();
    });

})(AJS.$);