define('smart-attachments/dialog/BulkForm',
    ['jquery', 'underscore', 'jira/lib/class'],
    function ($, _, Class) {

        var BulkForm = Class.extend({
            group: 'bulk',
            operations: ['delete', 'download', 'move', 'send', 'archive'],
            init: function (opts) {
                this.element = opts.element;
                this.forms = this.element.find('.attach-cat-bulk-container');
                this.saveBtn = this.forms.find('.attach-cat-save-bulk');
                this.cancelBtn = this.forms.find('.attach-cat-cancel-bulk');
                this.spinner = this.forms.find('.button-spinner');
                this.opSelect = this.forms.find('.attach-cat-bulk-op');
                this.catSelect = this.forms.find('.attach-cat-bulk-move');
                this.checkboxes = this.element.find('.attach-cat-container .attach-cat-bulk-checkbox');
                this.zipName = this.forms.find('.attach-cat-zip-name');
                this.operationCode = -1;
                this.connectContainer();
            },

            connectContainer: function () {
                this.cancelBtn.on('click', this.hide.bind(this));
                this.opSelect.on('change', this.onOperationChange.bind(this));
                this.catSelect.on('change', this.onCategoryChange.bind(this));
                this.saveBtn.on('click', this.onSaveClick.bind(this));
            },

            onOperationChange: function (event) {
                var value = event.target.value;
                this.opSelect.find('option:selected').removeAttr('selected', 'selected');
                this.opSelect.find('option[value="' + value + '"]').attr('selected', 'selected');
                this.operationCode = parseInt(value);
                if (value === '2' || value === '4') {
                    this.catSelect.show();
                } else {
                    this.catSelect.hide();
                }

                if (value === '4') {
                    this.zipName.show().val('');
                } else {
                    this.zipName.hide();
                }

                this.enableConfirm();
            },

            onCategoryChange: function (event) {
                var value = event.target.value;
                this.catSelect.find('option:selected').removeAttr('selected', 'selected');
                this.catSelect.find('option[value="' + value + '"]').attr('selected', 'selected');
            },

            onSaveClick: function (event) {
                event.preventDefault();
                event.stopPropagation();
                var operationName = this.operations[this.operationCode];
                var operationParams = [];
                if (this.operationCode === 2 || this.operationCode === 4) {
                    operationParams.push(parseInt(this.catSelect.val()));
                }
                if (this.operationCode === 4) {
                    operationParams.push(this.zipName.val() || AJS.I18n.getText("attach.cat.archive.def.name"));
                }
                this.trigger(this.group + ":" + operationName, operationParams);
                return false;
            },

            show: function () {
                this.element.addClass('bulk-mode');
                this.resetView();
                this.element.on('click.batch', this.enableConfirm.bind(this));
                this.trigger('bulk:show');
            },

            hide: function () {
                this.element.removeClass('bulk-mode');
                this.element.find('li.selected-attach').removeClass('selected-attach');
                this.element.off('click.batch');
                this.trigger('bulk:hide');
                return false;
            },

            resetView: function () {
                this.checkboxes.removeAttr('checked');
                this.opSelect.find(':selected').removeAttr('selected', 'selected');
                this.catSelect.find(':selected').removeAttr('selected', 'selected');
                this.catSelect.hide();
                this.zipName.val('');
                this.zipName.hide();
                this.operationCode = -1;
                this.saveBtn.enable(false);
            },

            enableConfirm: function () {
                if (this.element.find('.selected-attach').length && this.operationCode > -1) {
                    this.saveBtn.enable();
                } else {
                    this.saveBtn.enable(false);
                }
            },

            startSpinner: function () {
                this.spinner.spin();
                this.saveBtn.enable(false);
                this.cancelBtn.enable(false);
            },

            stopSpinner: function () {
                this.spinner.spinStop();
                this.saveBtn.enable();
                this.cancelBtn.enable();
            }
        });
        return BulkForm;
    });