(function ($) {

    AJS.toInit(function () {
        SADialogEmail.bindAttachmentSentClick();
    });

    window.SADialogEmail = {

        bindAttachmentSentClick: function () {
            $('body')
                .off('click.sa-mail-attach')
                .on('click.sa-mail-attach', 'a.sa-mail-attach', function () {
                    var a = $(this);
                    var opts = a.parents('.aui-list');
                    var isDoc = opts.data('document');
                    var id = opts.data('attachId');
                    var item = AttachCat.getItemById(id);
                    if (isDoc) {
                        item = item.not('.revision');
                    }

                    var attachments = SADialogEmail.getIssueAttachmentsForSend([id], [item]);
                    SADialogEmail.createAndShowEmailDialog(attachments);

                    return false;
                })
                .off('change.sa-put-to-comment')
                .on('change.sa-put-to-comment', '#putEmailToComments', SADialogEmail.storeCheckboxStateOfCommenting);
        },

        createAndShowEmailDialog: function (attachments) {

            var emailDialog = new AJS.Dialog({
                width: 650,
                height: 550,
                id: "mail-attach-dialog",
                closeOnOutsideClick: false
            });

            emailDialog.addHeader(AJS.I18n.getText("attach.cat.mail.dialog.title"));

            var emailDialogBody = $(AttachCategory.mailDialog({
                attachments: attachments,
                issueKey: AttachCat.getIssueKey(),
                issueSummary: $('#summary-val').text()
            }));

            emailDialog.addPanel('panel', emailDialogBody, 'mail-attach-dialog-panel');

            emailDialog.addButton(AJS.I18n.getText("attach.cat.mail.dialog.send"), function() {
                if (!SADialogEmail.WriteMailForm.checkEnteredMails(emailDialogBody)) {
                    return false;
                }

                var attachmentIdsForSend = SADialogEmail.getFormCheckedAttachments(emailDialogBody);

                JIRA.SmartAjax.makeRequest({
                    type: 'POST',
                    url: AJS.params.baseURL + '/rest/attach-cat/1.0/attachments/send/mail',
                    data: {
                        issueKey: AttachCat.getIssueKey(),
                        subject: emailDialogBody.find('#subject').val(),
                        message: emailDialogBody.find('#message').val(),
                        ids: attachmentIdsForSend,
                        to: emailDialogBody.find('#to').val().trim(),
                        cc: emailDialogBody.find('#cc').val().trim(),
                        bcc: emailDialogBody.find('#bcc').val().trim(),
                        sendWithoutAttachments: false,
                        copyOfEmailToComments: $('#putEmailToComments').is(':checked')
                    },
                    success: function() {
                        SADialogEmail.gaAnalyticsAttachmentsSent(attachmentIdsForSend);
                        emailDialog.remove();
                        SADialogEmail.showSuccessMessageFlag();
                        JIRA.trigger(JIRA.Events.REFRESH_ISSUE_PAGE, [JIRA.Issue.getIssueId()])
                    },
                    error: SADialogEmail.showErrorMessageFlag
                });
            }, "send-attach-mail");

            emailDialog.addLink(AJS.I18n.getText("attach.cat.rename.dialog.cancel"), function (dialog) {
                dialog.remove();
            }, '#');

            SmartAttachAutocomplete.init(emailDialogBody.find('#to'));
            SmartAttachAutocomplete.init(emailDialogBody.find('#cc'));
            SmartAttachAutocomplete.init(emailDialogBody.find('#bcc'));
            emailDialog.show();

            SADialogEmail.synchronizeCheckboxAndStoredStateOfCommenting();
        },

        getIssueAttachmentsForSend: function (attachmentIds, attachmentDOMItem) {
            var attachments = [];
            $.each(attachmentIds, function (i, id) {
                attachments.push({id: id, name: AttachCat.getItemNameElement(attachmentDOMItem[i]).eq(0).text()})
            });
            return attachments;
        },

        getFormCheckedAttachments: function (formBody) {
            var markedAttachments = [];
            formBody.find('.attach:checked').each(function () {
                markedAttachments.push($(this).attr('id'));
            });
            return markedAttachments;
        },

        EmailChecker: {

            valid: function (email) {
                return /^([a-zA-Z0-9_\.\-])+@(([a-zA-Z0-9\-])+\.)+([a-zA-Z0-9]{2,4})+$/.test(email);
            },

            testEmails: function (string) {
                var result = {
                    success: true,
                    emailsCount: 0
                };
                $.each($.trim(string).split(/\s*,\s*/), function (i, v) {
                    if ($.trim(v).length > 0) {
                        result.emailsCount++;
                        if (!SADialogEmail.EmailChecker.valid(v)) {
                            result.success = false;
                        }
                    }
                });
                return result;
            }
        },

        WriteMailForm: {

            checkEnteredMails: function (form) {
                $('.description', form).hide();

                var resultTo = SADialogEmail.EmailChecker.testEmails($('#to', form).val());
                var resultCc = SADialogEmail.EmailChecker.testEmails($('#cc', form).val());
                var resultBcc = SADialogEmail.EmailChecker.testEmails($('#bcc', form).val());
                var resultMessage = $.trim($('#message', form).val());
                var resultSubject = $.trim($('#subject', form).val());

                if (resultTo.emailsCount === 0) {
                    $('#empty-to', form).show();
                } else if (!resultTo.success) {
                    $('#error-to', form).show();
                }

                if (!resultCc.success) {
                    $('#error-cc', form).show();
                }

                if (!resultBcc.success) {
                    $('#error-bcc', form).show();
                }

                if (!resultMessage) {
                    $('#empty-message', form).show();
                }

                if (!resultSubject) {
                    $('#empty-subject', form).show();
                }

                return resultTo.success &&
                    resultTo.emailsCount > 0 &&
                    resultCc.success &&
                    resultBcc.success &&
                    resultMessage &&
                    resultSubject;
            }
        },

        showSuccessMessageFlag: function () {
            JIRA.Messages.showMsg(AJS.I18n.getText("attach.cat.mail.dialog.success"), {
                type: JIRA.Messages.Types.SUCCESS,
                closeable: false,
                timeout: 3
            });
        },

        showErrorMessageFlag: function () {
            JIRA.Messages.showMsg(AJS.I18n.getText("attach.cat.mail.dialog.error"), {
                type: JIRA.Messages.Types.ERROR,
                closeable: false,
                timeout: 3
            });
        },
        
        gaAnalyticsAttachmentsSent: function (attachmentIds) {
            if (attachmentIds.length > 1) {
                SmartAttachAnalytics.eventBulkOpSendEmail(attachmentIds.length);
            } else {
                SmartAttachAnalytics.eventAttachmentSendToEmail();
            }
        },

        storeCheckboxStateOfCommenting: function () {
            JIRA.SmartAjax.makeRequest({
                type: 'POST',
                url: AJS.params.baseURL + '/rest/attach-cat/1.0/settings/email-to-comment',
                data: {
                    projectKey: issueKeyToProjectKey(AttachCat.getIssueKey()),
                    state: $(this).is(':checked')
                }
            });
        },

        synchronizeCheckboxAndStoredStateOfCommenting: function () {
            JIRA.SmartAjax.makeRequest({
                type: 'GET',
                async: false,
                url: AJS.params.baseURL + '/rest/attach-cat/1.0/settings/email-to-comment',
                data: {
                    projectKey: issueKeyToProjectKey(AttachCat.getIssueKey())
                },
                success: function (data) {
                    $('#putEmailToComments').attr('checked', data === 'true')
                }
            });
        }
    };

    function issueKeyToProjectKey(issueKey) {
        if (issueKey === '') {
            return issueKey;
        } else {
            return issueKey.split('-')[0];
        }
    }

})(AJS.$);