(function ($) {

    window.SALoadVersionDialogHandler = {

        init: function ($context, $fileInputList, versioning) {
            this.createInputDocumentName($fileInputList, versioning);
            this.changeDialogTitle($context);

            var attachments = this.getAttachesWithoutCurrent(this.getAttachesWithoutDocuments(), versioning);
            if (!attachments.length) return;

            var dropdown = this.createDropdownWithAttachments($fileInputList, attachments);
            this.bindAttachmentDropdownChange(dropdown, $fileInputList.parent());
            this.bindAttachmentDropZoneChange($fileInputList);
        },

        createInputDocumentName: function ($fileInputList, versioning) {
            if (!versioning.isDoc) {
                $fileInputList.parent().before(AttachCatDialog.versionInput({name: versioning.name}));
            }
        },

        changeDialogTitle: function ($context) {
            var title = $context.find('.jira-dialog-heading h2');
            title.attr('title', AJS.I18n.getText("attach.cat.load.version"));
            title.text(AJS.I18n.getText("attach.cat.load.version"));
        },

        getAttachesWithoutDocuments: function () {
            return AttachCat.getAttachmentsFromSection()
                .not('.revision')
                .filter(function (i, attachment) {
                    return $(attachment).data('document') === undefined;
                });
        },

        getAttachesWithoutCurrent: function (attachments, versioning) {
            var currentAttachment = AttachCat.getItemById(versioning.callerAttachId).not('.revision');
            var index = attachments.index(currentAttachment);
            if (index > -1) {
                attachments.splice(index, 1);
            }
            return attachments;
        },

        createDropdownWithAttachments: function ($fileInputList, attachmentNodes) {
            var attachments = this.convertAttachmentsForSoyTemplate(attachmentNodes);
            var renderedTemplate = $(AttachCatDialog.attachSelect({attachments: attachments}));
            $fileInputList.parent().after(renderedTemplate);
            return renderedTemplate;
        },

        convertAttachmentsForSoyTemplate: function (attachments) {
            var result = [];
            $.each(attachments, function (i, attachment) {
                var $attachment = $(attachment);
                result.push({
                    id: AttachCat.getItemId($attachment),
                    name: AttachCat.getItemNameElement($attachment).text().trim()
                });
            });
            return result;
        },

        bindAttachmentDropdownChange: function ($dropdown, $fieldsetWhitDropZone) {
            $dropdown.change(function (event) {
                var attachId = $(event.target).val();
                if (attachId > 0) {
                    SALoadVersionDialogHandler.hideFieldsetWithDropZone($fieldsetWhitDropZone);
                } else {
                    $fieldsetWhitDropZone.show();
                }
            });
        },

        bindAttachmentDropZoneChange: function ($fileInputList) {

            var callBack = function () {
                var t = SALoadVersionDialogHandler;
                t.getAttachmentsFromDropZone(t.trackedItem).length ? t.hideAttachDropdown() : t.showAttachDropdown();
            };

            var disconnect = function (observer, fnCallback) {
                $(JIRA.Dialog.current).one('Dialog.beforeHide', function () {
                    if (observer) {
                        observer.disconnect();
                    } else if (fnCallback){
                        var item = SALoadVersionDialogHandler.trackedItem.get(0);
                        item.removeEventListener('DOMNodeInserted', fnCallback, false);
                        item.removeEventListener('DOMNodeRemoved', fnCallback, false);
                    }
                });
            };

            SALoadVersionDialogHandler.trackDOMChanges($fileInputList, callBack, disconnect);
        },

        trackDOMChanges: function ($itemForObservation, fnCallback, fnDisconnect) {
            var MutationObserver = window.MutationObserver || window.WebKitMutationObserver,
                eventListenerSupported = window.addEventListener;

            SALoadVersionDialogHandler.trackedItem = $itemForObservation;
            var itemForObservation = $itemForObservation.get(0);

            if (MutationObserver) {
                var observer = new MutationObserver(function () {
                    fnCallback();
                });

                observer.observe(itemForObservation, {
                    attributes: false,
                    childList: true,
                    characterData: false
                });

                fnDisconnect(observer);
            }
            else if (eventListenerSupported) {
                itemForObservation.addEventListener('DOMNodeInserted', fnCallback, false);
                itemForObservation.addEventListener('DOMNodeRemoved', fnCallback, false);
                fnDisconnect(undefined, fnCallback);
            }
        },

        getAttachmentsAndCreateRevision: function (form, versioning) {
            if (!versioning.isDoc) {
                versioning.name = form.find('#attach-cat-version').val().trim() || versioning.name;
            }

            if (this.isAttachDropdownSelected()) {
                var attachmentId = this.getAttachDropdownValue();
                var commentContent = this.getDialogCommentContent(form);
                this.postCreateVersionByAttachId(attachmentId, commentContent, versioning);
            } else {
                var files = AttachCatDialogHandler.getFilesFromJIRADropZone(form);
                this.postCreateVersionByAttachNames(files, versioning);
            }
        },

        postCreateVersionByAttachNames: function (files, versioning) {
            if (files.length === 0) {
                return;
            }

            JIRA.SmartAjax.makeRequest({
                type: 'POST',
                async: false,
                url: AJS.params.baseURL + '/rest/attach-cat/1.0/attachments/version',
                data: {
                    issueKey: AttachCat.getIssueKey(),
                    catId: versioning.catId,
                    parent: versioning.parent,
                    name: versioning.name,
                    documentId: versioning.documentId,
                    attachments: files
                }
            });
        },

        postCreateVersionByAttachId: function (attachId, commentContent, versioning) {
            JIRA.SmartAjax.makeRequest({
                type: 'POST',
                async: false,
                url: AJS.params.baseURL + '/rest/attach-cat/1.0/attachments/revision-comment',
                data: {
                    attachmentId: attachId,
                    commentContent: commentContent,
                    catId: versioning.catId,
                    parent: versioning.parent,
                    documentName: versioning.name,
                    documentId: versioning.documentId
                },
                success: function () {
                    JIRA.Dialog.current.hide();
                    if (AttachCat.isAgile) {
                        GH.DetailsView.reload();
                    } else {
                        JIRA.trigger(JIRA.Events.REFRESH_ISSUE_PAGE, [JIRA.Issue.getIssueId()]);
                    }
                }
            });
        },

        hideFieldsetWithDropZone: function (fieldset) {
            this.removeAttachments(fieldset);
            fieldset.hide()
        },

        removeAttachments: function (fieldset) {
            this.getAttachmentsFromDropZone(fieldset).remove();
        },

        getAttachmentsFromDropZone: function ($fileInputList) {
            return $fileInputList.find('input[name="filetoconvert"]:checked');
        },

        getFieldContainerDropdown: function () {
            return $('#sa-attach-select-field-container');
        },

        hideAttachDropdown: function () {
            var container = SALoadVersionDialogHandler.getFieldContainerDropdown();
            container.find('select').val('-1');
            container.fadeOut('fast');
        },

        showAttachDropdown: function () {
            SALoadVersionDialogHandler.getFieldContainerDropdown().show();
        },

        isAttachDropdownSelected: function () {
            var selectedValue = this.getAttachDropdownValue();
            return selectedValue !== undefined && parseInt(selectedValue) !== -1;
        },

        getAttachDropdownValue: function () {
            return SALoadVersionDialogHandler.getFieldContainerDropdown().find('select').val();
        },

        getDialogCommentContent: function (form) {
            return form.find('#comment').val();
        }
    }

})(AJS.$);