(function ($) {

    window.SAProjectCategoryDialogs = {

        showImport: function () {
            var projects;

            JIRA.SmartAjax.makeRequest({
                type: 'GET',
                url: AJS.params.baseURL + '/rest/attach-cat/1.0/project/projects',
                async: false,
                data: {
                    projectKey: getProjectKey()
                },
                success: function (data) {
                    projects = data;
                }
            });

            var dialog = new AJS.Dialog({
                width: $('table.attach-cat').width() + 100,
                height: $(window).height() * 0.7,
                id: "import-categories-dialog",
                closeOnOutsideClick: false
            });

            dialog.addHeader(AJS.I18n.getText("attach.cat.project.import.dialog.title"));

            dialog.addPanel('panel', AttachCat.importDialogBody({projects: projects}), 'import-categories-dialog-panel');

            var tBody = $('table.attach-cat-import tbody');
            var select = $('#attach-cat-project');

            var updateTable = function (readOnly) {
                SAProjectCategory.importProjectsUpdateTable(projects, tBody, readOnly);
            };

            dialog.addButton(AJS.I18n.getText("attach.cat.project.import.dialog.save"), function () {
                var content = $('#attach-cat-dialog-content');
                var loader = $('#attach-cat-dialog-loader');
                loader.show();
                content.hide();

                JIRA.SmartAjax.makeRequest({
                    type: 'POST',
                    url: AJS.params.baseURL + '/rest/attach-cat/1.0/project/import',
                    data: {
                        projectKey: getProjectKey(),
                        newKey: select.val()
                    },
                    success: function (data) {
                        SmartAttachAnalytics.eventProjectImport();
                        projects = data;
                        tBody = $('#attach-categories');
                        S2Wrapper.loadSelectedRestrictions(select.val(), $(S2Wrapper.defSourceRestrictionsContainer));
                        updateTable();
                        dialog.remove();
                        JIRA.Messages.showMsg(AJS.I18n.getText("attach.cat.project.import.message.success"), {
                            type: JIRA.Messages.Types.SUCCESS,
                            closeable: true,
                            timeout: 3
                        });
                    },
                    error: function () {
                        loader.hide();
                        content.show();
                    }
                });
            }, "import-attach-cat-btn");

            dialog.addLink(AJS.I18n.getText("attach.cat.rename.dialog.cancel"), function (dialog) {
                dialog.remove();
            }, '#');

            if (projects.length) {
                S2Wrapper.loadSelectedRestrictions(select.val(), $(S2Wrapper.importSourceRestrictionsContainer));
                S2Wrapper.initSelect2ImportProject(select, updateTable);
                updateTable(true);
            } else {
                $('.import-attach-cat-btn').prop('disabled', true);
            }

            $('#attach-cat-import-message-container .icon-close').click(function (e) {
                $(e.target).parent().remove();
            });

            dialog.show();
            return false;
        },

        showHelp: function () {
            var helpDialog = AJS.InlineDialog($("#icon-help-dialog"), "smart-help-dialog",
                function (content, trigger, showPopup) {
                    content.css({"padding": "20px"}).html($('#icon-help-dialog-container').html());
                    showPopup();
                    return false;
                },
                {
                    offsetX: -10,
                    persistent: true
                }
            );

            JIRA.SmartAjax.makeRequest({
                type: 'GET',
                url: AJS.params.baseURL + '/rest/attach-cat/1.0/project/notification-help',
                success: function (data) {
                    if (data && $('#enable-smart-attach-checkbox').is(":checked")) {
                        helpDialog.show();
                    }
                }
            });

            $('body').on('click', '#smart-help-acknowledge', function () {
                helpDialog.hide();
            });
        },

        showFeatureDiscovery: function () {
            var dialog = new AJS.Dialog({
                width: 500,
                height: 170,
                id: "enable-fd-dialog",
                closeOnOutsideClick: false
            });

            dialog.addHeader(AJS.I18n.getText("attach.cat.project.fd.dialog.title"));
            dialog.addPanel('panel', "<div>" + AJS.I18n.getText("attach.cat.project.fd.dialog.body") + "</div>", 'enable-fd-dialog-panel');

            dialog.addButton(AJS.I18n.getText("attach.cat.action.yes"), function () {
                JIRA.SmartAjax.makeRequest({
                    type: 'POST',
                    url: AJS.params.baseURL + '/rest/attach-cat/1.0/project/enable-fd',
                    data: {
                        projectKey: getProjectKey(),
                        enable: true
                    },
                    success: function () {
                        dialog.remove();
                        SAProjectCategoryDialogs.showHelp();
                    }
                });
            }, "enable-fd");

            dialog.addButton(AJS.I18n.getText("attach.cat.action.no"), function () {
                JIRA.SmartAjax.makeRequest({
                    type: 'POST',
                    url: AJS.params.baseURL + '/rest/attach-cat/1.0/project/enable-fd',
                    data: {
                        projectKey: getProjectKey(),
                        enable: false
                    },
                    success: function () {
                        dialog.remove();
                        SAProjectCategoryDialogs.showHelp();
                    }
                });
            }, "disable-fd");

            dialog.addLink(AJS.I18n.getText("attach.cat.rename.dialog.cancel"), function (dialog) {
                dialog.remove();
                SAProjectCategoryDialogs.showHelp();
            }, '#');

            dialog.show();
        },

        showDisableConfirmation: function (toggleButton) {
            setTimeout(function () {
                toggleButton.prop('checked', true);
            }, 5);

            var dialog = new AJS.Dialog({
                width: 600,
                height: 220,
                id: "disable-sa-confirm-dialog",
                closeOnOutsideClick: false
            });

            dialog.addHeader(AJS.I18n.getText("attach.cat.project.disable-addon.dialog.title"));

            dialog.addPanel('panel', AttachCat.disableDialogBody(), 'disable-sa-confirm-dialog-panel');

            dialog.addButton(AJS.I18n.getText("attach.cat.action.yes"), function () {
                toggleButton.prop('checked', false);
                SAProjectCategory.disable(toggleButton);
                dialog.remove();
            }, "disable-sa");

            dialog.addButton(AJS.I18n.getText("attach.cat.action.no"), function () {
                dialog.remove();
            }, "leave-enabled-sa");

            dialog.show();
        }

    };

    function getProjectKey() {
        return $('meta[name=projectKey]').attr('content').trim();
    }

})(AJS.$);