define('smart-attachments/dialog/RemoveDialog',
    ['jquery', 'underscore', 'jira/lib/class', 'smart-attachments/base/Operations'],
    function ($, _, Class, Operations) {
        var CONFIG = {
            width: 600,
            height: 180,
            id: "delete-attach-document-dialog",
            closeOnOutsideClick: false
        };

        var RemoveDialog = Class.extend({
            templateDelete: AttachCategory.deleteDocumentDialog,

            init: function (opts) {
                _.extend(this, opts);
                this.render();
            },

            deleteAttachRequest: function () {
                return (this.isDocument)
                    ? Operations.deleteDocuments(this.issue.getIssueKey(), [this.attachmentId])
                    : Operations.deleteAttachment(this.attachmentId);
            },

            getTitleTexts: function () {
                var title = AJS.I18n.getText('attach.cat.delete.attachment.dialog.title', this.attachmentName);
                var text = AJS.I18n.getText('attach.cat.delete.attachment.dialog.body');
                if (this.isDocument) {
                    title = AJS.I18n.getText('attach.cat.delete.document.dialog.title', this.attachmentName);
                    text = AJS.I18n.getText('attach.cat.delete.document.dialog.body');
                } else if (this.isRevision) {
                    title = AJS.I18n.getText('attach.cat.delete.revision.dialog.title', this.attachmentName);
                    text = AJS.I18n.getText('attach.cat.delete.revision.dialog.body');
                }
                return {
                    title: title,
                    text: text
                }
            },

            render: function () {
                var dialog = new AJS.Dialog(CONFIG);
                var titles = this.getTitleTexts();

                dialog.addHeader(titles.title);
                dialog.addPanel('panel', this.templateDelete(titles), 'delete-attach-document-dialog-panel');

                dialog.addButton(AJS.I18n.getText("attach.cat.action.delete"), this.deleteClick.bind(this), "save-attach-rename");
                dialog.addLink(AJS.I18n.getText("attach.cat.rename.dialog.cancel"), function (dialog) {
                    dialog.remove();
                }, '#');

                this.dialog = dialog;
            },

            deleteClick: function () {
                this.deleteAttachRequest()
                    .then(function () {
                        this.dialog.remove();
                        this.trigger('remove:complete');
                    }.bind(this));
            },

            show: function () {
                this.dialog.show();
            }
        });

        return RemoveDialog;

    });