define('smart-attachments/dialog/RenameDialog',
    [
        'jquery',
        'underscore',
        'jira/lib/class',
        'smart-attachments/base/Operations'
    ], function ($, _, Class, Operations) {

    var notPermittedCharacters = ['#', '%', '&', '{', '}', '\\', '<', '>', '*', '?', '/', '$', '!', '‘', '“', ':', '@', '+', '`', '|', '='];
    var DIALOG_CONFIG = {
        width: 600,
        height: this.isDocument ? 180 : 230,
        id: "rename-attach-dialog",
        closeOnOutsideClick: false
    };

    var RenameDialog = Class.extend({
        init: function (opts) {
            _.extend(this, opts);
            this.render();
        },

        show: function () {
            this.dialog.show();
        },

        remove: function () {
            this.dialog.remove();
        },

        addErrorInlineDialog: function ($target, isEmpty) {
            if (this.errorInlineDialog) {
                this.errorInlineDialog.remove();
                delete this.errorInlineDialog;
            }
            var data = {
                symbols: '# % & { } \\ < > * ? / $ ! ‘ “ : @ + ` | =',
                isEmpty: isEmpty
            };
            return AJS.InlineDialog($target, $target.attr('id'),
                function (content, trigger, showPopup) {
                    content.html(AttachCategory.renameDialogError(data));
                    showPopup();
                    return false;
                }, {
                    offsetX: -10,
                    onHover: true,
                    gravity: 's'
                }
            );
        },

        render: function () {
            var dialog = new AJS.Dialog(DIALOG_CONFIG);
            var title = (this.isDocument
                ? AJS.I18n.getText("attach.cat.rename.document.dialog.title")
                : AJS.I18n.getText("attach.cat.rename.dialog.title"));

            dialog.addHeader(title);

            var body = AttachCategory.renameDialog({name: this.value, isDoc: this.isDocument});
            var $body = $(body);
            var $nameInput = $($body.find('#attach-new-name')[0]);
            var $iconOk = $body.find('#attach-rename-ok');
            var $iconError = $body.find('#attach-rename-error');

            dialog.addPanel('panel', $body, 'rename-attach-dialog-panel');

            dialog.addButton(
                AJS.I18n.getText("attach.cat.rename.dialog.save"),
                _.partial(this.onButtonSave.bind(this), _, $nameInput),
                "save-attach-rename");

            var $submitButton = dialog.getPage(0).buttonpanel.find('.save-attach-rename');

            dialog.addLink(AJS.I18n.getText("attach.cat.rename.dialog.cancel"), function (dialog) {
                dialog.remove();
            }, '#');

            this.dialog = dialog;

            $nameInput
                .on('input', this.fileNameValidate($nameInput, $submitButton, $iconOk, $iconError).bind(this))
                .on('keydown', function (e) {
                    if (e.keyCode === 13) {
                        if (!$submitButton.prop("disabled")) {
                            $submitButton.click();
                        }
                        return false;
                    }
                });
        },

        onButtonSave: function (event, $nameInput) {
            var value = $nameInput.val();
            var issueKey = this.issue.getIssueKey();
            Operations.renameAttachment(issueKey, this.categoryId, this.attachmentId, this.documentId, value)
                .then(function () {
                    this.remove();
                    this.trigger('rename:complete', [value]);
                }.bind(this));
        },

        fileNameValidate: function ($nameInput, $buttonSubmit, $iconOk, $iconError) {
            return function () {
                var value = $nameInput.val().trim();
                if (value.length !== 0) {
                    var error = _.some(notPermittedCharacters, function (item) {
                        return value.indexOf(item) > -1;
                    });
                    if (error) {
                        this.errorInlineDialog = this.addErrorInlineDialog($iconError, false);
                        $iconOk.hide();
                        $iconError.css('display', 'inline-block');
                        $nameInput.css('border-color', '#d04437');
                    } else {
                        if (this.errorInlineDialog) {
                            this.errorInlineDialog.remove();
                            delete this.errorInlineDialog;
                        }
                        $iconOk.css('display', 'inline-block');
                        $iconError.hide();
                        $nameInput.css('border-color', '#cccccc');
                    }
                    $buttonSubmit.prop("disabled", error);
                } else {
                    this.errorInlineDialog = this.addErrorInlineDialog($iconError, true);
                    $buttonSubmit.prop("disabled", true);
                    $iconOk.hide();
                    $iconError.css('display', 'inline-block');
                    $nameInput.css('border-color', '#d04437');
                }
            }
        }
    });

    return RenameDialog;
});