(function ($) {

    window.SADraggable = {

        DROPPABLE: 'droppable',

        draggableCategory: '',
        draggableAttachment: '',
        targetDropCategory: '',

        bind: function () {
            var categoriesContainer = $('#attach-cat-categories');
            this.bindCategories(categoriesContainer);
            this.bindAttachments(categoriesContainer);
            this.bindFiles();
        },

        bindCategories: function (categoriesContainer) {
            var t = this;
            var categoriesTitles = categoriesContainer.find('.attach-cat-title');
            var categories = categoriesContainer.find('.attach-cat-container');

            categoriesContainer.find('[draggable=true]').die("dragstart").removeAttr('draggable');
            categoriesTitles.attr('draggable', true);

            categories.unbind('dragover.cat').bind('dragover.cat', t.dragOverCategory);
            $(categoriesContainer).find(categoriesTitles)
                .off('dragstart.cat').on('dragstart.cat', t.dragStartCategory)
                .off('dragend.cat').on('dragend.cat', t.dragEndCategory);
        },

        bindFiles: function () {

            SADraggable.unbindOverlaySectionsEvents();

            var issueContainer = AttachCat.isAgile ? '#gh' : '.issue-container';
            var $issueContainer = $(issueContainer);

            var dragenterCounter = 0;
            $issueContainer.bind('dragover', function (e) {
                e.preventDefault();
            }).bind('dragenter', function (e) {
                e.preventDefault();
                dragenterCounter++;
                SADraggable.showOverlaySections(e);
            }).bind('dragleave', function () {
                dragenterCounter--;
                if (dragenterCounter === 0) {
                    SADraggable.hideOverlaySections();
                }
            });

            $('.wrap-categories-container').on('dragenter', function (e) {
                $('.cat-cell-container-hover').removeClass('cat-cell-container-hover');
                $(e.target).closest('.cat-cell-container').addClass('cat-cell-container-hover');
            });

            $('#attach-cat-content .attach-cat-container')
                .bind('dragover', function (e) {
                    e.preventDefault();
                    if (SADraggable.isDraggingFileAttachment(e) && !e.delegateTarget.classList.contains(SADraggable.DROPPABLE))
                        $(e.delegateTarget).addClass(SADraggable.DROPPABLE);
                })
                .bind('dragenter', function (e) {
                    e.preventDefault();
                    if (!SADraggable.isCategoryDragging() && SADraggable.isDraggingFileAttachment(e)) {
                        $(e.delegateTarget).addClass(SADraggable.DROPPABLE);
                    }
                }).bind('dragleave', function (e) {
                    $(e.delegateTarget).removeClass(SADraggable.DROPPABLE);
                });

            $('.attach-cat-container, .issue-container').on('drop.file dragdrop', function (e) {
                e.preventDefault();
                e.stopPropagation();
                e.stopImmediatePropagation();
                var $targetCategory = $(this);

                SADraggable.hideOverlaySections();

                if (SADraggable.isAttachmentDragging(e)) {
                    $targetCategory.removeClass(SADraggable.DROPPABLE);
                    SADraggable.targetDropCategory = $targetCategory.hasClass('attach-cat-container') ? $targetCategory : '';
                } else {
                    if (AttachCat.isServiceDeskAttachmentModuleExist()) {
                        return;
                    }

                    SADraggable.triggerEventDropHandled();
                    AttachCat.dropFileToCategory(e, $targetCategory);
                }
            });
        },

        bindAttachments: function (categoriesContainer) {
            var t = this;
            var attachments = categoriesContainer.find('.attachment-content');
            attachments.attr('draggable', true);
            attachments.off('dragstart.attach').on('dragstart.attach', t.dragStartAttachment);
            attachments.off('dragend.attach').on('dragend.attach', t.dragEndAttachment);
        },

        dragStartAttachment: function (e) {
            e.stopPropagation();
            var $target = $(e.target);
            var $attachment = $target.closest('.attachment-content');
            SADraggable.setDataTransferAttachmentDownloadURL(e, $attachment);

            if (!AttachCat.hasCreatePermission) {
                return false;
            }

            if (!$attachment.hasClass('revision')) {
                SADraggable.draggableAttachment = $attachment.get(0);
                SADraggable.setDataTransferAttach(e, SADraggable.draggableAttachment);
            }
        },

        dragEndAttachment: function (e) {
            e.stopPropagation();
            if (SADraggable.isAttachmentDragging() && SADraggable.targetDropCategory !== '') {
                var $attachment = $(SADraggable.draggableAttachment);
                var newCategory = $(SADraggable.targetDropCategory);
                var oldCategory = $attachment.parents('.attach-cat-container');
                var targetType = $attachment.parent().hasClass('attach-images') ? 'images' : 'files';

                AttachCat.move(oldCategory.attr('id'), newCategory.attr('id'), newCategory, oldCategory, $attachment, targetType);
            }
            SADraggable.draggableAttachment = '';
        },

        dragOverCategory: function (e) {
            if (SADraggable.isCategoryDragging()) {
                if (e.currentTarget !== SADraggable.draggableCategory) {
                    var $hoveredCategory = $(e.currentTarget);
                    var hoveredCategory = $hoveredCategory.get(0);
                    var yPositionOfHoveredCategoryArea = e.originalEvent.pageY - $hoveredCategory.offset().top;
                    var beforeElem = yPositionOfHoveredCategoryArea < $hoveredCategory.height() / 2 ? hoveredCategory : hoveredCategory.nextSibling;

                    if (beforeElem !== null) {
                        if (SADraggable.draggableCategory != beforeElem && SADraggable.draggableCategory != beforeElem.previousSibling){
                            hoveredCategory.parentNode.insertBefore(SADraggable.draggableCategory, beforeElem);
                        }
                    } else {
                        hoveredCategory.parentNode.appendChild(SADraggable.draggableCategory);
                    }
                }
            }
        },

        dragStartCategory: function (e) {
            e.stopPropagation();
            var $target = $(e.target);
            if (!$target.hasClass('attach-cat-title')) {
                return false;
            }
            SADraggable.draggableCategory = $target.closest('.attach-cat-container').get(0);
            SADraggable.setDataTransferCategory(e);
        },

        dragEndCategory: function (e) {
            e.stopPropagation();
            if (SADraggable.isCategoryDragging()) {
                JIRA.SmartAjax.makeRequest({
                    type: 'POST',
                    url: AJS.params.baseURL + '/rest/attach-cat/1.0/attachments/reorder',
                    data: {
                        key: AttachCat.getProjectKey(),
                        ids: AttachCat.getIdsPositionsOfCategories()
                    }
                });
            }
            SADraggable.draggableCategory = '';
            $(e.delegateTarget).closest('.' + SADraggable.DROPPABLE).removeClass(SADraggable.DROPPABLE);
        },

        unbindOverlaySectionsEvents: function () {
            if (AttachCat.isServiceDeskAttachmentModuleExist()) {
                return;
            }
            $(document.body).off('dragenter').off('dragleave');
            $('.issue-container').off('dragenter dragover drop').off('dragleave');
        },

        showOverlaySections: function (e) {
            if (document.body.classList.contains('attachments-drop-zone__dragover')){
                return;
            }

            if (!SADraggable.isAttachmentDragging() && e.originalEvent.dataTransfer && $.inArray("Files", e.originalEvent.dataTransfer.types) !== -1) {
                $(document.body)
                    .addClass('attachments-drop-zone__dragover')
                    .addClass("attachments-drop-zone__window-dragover");
                //$('.issue-container').addClass("attachments-drop-zone__dragover-fade");
            }
        },

        hideOverlaySections: function () {
            $(document.body)
                .removeClass("attachments-drop-zone__dragover-fade")
                .removeClass("attachments-drop-zone__window-dragover")
                .removeClass("issue-drop-zone-document__dragover")
                .removeClass("attachments-drop-zone__dragover");
        },

        setDataTransferAttach: function (e, draggableItem) {
            SADraggable.setDataTransfer(e);
            if (e.originalEvent.dataTransfer && 'function' === typeof DataTransfer.prototype.setDragImage) {
                var dragPosition = SADraggable.getDragPosition(e);
                e.originalEvent.dataTransfer.setDragImage(SADraggable.createAttachDragImage($(draggableItem)), dragPosition.x, dragPosition.y);
            }
        },

        setDataTransferCategory: function (e) {
            SADraggable.setDataTransfer(e);
            if (e.originalEvent.dataTransfer && 'function' === typeof DataTransfer.prototype.setDragImage) {
                var dragPosition = SADraggable.getDragPosition(e);
                e.originalEvent.dataTransfer.setDragImage(SADraggable.draggableCategory, dragPosition.x, dragPosition.y);
            }
        },

        setDataTransfer: function (e) {
            if (e.originalEvent.dataTransfer) {
                e.originalEvent.dataTransfer.effectAllowed = 'move';
                e.originalEvent.dataTransfer.setData('text', '');
            }
        },

        setDataTransferAttachmentDownloadURL: function (e, $attachment) {
            if (e.originalEvent.dataTransfer) {
                var downloadUrl = $attachment.data('downloadurl') ? $attachment.data('downloadurl') :
                    $attachment.find('a[data-downloadurl]').data('downloadurl');
                if (downloadUrl && !SADraggable.isMicrosoftBrowser()) {
                    e.originalEvent.dataTransfer.setData("DownloadURL", downloadUrl);
                }
            }
        },

        createAttachDragImage: function ($attachment) {
            var container = $($attachment.parent().get(0).cloneNode(false));
            $(document.body).append(container);

            var $img = $($attachment.clone());
            $img.find('.attachment-delete').remove();
            $img.addClass('draggable-ghost');
            $img.css("width", $attachment.width()).appendTo($(container).get(0));

            setTimeout(function () {
                container.remove();
            });
            return $img[0];
        },

        getDragPosition: function (e) {
            var offset = $(e.currentTarget).offset();

            return {
                x: e.originalEvent.pageX - offset.left,
                y: e.originalEvent.pageY - offset.top
            }
        },

        isDraggingRealFile: function (e) {
            return e.originalEvent
                && e.originalEvent.dataTransfer
                && e.originalEvent.dataTransfer.types
                && $.inArray('File', e.originalEvent.dataTransfer.types) !== -1;
        },

        isAttachmentDragging: function () {
            return SADraggable.draggableAttachment !== '';
        },

        isDraggingFileAttachment: function (e) {
            return SADraggable.isDraggingRealFile(e) ? true : SADraggable.isAttachmentDragging();
        },

        isCategoryDragging: function () {
            return SADraggable.draggableCategory !== '';
        },

        triggerEventDropHandled: function () {
            $(document).trigger($.Event('dropHandled.dropFileToElement'));
        },

        isMicrosoftBrowser: function () {
            return (navigator.userAgent && navigator.userAgent.indexOf("MSIE") > -1)
                || (navigator.userAgent && navigator.userAgent.indexOf("Edge") > -1)
                || !!document.documentMode === true;
        }
    }

})(AJS.$);