define('smart-attachments/dropzone/AttachmentsDropZone', [
    'underscore',
    'jquery',
    'jira/util/events',
    'smart-attachments/base/AutoDistribution',
    'smart-attachments/toolbox',
    'smart-attachments/dropzone/CategoryZones',
    'dndattachment/dropzones/AttachmentsDropZone',
    'exports'
], function (_, $, Events, AutoDistribution, Toolbox, CategoryZones, AttachmentsDropZone, exports) {

    var connectContainer = AttachmentsDropZone.prototype.connectContainer;
    var handleFilesReceived = AttachmentsDropZone.prototype.handleFilesReceived;
    var onDragOverDrop = AttachmentsDropZone.prototype.onDragOverDrop;
    var onDragLeave = AttachmentsDropZone.prototype.onDragLeave;
    var removeAllDragoverClasses = AttachmentsDropZone.prototype.removeAllDragoverClasses;
    var placeUploadProgressBar = AttachmentsDropZone.prototype.placeUploadProgressBar;
    var attachFile = AttachmentsDropZone.prototype.attachFile;
    var handleNewProgressBar = AttachmentsDropZone.prototype.handleNewProgressBar;
    exports.connect = function () {
        _.extend(AttachmentsDropZone.prototype, {
            connectContainer: function () {
                this.issue = require('jira/issue');
                connectContainer.apply(this, arguments);
                this.initDeferred = $.Deferred();
                this.isServiceDesk = Toolbox.isSDProject();
                this.isAgile = Toolbox.isAgile();
                var finishFunction = _.partial(this.finishInitialize.bind(this), arguments);
                Toolbox.fetchCategories(this.issue.getIssueId()).then(finishFunction);

                this.onPanelRefreshed = this.reDrawDropZones.bind(this);
                Events.unbind('dropzone:sa.rerender');
                Events.bind('dropzone:sa.rerender', this.onPanelRefreshed);
            },

            finishInitialize: function (originalArguments, categoryConfig) {
                this.categoryConfig = categoryConfig;
                if (this.isRequireCategories(categoryConfig)) {
                    this.attachListeners();
                } else {
                    this.disconnectContainer();
                    this.restorePrototypeMethods();
                    connectContainer.apply(this, originalArguments);
                }
                this.initDeferred.resolve(categoryConfig);
            },

            isRequireCategories: function (config) {
                return config && config.categories;
            },

            attachListeners: function () {
                console.log('AttachmentsDropZone customized');
                this.$body.off('dragleave dragenter');
                this.progressBars = [];
                this.createCategoryZones();
            },

            restorePrototypeMethods: function () {
                this.handleFilesReceived = handleFilesReceived;
                this.onDragLeave = onDragLeave;
                this.onDragOverDrop = onDragOverDrop;
                this.removeAllDragoverClasses = removeAllDragoverClasses;
                this.connectContainer = connectContainer;
                this.placeUploadProgressBar = placeUploadProgressBar;
                this.attachFile = attachFile;
                this.handleNewProgressBar = handleNewProgressBar;
            },

            reDrawDropZones: function (event, data) {
                if (!this.categoryConfig) {
                    return;
                }

                Toolbox.fetchCategories(this.issue.getIssueId()).then(function (categoryConfig) {
                    this.categoryConfig = categoryConfig;
                    this.attachListeners();
                }.bind(this));
            },

            createCategoryZones: function () {
                var categoriesForRender = this.getCategoriesForRender();
                this.categoryZones = new CategoryZones(this.$dragoverBorder[0], categoriesForRender);
                this.categoryZones.bind('dropToCategory', this.onDropToCategory.bind(this));
                this.categoryZones.bind('leaveCategories', this.realDragLeave.bind(this));
                this.categoryZones.bind('changeCategory', this.onChangeActiveCategory.bind(this));
            },

            getCategoriesForRender: function () {
                if (this.categoryConfig.categories.length <= 1) {
                    return [];
                }

                /*if (!this.isServiceDesk && !this.categoryConfig.dropZones) {
                    return [];
                }*/

                var categoriesForRender = this.categoryConfig.categories.slice();
                var allowAutoDistirbution = AutoDistribution.isNeedToShowAutoDistributionZone(categoriesForRender);
                if (this.isServiceDesk && !this.categoryConfig.dropZones) {
                    allowAutoDistirbution = false;
                }
                if (allowAutoDistirbution) {
                    categoriesForRender.unshift(AutoDistribution.getAutoDistributionCategoryTemplate());
                }
                return categoriesForRender;
            },

            realDragLeave: function () {
                onDragLeave.call(this);
            },

            onChangeActiveCategory: function (event, categoryId) {
                console.log('onChangeActiveCategory: ' + categoryId);
                this.categoryMatcher = AutoDistribution.createMatcher(categoryId, this.categoryConfig.categories);
            },

            onDropToCategory: function (event, data) {
                var eventName = 'attachmentDnDToIssue';
                if (this.isAgile) {
                    eventName = 'agileBoardDnDAttachment';
                } else if (Toolbox.isIssueNavigator()) {
                    eventName = 'issueNavigatorDnDAttachment';
                } else if (Toolbox.isSearchForIssues()) {
                    eventName = 'searchForIssuesAttachmentDnD';
                } else if (Toolbox.isServiceDeskQueues()) {
                    eventName = 'queuesDnDAttachment';
                }
                SmartAttachmentsAnalytics.event(eventName, [data.categoryId, data.attachmentsCount]);
                this.realDragLeave();
            },

            onDragLeave: function (event) {
                event.stopPropagation();
                event.preventDefault();
            },

            removeAllDragoverClasses: function () {
                removeAllDragoverClasses.apply(this, arguments);
                this.$dragoverBorder.removeClass('hidden');
            },

            onDragOverDrop: function (event) {
                if (event.type === 'drop') {
                    if (event.target.nodeName === 'TEXTAREA') {
                        this.$dragoverBorder.addClass('hidden');
                        this.categoryMatcher = AutoDistribution.createMatcher('auto', this.categoryConfig.categories)
                    }
                }
                $(".issue-container").off('dragleave dragenter');
                onDragOverDrop.apply(this, arguments);
            },

            handleFilesReceived: function (files) {
                _.each(files, function (file) {
                    file.categoryId = this.categoryMatcher.match(file);
                    file.matched = true;
                }, this);

                var args = arguments;
                this.initDeferred.then((function () {
                    handleFilesReceived.apply(this, args);
                }).bind(this));
            },

            handleNewProgressBar: function (progressBar) {
                this.progressBars.push(progressBar);
            },

            placeUploadProgressBar: function ($progressBar, progressBar) {
                placeUploadProgressBar.apply(this, arguments);
                var categoryId;
                if (progressBar.file.matched) {
                    console.log('Use category from file');
                    categoryId = progressBar.file.categoryId;
                } else {
                    console.log('Use category from matcher');
                    categoryId = this.categoryMatcher.match(progressBar.file);
                }
                Events.trigger("progressBar:add", [progressBar, categoryId]);
            },

            attachFile: function (fileIDs) {
                var result = attachFile.apply(this, arguments);
                result.then(function (beans) {
                    _.each(fileIDs, function (fileID, idx) {
                        var progressBar = _.find(this.progressBars, function (progressBar) {
                            return progressBar.getFileID() === fileID;
                        });
                        if (progressBar) {
                            console.log('Set fileID:' + progressBar.getFileID() + ' to attachmentId: ' + beans[idx].id);
                            progressBar.attachmentId = beans[idx].id
                        }
                    }, this);
                }.bind(this));
                return result;
            }
        });
    };
    //return AttachmentsDropZone;
});