define('smart-attachments/dropzone/CategoryDropZone',
    [
        'jquery',
        'underscore',
        'jira/lib/class',
        'smart-attachments/base/CommonClasses',
        'smart-attachments/base/DropZoneSection',
        'smart-attachments/base/AutoDistribution',
        'smart-attachments/base/DNDHelper',
        'smart-attachments/dropzone/ExtendedFormDropZone',
        'dndattachment/util/DataTransfer'
    ],
    function ($, _, Class, CommonClasses, DropZoneSection, AutoDistribution, DNDHelper, ExtendedFormDropZone, DataTransfer) {

        var CategoryFormDropZone = Class.extend({
            init: function (categories, $form, dropzone) {
                this.uploadLimit = 0;
                this.uploadSize = 0;
                this.renderedCategories = [];
                this.dropHandler = this.onDrop.bind(this);
                dropzone.disconnectContainer();
                this.connectListener($form);
                var $formFieldGroup = dropzone.$node.parent();
                $formFieldGroup.hide();
                var initializedDropZone = $formFieldGroup.find('.issue-drop-zone');
                if (initializedDropZone.length > 1) {
                    initializedDropZone = $(initializedDropZone.get(0));
                }

                var jiraAttachContainer = initializedDropZone.parent();

                this.createDragoverMask(categories, $formFieldGroup);
                this.uploadLimit = dropzone.uploadLimit;
                this.uploadSize = dropzone.uploadSize;
                var dropZoneTemplate = this.getDropZoneTemplate();
                var ourContainer = this.createNewContainer(jiraAttachContainer);
                var customDropZones = categories.map(function (category) {
                    var attachDropZone = $(dropZoneTemplate);
                    attachDropZone.attr('data-category-id', category.id).attr('data-category-name', category.name);
                    ourContainer.append(attachDropZone);
                    var newSection = ourContainer.find('[data-category-id=' + category.id + "]");
                    var extendedDropZone = new ExtendedFormDropZone(newSection, category);
                    extendedDropZone.bind('progressBar:move', this.onProgressBarMove.bind(this));
                    this.customizeDropZone(ourContainer);
                    return new CommonClasses.CustomDropZone(extendedDropZone, newSection, category);
                }, this);

                this.renderedCategories = categories;
                this.customDropZones = customDropZones;
                $form.prop('_decoratedDropZone', this);
            },

            connectListener: function ($form) {
                this.$containerNode = $form.parent();
                this.$containerNode.on("drop", this.dropHandler);
                $form.prepend('<div class="issue-drop-zone__target" style="width: 0; height: 0" data-wiki-capture="true"></div>');
                $form.find('.issue-drop-zone__target').on("drop", this.onDropOnWiki.bind(this));
            },

            onDropOnWiki: function (event) {
                if (!DNDHelper.dragEventContainsFiles(event)) {
                    return;
                }
                if (event.type === 'drop') {
                    var eventDataTransfer = DNDHelper.getDataTransferFromEvent(event);
                    if (!eventDataTransfer) {
                        return;
                    }
                    if ((event.target.dataset['wikiCapture'] === 'true')) {
                        var categories = this.renderedCategories;
                        var matcher = AutoDistribution.createMatcher('auto', categories);
                        var filesByCategroies = _.reduce(eventDataTransfer.files, function (result, file) {
                            var categoryId = matcher.match(file);
                            file["predefinedCategoryId"] = categoryId;
                            var files = result[categoryId];
                            if (!files) {
                                files = [];
                                result[categoryId] = files;
                            }
                            files.push(file);
                            return result;
                        }, []);
                        event.preventDefault();
                        event.stopPropagation();
                        event.stopImmediatePropagation();
                        _.each(filesByCategroies, function (files, categoryId) {
                            var dropEvent = $.Event("drop");
                            dropEvent.dataTransfer = {files: files};
                            var dropzone = this.getDropZoneByCategoryId(categoryId);
                            if (dropzone) {
                                dropzone.onDragOverDrop(dropEvent);
                            }
                        }, this);
                    }
                }
            },

            onDrop: function (event) {
                if (!DNDHelper.dragEventContainsFiles(event)) {
                    return;
                }
                if (event.type === 'drop') {
                    var eventDataTransfer = DNDHelper.getDataTransferFromEvent(event);
                    if (!eventDataTransfer) {
                        return;
                    }
                    if (DNDHelper.isEventTargetFormOrWikiTextField(event)) {
                        var categories = this.renderedCategories;
                        if (AutoDistribution.hasCategoryAutoDistributionRule(categories)) {
                            _.each(eventDataTransfer.files, function (file) {
                                file["predefinedCategoryId"] = AutoDistribution.matchByRuleCategoryId(categories, file);
                            });
                        }
                    }
                }
            },

            getProgressBars: function () {
                return this.customDropZones.reduce(function (result, customDropZone) {
                    return result.concat(customDropZone.dropZone.progressBars);
                }, []);
            },

            onProgressBarMove: function (event, fileId, sourceCategoryId, targetCategoryId) {
                var source = this.getDropZoneByCategoryId(sourceCategoryId);
                var target = this.getDropZoneByCategoryId(targetCategoryId);
                if (source && target) {
                    var progressBar = source.releaseProgressBarWithFileId(fileId);
                    if (progressBar) {
                        target.appendProgressBar(progressBar);
                    }
                }
            },

            getDefaultOrFirstCategoryDropzone: function () {
                var customDropZone = _.find(this.customDropZones, function (wrapper) {
                    return wrapper.isDefaultCategory();
                });
                if (customDropZone) {
                    return customDropZone.dropZone;
                } else {
                    return this.customDropZones[0].dropZone;
                }
            },

            getDropZoneByCategoryId: function (categroyId) {
                var wrapper = _.find(this.customDropZones, function (wrapper) {
                    return wrapper.getCategoryId() === categroyId;
                });
                if (wrapper) {
                    return wrapper.dropZone;
                } else {
                    this.getDefaultOrFirstCategoryDropzone();
                }
            },

            createDragoverMask: function (categories, $formFieldGroup) {
                var masks = $formFieldGroup.parents('form').parent();
                masks.find('.custom-dragover-mask').remove();
                var mask = $(AttachCatDialog.dragoverMask());
                masks.append(mask);
                mask = masks.find('.custom-dragover-mask');
                mask.append(AttachCatDialog.container());
                var catContainer = mask.find('#categories-container');

                var showAutoDistributionZone = AutoDistribution.isNeedToShowAutoDistributionZone(categories);

                var categoriesForRender = categories.slice(0);
                if (showAutoDistributionZone) {
                    categoriesForRender.unshift(AutoDistribution.getAutoDistributionCategoryTemplate());
                }

                DropZoneSection.createSections(mask, catContainer, categoriesForRender, AttachCatDialog.categoryInDnD);

                if (showAutoDistributionZone) {
                    AutoDistribution.bindAutoDistributionDragOverZoneForForms(categories, catContainer);
                }
            },

            getDropZoneTemplate: function () {
                return AttachCatDialog.attachmentDropZone({
                    uploadLimit: this.uploadLimit,
                    uploadSize: this.uploadSize
                });
            },

            createNewContainer: function (container) {
                var classes = container.attr('class');
                var newContainer = $(AttachCatDialog.fieldGroupAttachmentDropZone({classes: classes}));
                newContainer.insertAfter(container);
                return newContainer;
            },

            customizeDropZone: function ($container) {
                var dropZone = this.getNotCustomizedDropZone($container);
                dropZone.addClass('customized-drop-zone');
                var categoryTitle = dropZone.data('categoryName');
                var template = AttachCatDialog.dropZoneCategoryTitle({categoryTitle: categoryTitle});
                dropZone.prepend(template);
            },

            getNotCustomizedDropZone: function ($container) {
                return $container.find('.issue-drop-zone').not('.customized-drop-zone');
            },

            getTempAttachments: function () {
                return this.customDropZones.reduce(function (result, descriptor) {
                    var files = descriptor.dropZone.progressBars.map(function (progressBar) {
                        return new CommonClasses.TempAttachment(progressBar.getFileName(), progressBar.category.id);
                    });
                    return result.concat(files);
                }, []);
            },

            getFiles: function () {
                return this.customDropZones.reduce(function (result, descriptor) {
                    var files = descriptor.dropZone.progressBars.map(function (progressBar) {
                        return {name: progressBar.getFileName(), categoryId: progressBar.category.id};
                    });
                    return result.concat(files);
                }, []);
            },

            getJiraTempAttachments: function () {
                return this.customDropZones.reduce(function (result, descriptor) {
                    var files = descriptor.dropZone.progressBars.map(function (progressBar) {
                        return new CommonClasses.JiraTempAttachment(progressBar.getFileID(), progressBar.category.id);
                    });
                    return result.concat(files);
                }, []);
            }
        });

        return CategoryFormDropZone;
    });