define('smart-attachments/dropzone/CategoryZones',
    [
        'jquery',
        'jira/lib/class'
    ],
    function ($, Class) {
        var dropZoneItemTemplate = SmartAttachment.Template.dropZoneItem;
        var DEFAULT_ZONE = {
            id: 'auto',
            name: void 0,
            position: 0
        };

        var CategoryZones = Class.extend({
            init: function (element, categories) {
                this.borderItems = [];
                this.$node = $(element);
                this.categories = categories;
                if (categories.length === 0) {
                    categories.push(DEFAULT_ZONE);
                }
                this.render();
            },

            createDropZoneItem: function (category, width, height) {
                var $dropZoneItem = $(dropZoneItemTemplate({
                    category: category,
                    width: width,
                    height: height
                }));

                $dropZoneItem.on('dragenter dragover', this.onDragOver.bind(this));
                $dropZoneItem.on('dragleave', _.debounce(this.onDragLeave.bind(this), 50));
                $dropZoneItem.on('drop', (function (event) {
                    var target = event.target;
                    var $target = $(target);
                    if (target.tagName !== 'LI') {
                        $target = $target.closest('li');
                    }
                    this.trigger("dropToCategory", $target.data('categoryId'));
                }).bind(this));
                return $dropZoneItem;
            },

            render: function () {
                var $container = $('<div class="zone-container"></div>');
                this.$node.append($container);
                var MARGIN = 10;
                var containerWidth = $container.width() - MARGIN;
                var containerHeight = $container.height() - MARGIN;
                var count = this.categories.length;
                var columns = Math.ceil(Math.sqrt(count));
                var rows = Math.ceil(count / columns);
                var heightItem = ((containerHeight / rows) * 100) / containerHeight;

                var categoriesContainer = $('<ul id="categories-drop-zones-container"></ul>');
                var sum = 0;

                for (var i = 0; i < rows; i++) {
                    var amountInRow = Math.ceil((count - sum) / (rows - i));
                    var widthItem = ((containerWidth / amountInRow) * 100) / containerWidth;

                    for (var j = 0; j < amountInRow; j++) {
                        var $dropZoneItem = this.createDropZoneItem(this.categories[sum + j], widthItem, heightItem);
                        this.borderItems.push($dropZoneItem.find('div.category-drop-zone-border')[0]);
                        categoriesContainer.append($dropZoneItem);
                    }
                    sum += amountInRow;
                }

                $container.append(categoriesContainer);
                return categoriesContainer;
            },

            decorateCategoryZone: function (target) {
                var activeItem = target.parentElement.querySelector('.category-drop-zone-border');
                _.each(this.borderItems, function (item) {
                    if (activeItem !== item) {
                        item.classList.remove('drag-hovered');
                    }
                });

                if (!activeItem.classList.contains('drag-hovered')) {
                    activeItem.classList.add('drag-hovered');
                    this.trigger('changeCategory', activeItem.parentElement.dataset['categoryId']);
                }
            },

            onDragOver: function (event) {
                event.stopPropagation();
                event.preventDefault();
                if (this.leaveDropZone) {
                    clearTimeout(this.leaveDropZone);
                }
                _.throttle(this.decorateCategoryZone(event.target), 50);
            },

            onDragLeave: function (event) {
                event.stopPropagation();
                event.preventDefault();
                this.leaveDropZone = setTimeout((function () {
                    this.trigger("leaveCategories")
                }).bind(this), 500);
            },

            getSelectedCategoryId: function () {
                return this.$node.find('.drag-hovered').closest('li').data('categoryId');
            }
        });
        return CategoryZones;
    });