define('smart-attachments/dropzone/ExtendedFormDropZone',
    [
        'jquery',
        'underscore',
        'jira/util/events',
        'smart-attachments/base/DNDHelper',
        'smart-attachments/base/AutoDistribution',
        'dndattachment/util/DataTransfer',
        'dndattachment/dropzones/FormDropZone'
    ], function ($, _, Events, DNDHelper, AutoDistribution, DataTransfer, FormDropZone) {

        var handleNewProgressBar = FormDropZone.prototype.handleNewProgressBar;

        function stopEventProcessing(event) {
            event.preventDefault();
            event.stopPropagation();
            event.stopImmediatePropagation();
        }

        var ExtendedFormDropZone = FormDropZone.extend({
            init: function (element, category) {
                this.category = category;
                this._super.apply(this, arguments);
            },

            placeUploadProgressBar: function ($progressBar) {
                var file = $progressBar.data('file');

                this.addAttachmentNameToTitleForProgressBar($progressBar);
                //todo возможно эта проверка уже не нужна
                if (file && file.predefinedCategoryId !== undefined) {
                    var $categoryDropZone = this.getCategoryDropZoneById(this.$containerNode, file.predefinedCategoryId);
                    if ($categoryDropZone.length) {
                        return $categoryDropZone.append($progressBar);
                    }
                }
                return this.$node.append($progressBar);
            },

            appendProgressBar: function (progressBar) {
                this.handleNewProgressBar(progressBar);
                progressBar.connectListeners();
                this.$node.append(progressBar.$node);
                this.trigger('uploadFinished', progressBar);
            },

            connectContainer: function () {
                var categoryId = this.$node.data('categoryId');
                if (categoryId === undefined) {
                    return;
                }
                this.$containerNode = this.$node.parents('form').parent();
                this.$dragoverMask = this.$containerNode.find('.custom-dragover-mask li[data-category-id=' + categoryId + ']');
                this.$body = $(document.body);

                this.onDragOverDropHandler = this.onDragOverDrop.bind(this);
                this.onDragLeaveHandler = this.onDragLeave.bind(this);
                this.onBodyDragOverDropHandler = this.onBodyDragOverDrop.bind(this);
                this.$containerNode.on('dragover drop', this.onDragOverDropHandler);
                this.$containerNode.on('dragleave', this.onDragLeaveHandler);
                this.$body.on('dragover drop', this.onBodyDragOverDropHandler);

                this.onDragOverMaskItemHandler = this.onDragOverMaskItem.bind(this);
                this.$dragoverMask.on('drop', this.onDragOverDropHandler);
                this.$dragoverMask.on('dragover', this.onDragOverMaskItemHandler);

                this.onBeforeSubmitHandler = this.onBeforeSubmit.bind(this);
                this.$formNode.on('before-submit', this.onBeforeSubmitHandler);

                this.onContentAddedHandler = this.onContentAdded.bind(this);
                JIRA.bind(JIRA.Events.NEW_CONTENT_ADDED, this.onContentAddedHandler);
                Events.bind("Dialog.hide", this.onDialogHide.bind(this));
                if (AJS.dialog2) {
                    AJS.dialog2.on("hide", this.onDialogHide.bind(this));
                }
                this.bind('uploadFinished', this.onDropZoneAttachmentUploadFinished);
                this.bind('progressBarDestroyed', this.onDestroyProgressBar.bind(this));
                var masks = this.$node.find(' >*');
                masks.on('drop.sa-move', this.onProgressBarDrop.bind(this));
            },

            hasNotSamePredefinedCategory: function (eventDataTransfer) {
                var currentCatrogyrId = this.category.id;
                return _.some(eventDataTransfer.files, function (file) {
                    return file.predefinedCategoryId && file.predefinedCategoryId !== currentCatrogyrId;
                })
            },

            onProgressBarDrop: function (e) {
                if (!e.originalEvent) return;
                var infoText = e.originalEvent.dataTransfer.getData('text');

                if (infoText) {
                    var info = JSON.parse(infoText);
                    if (info.fileId) {
                        var fileId = info.fileId;
                        console.log('move progress bar with file: ' + fileId + ' from ' + info.categoryId + ' to ' + this.category.id);
                        this.trigger('progressBar:move', [fileId, info.categoryId, this.category.id]);
                    }
                }
            },

            onDragOverDrop: function (event) {
                if (!this.isAttached()) {
                    this.disconnectContainer();
                    return;
                }
                if (!DNDHelper.dragEventContainsFiles(event)) {
                    return;
                }

                if (event.originalEvent) {
                    event.originalEvent.dataTransfer.dropEffect = "copy";
                }

                this.$containerNode.addClass('attach-files-drop-zone__dragover');

                if (event.type === 'drop') {
                    this.$containerNode.removeClass('attach-files-drop-zone__dragover');
                    this.$containerNode.find('.cat-cell-container-hover').removeClass('.cat-cell-container-hover');
                    $(document).trigger("dropHandled");
                    var eventDataTransfer = DNDHelper.getDataTransferFromEvent(event);
                    if (eventDataTransfer) {
                        if (this.hasNotSamePredefinedCategory(eventDataTransfer)) {
                            return;
                        }
                        var dataTransfer = new DataTransfer(eventDataTransfer);
                        dataTransfer.getFiles().then(function (files) {
                            this.handleFilesReceived(files);
                            this.queueEvent('fileDrop', {count: files.length});
                        }.bind(this));
                    }
                }
                stopEventProcessing(event);
            },

            onDragOverMaskItem: function (event) {
                this.highlightDragoveredDropZone(this.$containerNode, $(event.delegateTarget));
            },

            getProgressBarByFileId: function (fileId) {
                return _.find(this.progressBars, function (progressBar) {
                    return progressBar.getFileID() === fileId;
                })
            },

            releaseProgressBarWithFileId: function (fileId) {
                var progressBar = this.getProgressBarByFileId(fileId);
                if (progressBar) {
                    this.progressBars = _.filter(this.progressBars, function (item) {
                        return item.getFileID() !== fileId;
                    });
                    var $instance = $(progressBar);
                    $instance.off('onBeforeDestroy');
                    $instance.off('onDestroy');
                    $instance.off('onFinished');
                    $instance.off('onFailed');
                    this.$node.find('input[value="' + fileId + '"]').remove();
                    return progressBar;
                }
            },

            highlightDragoveredDropZone: function ($containerNode, $eventTarget) {
                $containerNode.find('.cat-cell-container-hover').removeClass('cat-cell-container-hover');
                $eventTarget.find('.cat-cell-container').addClass('cat-cell-container-hover');
            },

            getCategoryDropZoneById: function ($containerNode, categoryId) {
                return $containerNode.find(".customized-drop-zone[data-category-id='" + categoryId + "']");
            },

            onDropZoneAttachmentUploadFinished: function () {
                var wikiAttachmentButton = $('.wiki-edit-attachment-picker-trigger');
                var hasHiddenClass = wikiAttachmentButton.hasClass('hidden');
                if (hasHiddenClass) {
                    wikiAttachmentButton.data('saIsHidden', wikiAttachmentButton.hasClass('hidden'));
                }
                wikiAttachmentButton.removeClass('hidden');
            },

            onDestroyProgressBar: function (event) {
                if (this.progressBars.length === 0) {
                    var wikiAttachmentButton = $('.wiki-edit-attachment-picker-trigger');
                    var hasHiddenClass = wikiAttachmentButton.data('saIsHidden');
                    if (hasHiddenClass) {
                        wikiAttachmentButton.addClass('hidden');
                    }
                }
            },

            decorateProgressBar: function (progressBar) {
                progressBar.$node.attr('draggable', true);
                var $img = progressBar.$node.find('img');
                $img.attr('draggable', false);
                if ($img.length === 0) {
                    var stubImg = $('<img class="hidden"/>');
                    progressBar.$node.append(stubImg);
                }
                progressBar.$node.off('dragstart.sa-move');
                progressBar.$node.on('dragstart.sa-move', function (e) {
                    var info = {
                        fileId: progressBar.getFileID(),
                        categoryId: progressBar.category.id
                    };
                    var dragPosition = DNDHelper.getDragPosition(progressBar.$node, e);
                    e.originalEvent.dataTransfer.setDragImage(DNDHelper.createDragImage(progressBar.$node), dragPosition.x, dragPosition.y);
                    e.originalEvent.dataTransfer.setData('text', JSON.stringify(info));
                });
            },

            handleNewProgressBar: function (progressBar) {
                progressBar.category = this.category;
                if (progressBar.finished) {
                    this.decorateProgressBar(progressBar);
                } else {
                    progressBar.bind('onFinished', _.partial(this.decorateProgressBar.bind(this), progressBar));
                }
                handleNewProgressBar.apply(this, arguments);
            },

            addAttachmentNameToTitleForProgressBar: function (progressBar) {
                var fileNameItem = $(progressBar).find('.upload-progress-bar__file-name');
                var fileName = fileNameItem.text();
                fileNameItem.attr('title', fileName);
            }
        });

        return ExtendedFormDropZone;
    });