define('smart-attachments/dropzone/FieldTypeDropZone',
    [
        'jquery',
        'underscore',
        'jira/lib/class',
        'smart-attachments/base/CommonClasses'
    ],
    function ($, _, Class, CommonClasses) {
        function isDefault(category) {
            return category.defaultCategory;
        }

        function isEqualCategoryId(categoryId) {
            return function (category) {
                return category.id === categoryId;
            };
        }

        var FieldTypeDropZone = Class.extend({
            init: function (categories, $form, dropzone, lastCategoryId) {
                this.categories = categories;
                dropzone.$node.show();
                var $formFieldGroup = dropzone.$node.parent();
                if (categories.length) {
                    this.initDropdownCategories(categories, lastCategoryId, $form, $formFieldGroup);
                } else {
                    this.getSelectedCategoryId = function () {
                        return lastCategoryId;
                    }
                }
                this.dropzone = dropzone;
                $form.prop('_decoratedDropZone', this);
            },

            initDropdownCategories: function (categories, lastCategoryId, $form, $fileInputList) {
                var selectedCategory = this.getCategoryByIdOrDefault(lastCategoryId);
                var selectedCategoryId = selectedCategory.id;
                var $selectBlock = $(AttachCatDialog.catSelect({
                    categories: categories,
                    defaultId: selectedCategoryId
                }));
                $fileInputList.parent().before($selectBlock);
                this.categorySelect = $selectBlock.find('select');
                this.categorySelect.val(selectedCategoryId);
            },

            getCategoryByIdOrDefault: function (categoryId) {
                var result = null;
                if (categoryId) {
                    result = _.find(this.categories, isEqualCategoryId(categoryId))
                }
                if (!result) return this.getDefaultOrFirstCategory();
                return result;
            },

            getDefaultOrFirstCategory: function () {
                var result = _.find(this.categories, isDefault);
                if (!result) {
                    result = this.categories[0];
                }
                return result;
            },

            getDefaultOrFirstCategoryDropzone: function () {
                this.dropzone.category = this.getDefaultOrFirstCategory();
                return this.dropzone;
            },

            getProgressBars: function () {
                return this.dropzone.progressBars;
            },

            appendProgressBar: function (progressBar) {

            },

            getSelectedCategoryId: function () {
                if (this.categorySelect) return parseInt(this.categorySelect.val());
                return 0;
            },

            getTempAttachments: function () {
                var selectedCategoryId = this.getSelectedCategoryId();
                if (selectedCategoryId) {
                    return this.dropzone.progressBars.map(function (progressBar) {
                        return new CommonClasses.TempAttachment(progressBar.getFileName(), selectedCategoryId);
                    });
                }
                return [];
            },

            getFiles: function () {
                var selectedCategoryId = this.getSelectedCategoryId();
                if (selectedCategoryId) {
                    return this.dropzone.progressBars.map(function (progressBar) {
                        return {
                            name: progressBar.getFileName(),
                            categoryId: selectedCategoryId
                        };
                    });
                }
                return [];
            },

            getJiraTempAttachments: function () {
                var selectedCategoryId = this.getSelectedCategoryId();
                if (selectedCategoryId) {
                    return this.dropzone.progressBars.map(function (progressBar) {
                        return new CommonClasses.JiraTempAttachment(progressBar.getFileID(), selectedCategoryId);
                    });
                }
                return [];
            },

            getDropZoneByCategoryId: function (categoryId) {
                var category = _.find(this.categories, isEqualCategoryId(categoryId));
                if (category) {
                    this.dropzone.category = category;
                } else {
                    this.dropzone.category = this.getDefaultOrFirstCategory();
                }
                return category;
            }
        });

        return FieldTypeDropZone;
    });