(function ($) {
    window.EditInOffice = {

        getType: function (href) {
            var extension = href.substring(href.lastIndexOf('.') + 1).toLowerCase();

            switch (extension) {
                case "doc":
                case "docx":
                case "docm":
                case "dot":
                case "dotx":
                    return "Word.Document";
                case "xls":
                case "xlt":
                case "xlsx":
                case "xlst":
                case "xlsm":
                    return "Excel.Sheet";
                case "ppt":
                case "pptx":
                case "ppsx":
                case "pot":
                case "potx":
                case "pptm":
                    return "PowerPoint.Show";
                case "vsd":
                case "vss":
                case "vssx":
                case "vssm":
                case "vst":
                case "vstx":
                case "vstm":
                case "vdx":
                case "vsx":
                case "vtx":
                case "vsl":
                case "vsdx":
                case "vsdm":
                case "vsw":
                    return "Visio.Drawing";
                default:
                    return ''
            }
        },

        clickHandler: function(e) {
            e.preventDefault();
            $(document).click();
            var link = $(this);

            if (!link.hasClass('disabled')) {
                SmartAttachAnalytics.eventAttachmentEditInOffice(link.attr('href'));
                EditInOffice.edit(link.attr('href'), link.attr('data-office-type'));
            }
        },

        edit: function (webDavUrl, officeType) {
            var t = this;
            var editor;

            if (window.ActiveXObject !== undefined) {
                try {
                    editor = new ActiveXObject('SharePoint.OpenDocuments.1');
                    if (editor) {
                        t.extendUrl(webDavUrl, function(url){
                            editor.EditDocument(t.getBaseUrl() + url, officeType);
                        }, t.handleError);
                        t.informReloadAfterEdit();
                    } else {
                        t.handleMessage(AJS.I18n.getText("attach.cat.edit.office.ie.no-office"), JIRA.Messages.Types.WARNING);
                    }
                } catch(e) {
                    t.handleError(AJS.I18n.getText("attach.cat.edit.office.ie.error"));
                }
            } else if (window.URLLauncher) {
                try {
                    editor = new URLLauncher();
                    EditInOffice.extendUrl(webDavUrl, function(url) {
                        editor.open(url);
                    }, EditInOffice.handleError);
                    t.informReloadAfterEdit();
                } catch (e) {
                    t.handleError(e);
                }
            } else if (window.InstallTrigger) {
                if(window.confirm(AJS.I18n.getText("attach.cat.edit.office.ff.extension"))) {
                    InstallTrigger.install({'WebDAV Launcher': 'https://update.atlassian.com/office-connector/URLLauncher/latest/webdavloader.xpi'});
                }
            } else {
                t.handleMessage(AJS.I18n.getText("attach.cat.edit.office.browser"), JIRA.Messages.Types.WARNING);
            }
        },

        extendUrl: function(url, onSuccess, onError) {
            JIRA.SmartAjax.makeRequest({
                type: 'GET',
                url: AJS.params.baseURL + '/rest/attach-cat/1.0/auth/get',
                success: function(data){
                    if (data.token){
                        var parts = url.trim().split('/');
                        var newPath = '';
                        for (var i = 0; i < parts.length - 1; i++){
                            if (parts[i].length){
                                newPath = newPath + '/' + parts[i];
                            }
                        }
                        var cat = AttachCat.getItemById(parts[parts.length - 3]).parents('.attach-cat-container').attr('id');
                        newPath = newPath + '/cat/' + cat + '/attach-cat-auth/' + data.token + '/' + parts[parts.length - 1];
                        onSuccess(newPath);
                    }
                    else {
                        onError(AJS.I18n.getText("attach.cat.edit.office.auth.error"));
                    }
                },
                error: function(jqXhr, textStatus, errorThrown) {
                    onError(textStatus + ": " + errorThrown);
                }
            });
        },

        informReloadAfterEdit: function() {
            this.handleMessage(AJS.I18n.getText("attach.cat.edit.office.reload.info"), JIRA.Messages.Types.SUCCESS);
        },

        handleError: function(message) {
            EditInOffice.handleMessage(message, JIRA.Messages.Types.ERROR);
        },

        handleMessage: function(message, type) {
            JIRA.Messages.showMsg(message, {
                type: type,
                closeable: true,
                timeout: -1
            });
        },

        getBaseUrl: function() {
            return location.protocol + "//" + location.host;
        },

        isSupportedBrowser: function() {
            return window.ActiveXObject !== undefined || window.URLLauncher || window.InstallTrigger;
        },
        
        makeTempAttachmentNameForURL: function (attachmentName) {
            var dotLastIndex = attachmentName.lastIndexOf('.');
            return dotLastIndex === -1 ? attachmentName : "temp-file-for-editing" + attachmentName.substring(dotLastIndex);
        }
    }
})(AJS.$);
