(function ($) {

    var inputEventTypeCreate, inputEventTypeUpdate, markupDataStore;

    AJS.toInit(function () {
        markupDataStore = $('#jira-stored-event-type-ids');
        initSelect2Inputs();
        bindHandlers();
    });

    function initSelect2Inputs() {
        inputEventTypeCreate = $('#jira-create-issue-event-type-select').auiSelect2();
        inputEventTypeUpdate = $('#jira-update-issue-event-type-select').auiSelect2();
        catchSelect2ChosenValues();
    }

    function bindHandlers() {
        bindSelect2OnChange();
        bindOnClickCancel();
        bindOnClickSave();
    }

    function bindSelect2OnChange() {
        inputEventTypeCreate.on('change', onSelect2Change);
        inputEventTypeUpdate.on('change', onSelect2Change);
    }

    function bindOnClickCancel() {
        $('#jira-events-cancel-button').click(function () {
            var storedValues = markupDataStore.data('eventTypeIds');
            inputEventTypeCreate.val(storedValues.createEventTypeIds).auiSelect2().trigger('change');
            inputEventTypeUpdate.val(storedValues.updateEventTypeIds).auiSelect2().trigger('change');
            return false;
        });
    }

    function bindOnClickSave() {
        $('#jira-events-save-button').click(function () {
            var selectedValues = getSelectedValues();

            if (haveInputsSameEventTypes(selectedValues)) {
                showErrorMessageSameValues();
                return false;
            }

            $.ajax({
                type: 'POST',
                url: AJS.params.baseURL + '/rest/attach-cat/1.0/settings/jira-events',
                data: selectedValues,
                success: function () {
                    catchSelect2ChosenValues();
                    inputEventTypeCreate.trigger('change');
                    inputEventTypeUpdate.trigger('change');
                    showSuccessMessageStored();
                },
                error: function () {
                    showErrorMessageAjaxFailed();
                }
            });

            return false;
        });
    }

    function catchSelect2ChosenValues() {
        markupDataStore.data('eventTypeIds', getSelectedValues());
    }

    function getSelectedValues() {
        return {
            createEventTypeIds: getSelectedEventTypeIds(inputEventTypeCreate),
            updateEventTypeIds: getSelectedEventTypeIds(inputEventTypeUpdate)
        }
    }

    function getSelectedEventTypeIds($input) {
        var selectedItemsCreate = [];
        $.each($input.select2('data'), function (i, option) {
            selectedItemsCreate.push(option.id);
        });
        return selectedItemsCreate.sort(function (a, b) {
            return a.localeCompare(b);
        });
    }

    function onSelect2Change() {
        var storedValues = markupDataStore.data('eventTypeIds');
        var selectedValues = getSelectedValues();
        var buttonContainer = $('.jira-event-buttons-container');

        isEqualStoredAndSelectedValues(storedValues, selectedValues)
            ? buttonContainer.removeClass('not-saved') : buttonContainer.addClass('not-saved');
    }

    function isEqualStoredAndSelectedValues(stored, selected) {
        return JSON.stringify(stored) === JSON.stringify(selected);
    }

    function showSuccessMessageStored() {
        var message = AJS.I18n.getText("attach.cat.action.events.message.stored");
        showAutocloseableMessage(message, JIRA.Messages.Types.SUCCESS);
    }

    function showErrorMessageAjaxFailed() {
        var message = AJS.I18n.getText("attach.cat.change.error");
        showAutocloseableMessage(message, JIRA.Messages.Types.ERROR);
    }

    function showErrorMessageSameValues() {
        var message = AJS.I18n.getText("attach.cat.action.events.message.unique-create-update-handlers");
        showAutocloseableMessage(message, JIRA.Messages.Types.ERROR);
    }

    function showAutocloseableMessage(message, type) {
        var flag = JIRA.Messages.showMsg(message, {
            type: type,
            timeout: 0,
            closeable: true
        });
        setTimeout(function () {
            flag[0].close();
        }, 10000);
    }

    function haveInputsSameEventTypes(selectedValues) {
        var common = $.grep(selectedValues.createEventTypeIds, function (id) {
            return $.inArray(id, selectedValues.updateEventTypeIds) !== -1;
        });
        return common.length > 0;
    }

})(AJS.$);