(function ($) {

    AJS.toInit(function () {
        catchStoredEvents();
        bindEvents();
    });

    function bindEvents() {
        $('#sa-events-save-button').click(saveEvents);
        $('#sa-events-cancel-button').click(restoreEvents);
        $('.sa-event-form select').change(changeSelectValue);
    }

    function saveEvents(e) {
        e.preventDefault();

        var data = getSelectedOptions();

        if (isItHasSameValues(data)) {
            showMessageNotUniqueEvents();
        } else {
            $.ajax({
                type: 'POST',
                url: AJS.params.baseURL + '/rest/attach-cat/1.0/settings/sa-events',
                data: data,
                success: function () {
                    catchStoredEvents();
                    showMessageStored();
                    hideButtonControls();
                },
                error: function () {
                    showMessageAjaxFailed();
                }
            });
        }
    }

    function restoreEvents(e) {
        e.preventDefault();
        var data = getStoredEvents();
        getAddIntoCategorySelect().val(data.addIntoCategoryEventId);
        getAddIntoDocumentSelect().val(data.addIntoDocumentEventId);
        getAttachRenamedSelect().val(data.attachRenamedEventId);
        getAttachMovedSelect().val(data.attachMovedEventId);
        hideButtonControls();
    }
    
    function changeSelectValue() {
        var storedData = JSON.stringify(getStoredEvents());
        var selectedData = JSON.stringify(getSelectedOptions());
        storedData !== selectedData ? showButtonControls() : hideButtonControls();
    }

    function catchStoredEvents() {
        setStoredEvents(getSelectedOptions());
    }

    function getAddIntoCategorySelect() {
        return $('#sa-add-into-category-select');
    }

    function getAddIntoDocumentSelect() {
        return $('#sa-add-into-document-select');
    }

    function getAttachRenamedSelect() {
        return $('#sa-attach-renamed-select');
    }

    function getAttachMovedSelect() {
        return $('#sa-attach-moved-select');
    }

    function getStoredEvents() {
        return $('#sa-stored-events').data('storedEvents');
    }

    function setStoredEvents(storedEvents) {
        return $('#sa-stored-events').data('storedEvents', storedEvents);
    }

    function getSelectedOptions() {
        return {
            addIntoCategoryEventId: getAddIntoCategorySelect().val(),
            addIntoDocumentEventId: getAddIntoDocumentSelect().val(),
            attachRenamedEventId: getAttachRenamedSelect().val(),
            attachMovedEventId: getAttachMovedSelect().val()
        };
    }

    function isItHasSameValues(data) {
        var values = [];

        $.each(data, function (property, value) {
            if (value !== '-1') {
                values.push(value)
            }
        });

        for (var i = 0; i <= values.length; i++) {
            for (var j = i; j <= values.length; j++) {
                if (i !== j && values[i] === values[j]) {
                    return true;
                }
            }
        }
        return false;
    }
    
    function showButtonControls() {
        $('.sa-event-form .sa-event-buttons-container').addClass('not-saved');
    }

    function hideButtonControls() {
        $('.sa-event-form .sa-event-buttons-container').removeClass('not-saved');
    }
    
    function showMessageStored() {
        var message = AJS.I18n.getText("attach.cat.action.events.message.stored");
        showAutocloseableMessage(message, JIRA.Messages.Types.SUCCESS);
    }
    
    function showMessageNotUniqueEvents() {
        var message = AJS.I18n.getText("attach.cat.action.events.message.unique-events");
        showAutocloseableMessage(message, JIRA.Messages.Types.ERROR);
    }

    function showMessageAjaxFailed() {
        var message = AJS.I18n.getText("attach.cat.change.error");
        showAutocloseableMessage(message, JIRA.Messages.Types.ERROR);
    }

    function showAutocloseableMessage(message, type) {
        var flag = JIRA.Messages.showMsg(message, {
            type: type,
            timeout: 0,
            closeable: true
        });
        setTimeout(function () {
            flag[0].close();
        }, 10000);
    }

})(AJS.$);