(function ($) {

    window.AutocompleteFileViewer = {

        $textarea: "",
        cursor: "",
        isShow: false,

        bindTextAreaAutocomplete: function () {
            var t = AutocompleteFileViewer;
            t.$textarea = $(this);
            t.cursor = Cursores.startsWith('@');

            $(window).on('mousedown', '.sa-suggestions-wrapper li', t.replaceSuggestion);

            t.$textarea.off('keydown.sa-autocomplete').on('keydown.sa-autocomplete', function (e) {
                if ($.inArray(e.which, [13, 38, 40]) >= 0 && t.isShow) {
                    if (e.keyCode !== 13) {
                        t.changeActiveLi(e.keyCode === 40);
                    } else {
                        t.selectActiveLi();
                    }
                    return false;
                }
            });

            t.$textarea.off('keyup.sa-autocomplete').on('keyup.sa-autocomplete', function (e) {
                if ($.inArray(e.which, [13, 38, 40]) >= 0 && AutocompleteFileViewer.isShow) {
                    return;
                }
                var token = t.cursor.token(t.$textarea[0]);

                if (token.value.length > 2) {
                    t.showSuggestions(t.$textarea, token);
                } else {
                    t.removeSuggestions();
                }
            });

            $(window).off('mouseover.sa-sugg-li').on('mouseover.sa-sugg-li', '.sa-suggestions-wrapper li', function () {
                $('.sa-suggestions-wrapper li').each(function () {
                    $(this).removeClass('active');
                });
                $(this).addClass('active');
            });
        },

        replaceSuggestion: function () {
            var li = $(this);
            AutocompleteFileViewer.cursor.replace(AutocompleteFileViewer.$textarea[0], "[~" + li.data('userName') + "]");
            AutocompleteFileViewer.removeSuggestions();
            AutocompleteFileViewer.$textarea.focus();
            AutocompleteFileViewer.checkSelectedUserOnViewPermissions(li.data('userName'));
            return false;
        },

        showSuggestions: function ($textarea, token) {

            $.ajax({
                type: 'GET',
                cache: false,
                url: AJS.params.baseURL + "/rest/api/2/user/picker",
                data: {
                    query: token.value.substring(1),
                    maxResults: 2,
                    showAvatar: true
                },
                dataType: 'json',
                success: function (users) {
                    AutocompleteFileViewer.removeSuggestions();
                    AutocompleteFileViewer.isShow = true;
                    var suggestionContainer = AutocompleteFileViewer.getSuggestionContainer(AutocompleteFileViewer.$textarea);
                    suggestionContainer.append(FVITemplates.suggestions(users));

                    if ($textarea.hasClass('sa-reply-textarea')){
                        suggestionContainer.css('top', trackCaret.getCursorPosition($textarea, false));
                        FVI.scrollReplyControls($textarea.parents('.sa-thread-wrapper'));
                    }

                    if ($textarea.hasClass('sa-textarea-create-thread')){
                        $textarea.addClass('sa-textarea-create-thread-short').removeClass('sa-textarea-create-thread');
                        AutocompleteFileViewer.reScrollTextarea();
                    }
                },
                error: function () {
                    AutocompleteFileViewer.removeSuggestions();
                    AJS.log("Unable to find users for query {" + token.value + "}");
                }
            });
        },

        removeSuggestions: function () {
            AutocompleteFileViewer.isShow = false;
            var sidebar = $('#sa-comments');
            sidebar.find('.sa-textarea-create-thread-short').addClass('sa-textarea-create-thread').removeClass('sa-textarea-create-thread-short');
            sidebar.find('.sa-autocomplete-suggestions').empty();
        },

        removeSuggestionOnBlur: function (e) {
            setTimeout(function () {
                if (!$(e.currentTarget).is(':focus')) {
                    AutocompleteFileViewer.removeSuggestions();
                }
            }, 200);
        },

        reScrollTextarea: function () {
            var textarea = AutocompleteFileViewer.$textarea;
            textarea.blur();
            setTimeout(function () {
                textarea.focus();
                var contentHeight = trackCaret.getCursorPosition(textarea, true);
                if (contentHeight < textarea.scrollTop() || contentHeight > (textarea.scrollTop() + textarea.height())) {
                    textarea.scrollTop(contentHeight);
                }
            }, 10);
        },

        changeActiveLi: function (isIncrement) {
            var suggestions = $('.sa-suggestions-wrapper');
            var allLi = suggestions.find('li');
            var index = allLi.index(suggestions.find('li.active'));
            index = isIncrement ? index + 1 : index - 1;

            if (index >= 0 && index <= allLi.length - 1) {
                allLi.each(function () {
                    $(this).removeClass('active');
                });
                $(allLi.get(index)).addClass('active');
            }
        },

        selectActiveLi: function () {
            $('.sa-suggestions-wrapper li.active').trigger('mousedown');
        },

        getSuggestionContainer: function ($textarea) {
            return $textarea.parent().find('.sa-autocomplete-suggestions');
        },

        checkSelectedUserOnViewPermissions: function (userKey) {
            var category = AttachCat.getItemById(FVI.getOpenedAttachId()).parents('.attach-cat-container');
            var categoryId = category.attr('id');
            $.ajax({
                type: 'GET',
                cache: false,
                async: true,
                url: AJS.params.baseURL + '/rest/attach-cat/1.0/comment/canView',
                data: {
                    issueKey: AttachCat.getIssueKey(),
                    userKey: userKey,
                    categoryId: categoryId
                },
                success: function (canView) {
                    if (canView === 'false'){
                        var controls = AutocompleteFileViewer.$textarea.parents('form.aui').find('.sa-fv-controls');
                        if (!controls.find('.sa-comment-mentioning-info').length) {
                            FVI.initMentionHelpIcon(controls);
                        }
                    }
                },
                error: function () {
                }
            });
        }
    };

    window.trackCaret = {

        getCursorPosition: function ($textarea, withScroll) {
            var textarea = $textarea.get(0);
            var $parent = $textarea.parent();
            $parent.css('position', 'relative');

            var textareaMirror = createElement('div', $parent.get(0), 'textarea textareaMirror');
            var textareaMirrorInline = createElement('span', textareaMirror, '');

            var width = $textarea.outerWidth();
            var height = $textarea.outerHeight();
            var position = $textarea.position();

            $(textareaMirror).css('max-width', 'none')
                .css('width', width)
                .css('margin-left', position.left)
                .css('margin-top', position.top);

            textareaMirrorInline.innerHTML = textarea.value.substr(0, textarea.selectionStart).replace(/\n$/, "\n\001");
            var top = $(textareaMirror).outerHeight(true);
            if (!withScroll) {
                top = top > height ? height : top;
            }
            $parent.find('.textareaMirror').remove();
            $parent.css('position', '');
            return top;
        }
    };

    var createElement = function (type, parent, className) {
        var el = document.createElement(type);
        parent.appendChild(el);
        el.className = className;
        return el;
    };

})(AJS.$);