(function ($) {

    window.FakeFileViewer = {

        init: function (context) {
            FakeFileViewer.addAttachParams(context);
            FakeFileViewer.bindEvents(context, true);
        },

        show: function (e) {
            var attachLink = $(e.currentTarget).closest('[file-preview-id]');
            var fileUrl = FVI.getUrlFromAttachTitle(attachLink);

            var file = {
                'title': attachLink.attr('file-preview-title'),
                'url': fileUrl,
                'isImage': attachLink.attr('file-preview-type') === 'image',
                'isPDF': false
            };

            if ($('#cp-container').length) {
                FakeFileViewer.reinit(file);
            } else {
                $('body').addClass('no-scroll').append(FVITemplates.fakeFVTemplate(file));
            }

            FakeFileViewer.resizeImageContainer();
            return false;
        },

        open: function (attachmentId, attachment) {

            var attachTitle = attachment.find('.attachment-title');
            var attachLink = attachment.find('a[file-preview-type]');

            var file = {
                'title': attachTitle.text(),
                'url': FVI.getUrlFromAttachTitle(attachTitle),
                'isImage': attachLink ? attachLink.attr('file-preview-type') === 'image' : false,
                'isPDF': false,
                attachmentId: attachmentId
            };

            $('body').addClass('no-scroll').append(FVITemplates.fakeFVTemplate(file));
            FakeFileViewer.reinit(file);
        },

        hide: function () {
            $('body').removeClass('no-scroll');
            $('#cp-container').remove();
        },

        isShow: function () {
            return $('#cp-container').length !== 0;
        },

        bindEvents: function (context, isInitShow) {
            FakeFileViewer.unbindEvents();
            if (isInitShow) {
                context.find(FVI.files).off('simpleClick.fvOpen').on('simpleClick.fvOpen', FakeFileViewer.show);
            }
            $(document).off('click.sa-ffv-close').on('click.sa-ffv-close', '#cp-control-panel-close', FakeFileViewer.hide);
            $(window).off('resize.sa-ffv-resize').on('resize.sa-ffv-resize', FakeFileViewer.resizeImageContainer);
            $(window).off('keydown.sa-ffv-esc').on('keydown.sa-ffv-esc', FakeFileViewer.hideOnEsc);
        },

        unbindEvents: function () {
            FakeFileViewer.unbindFancyBox();
            $(document).off('click.sa-ffv-close');
            $(window).off('resize.sa-ffv-resize')
                .off('keydown.sa-ffv-esc');
        },

        addAttachParams: function (context) {
            context.find('.attachment-content').each(function () {
                var attach = $(this);
                var previewLink = attach.find('.attachment-thumb a');
                var titleLink = attach.find('.attachment-title');
                var type = 'other';

                if (attach.length) {
                    if (attach.get(0).hasAttribute('data-downloadurl')) {
                        type = detectContentType(attach);
                    } else {
                        var attachWithDataDownloadUrl = attach.find('[data-downloadurl]');
                        if (attachWithDataDownloadUrl.length) {
                            type = detectContentType(attachWithDataDownloadUrl);
                        }
                    }
                }

                if (type === 'image' || type === 'document') {
                    var attachId = AttachCat.getItemId(attach);
                    var attachName = AttachCat.getItemNameElement(attach).eq(0).text();
                    if (attachId !== undefined && attachName !== undefined) {
                        FVI.addFVLinkParams(previewLink, attachId, attachName, type);
                        FVI.addFVLinkParams(titleLink, attachId, attachName, type);
                    }
                }
            });
        },

        resizeImageContainer: function () {
            var scrollBar = 5;
            var fileBody = $('#cp-file-body');
            var layer = fileBody.find('.cp-viewer-layer');
            $('.cp-image-container').css('width', layer.width() + 'px').css('height', layer.height() + 'px');
            $('#ffv-pdf-container').css('width', layer.width() - scrollBar + 'px').css('height', layer.height() + 'px');
            $('#cp-img').css('max-width', layer.width() + 'px').css('max-height', layer.height() + 'px');
        },

        unbindFancyBox: function () {
            $(document).find('a.gallery, a.attachment-title').unbind('click.fb');
        },

        hideOnEsc: function (e) {
            if (e.keyCode === 27 && FakeFileViewer.isShow()) {
                if (!$('#sa-create-thread-wrapper').find('textarea').is(':focus'))
                    FakeFileViewer.hide();
            }
        },

        changeFileRevision: function (li) {
            var attachId = $(li).attr('file-preview-id');
            FakeFileViewer.hide();
            FakeFileViewer.open(attachId, AttachCat.getItemById(attachId));
        },

        reinit: function (newFile) {
            $('#cp-title-container').empty().append(FVITemplates.fakeFVTitle(newFile));
            $('#cp-download-button').empty().append(FVITemplates.fakeFVDownload(newFile));
            $('#cp-body').empty().append(FVITemplates.fakeFVFileBody(newFile));
            $('#fv-comments').parent().remove();
            FVI.initButtons(newFile.attachmentId);
        }
    };

    function detectContentType($elem) {
        var contentType = $elem.attr('data-downloadurl').split(':')[0];
        return contentType.indexOf('image') !== -1 ? 'image' : 'other';
    }

})(AJS.$);