(function ($) {

    window.FVI = {

        documents: [],
        files : "[file-preview-type=image], [file-preview-type=document]",
        isActiveSidebar : false,
        isControlsInit: false,

        init: function () {
            var categorySection = $('#attach-cat-content');
            FVI.initFileViewer(categorySection);
            FVI.bindEvents();
            FVI.openCommentByURLParams();
        },

        bindEvents: function () {
            var t = this;
            $(t.files).on('mousedown.sa-comm-fv', function () {
                var events = $(this).data("events").simpleClick;
                if (events !== undefined && events.length < 2) {
                    $(t.files).on("simpleClick.fvOpen", t.initButtons);
                }
            });

            FVI.unbindAllEvents();

            FVI.bindSidebarEvents();
            FVI.bindCommentEvents();
            FVI.bindThreadEvents();
            FVI.bindMentionsEvent();
        },

        unbindAllEvents: function () {
            $(window)
                .off('click.sa-resize-sidebar').off('click.sa-toggle-sidebar').off('click.sa-create-thread')
                .off('keydown.sa-thread-submit').off('click.sa-hide-sidebar').off('mousedown.sa-fv-nav')
                .off('DOMNodeInserted.sa-fv-cntrls').off('click.sa-create-thread-cancel')
                .off('click.sa-create-thread-show').off('click.sa-create-thread-hide').off('focus.sa-reply-show')
                .off('click.sa-reply-hide').off('click.sa-change-file-rev').off('click.sa-make-active')
                .off('click.sa-unmake-active').off('mousedown.sa-thread-remove').off('click.sa-reply-add')
                .off('keydown.sa-reply-submit').off('click.sa-comm-remove').off('click.sa-comm-edit')
                .off('click.sa-comm-edit-cancel').off('click.sa-comm-edit-save').off('keydown.sa-comm-edit-submit')
                .off('focus.sa-textarea').off('blur.sa-textarea').off('click.sa-next-thread')
                .off('keyup.sa-grayed-out-button').off('click.sa-prev-thread');

            $(document).off('keydown.sa-textarea-key-lr-arrows');
        },

        bindSidebarEvents: function () {
            $(window)
                .on('click.sa-resize-sidebar', '#cp-info', FVI.resizeSidebar)
                .on('click.sa-toggle-sidebar', '#fv-comments', FVI.toggleSidebar)
                .on('click.sa-hide-sidebar', '#cp-control-panel-close, .cp-toolbar-presentation', FVI.hideSidebar)
                .on('mousedown.sa-fv-nav', '#cp-nav-left, #cp-nav-right, .cp-thumbnail', FVI.resetControlsFlag)
                .on('click.sa-create-thread-cancel', '#sa-comment-cancel', FVI.clearAndHideCreateThread)
                .on('click.sa-create-thread-show', '#sa-new-thread-show', FVI.showCreateThread)
                .on('click.sa-create-thread-hide', '#sa-new-thread-hide', FVI.hideCreateThread)
                .on('focus.sa-reply-show', '.sa-reply-input', FVI.showReplyControls)
                .on('click.sa-reply-hide', '.sa-reply-cancel', FVI.hideReplyControls)
                .on('click.sa-change-file-rev', '.sa-revision-item', FVI.changeFileRevision);

            $(document).on('keydown.sa-textarea-key-lr-arrows', '#sa-comments textarea', FVI.catchFVKeydownLRArrowEvents);
        },

        bindThreadEvents: function () {
            $(window)
                .on('click.sa-create-thread', '#sa-create-thread', FVI.createThread)
                .on('keydown.sa-thread-submit', '#sa-create-thread-wrapper textarea', FVI.submitThread)
                .on('click.sa-make-active', '#sa-threads-container .sa-thread-wrapper', FVI.makeThreadActiveByEvent)
                .on('click.sa-unmake-active', '#sa-threads-container', FVI.unmakeThreadActiveByEvent)
                .on('mousedown.sa-thread-remove', '.sa-thread-delete-menu', FVI.showDeleteThreadDialog);
            FVI.bindThreadArrowsNavigation();
        },

        bindCommentEvents: function () {
            $(window).on('click.sa-reply-add', '.sa-reply-add', FVI.addReply)
                .on('keydown.sa-reply-submit', '.sa-reply-textarea', FVI.ctrlEnterSubmitReply)
                .on('click.sa-comm-remove', '.sa-comment-delete-dialog', FVI.showDeleteCommentDialog)
                .on('click.sa-comm-edit', '.sa-comment-edit', FVI.editComment)
                .on('click.sa-comm-edit-cancel', '.sa-comment-edit-cancel', FVI.editCommentCancel)
                .on('click.sa-comm-edit-save', '.sa-comment-edit-save', FVI.editCommentSave)
                .on('keydown.sa-comm-edit-submit', '.sa-edit-comment-textarea', FVI.submitEditComment)
                .on('keyup.sa-grayed-out-button', '.sa-reply-textarea, .sa-edit-comment-textarea, #sa-create-thread-wrapper textarea', FVI.checkOnEmptyTextareaAndEnableButtons);
        },

        bindMentionsEvent: function () {
            $(window)
                .on('focus.sa-textarea', '#sa-comments textarea', AutocompleteFileViewer.bindTextAreaAutocomplete)
                .on('blur.sa-textarea', '#sa-comments textarea', AutocompleteFileViewer.removeSuggestionOnBlur);
        },

        bindCheckFVControls: function () {
            waitForElement('#cp-title-container', function (selector, context) {
                observeDOMTitleUpdated(context.get(0), function () {
                    FVI.resetControlsFlag();
                    FVI.checkFVControls();
                });
            }, 200);
        },

        initFileViewer: function (context) {
            if ($('#ghx-tab-attachments').length === 0) {
                if (!FVI.isFVEnabled()) {
                    FakeFileViewer.init(context);
                } else {
                    FakeFileViewer.bindEvents(context, false);
                }
            }
        },

        initButtons: function () {
            FVI.addButtonComments();
            FVI.addButtonRevisions();
            FVI.addButtonOpenInBrowser();
        },

        toggleSidebar: function () {
            var t = FVI;
            var sidebar = $('#sa-comments');
            if (sidebar.is(":visible")) {
                t.hideSidebar();
            } else {
                t.showSidebar();
            }
        },

        hideSidebar: function (e) {
            FVI.isActiveSidebar = false;
            $('#sa-comments').hide();
            $('#cp-file-body').removeClass('fv-cp-viewer-layer');
            $('#cp-nav-right').removeClass('fv-cp-nav-right');
            $('#inline-dialog-sa-file-revision-dialog').remove();
            resizeImageContainer();

            if (e && e.currentTarget.id === "cp-control-panel-close" && $(e.currentTarget).is(":visible")) {
                var newEvent = jQuery.Event("keydown");
                newEvent.which = 27;
                $(document).trigger(newEvent);
            }
        },

        showSidebar: function () {
            FVI.isActiveSidebar = true;
            $('#sa-comments').show();
            $('#cp-file-body').addClass('fv-cp-viewer-layer');
            $('#cp-nav-right').addClass('fv-cp-nav-right');
            FVI.loadComments();
            FVI.resizeSidebar();
            resizeImageContainer();
        },

        resizeSidebar: function () {
            var fileBody = $('#cp-file-body');
            var sidebar = $('#sa-comments');
            if (fileBody.hasClass('meta-banner')) {
                sidebar.addClass('fv-meta-banner');
            } else {
                sidebar.removeClass('fv-meta-banner');
            }
            if (fileBody.hasClass('short')) {
                sidebar.addClass('fv-short');
            } else {
                sidebar.removeClass('fv-short');
            }
        },

        checkFVControls: function () {
            if (!$(this).find('#fv-comments').length && !FVI.isControlsInit) {
                FVI.isControlsInit = true;
                FVI.initButtons();
            }
        },

        addButtonComments: function () {
            waitForElement('#cp-file-controls', function (selector, context) {
                var sidebar = $('#sa-comments');
                var shortSidebar = $('.fv-cp-viewer-layer');
                var isInitCommentButton = $('#fv-comments').length > 0;

                if (!isInitCommentButton) {
                    var attachId = AttachCat.getItemId($('#cp-control-panel-download').parent());
                    $(context).find('#cp-control-panel-download').parent().after(FVITemplates.buttonComments({'attachId': attachId}));
                }
                if (!sidebar.length) {
                    $('#cp-body').prepend(FVITemplates.sidebar(FVI.getLoggedInUser()));
                }

                if (FVI.isActiveSidebar && !sidebar.is(":visible")) {
                    FVI.showSidebar();
                } else {
                    FVI.isActiveSidebar = sidebar.is(":visible");
                    if (shortSidebar.length && !FVI.isActiveSidebar) {
                        shortSidebar.removeClass('fv-cp-viewer-layer');
                    }
                    if (FVI.isActiveSidebar) {
                        FVI.reloadComments();
                    }
                }
                FVI.resizeSidebar();
                FVI.bindCheckFVControls();
            }, 200);
        },

        addButtonRevisions: function () {
            waitForElement('#cp-title-container', function (selector, context) {
                $('#sa-file-revisions').remove();
                $('#sa-file-revisions-container').remove();

                var attachId = FVI.getOpenedAttachId();
                var title = '';
                var ids = [];

                $.each(FVI.documents, function () {
                    var index = this.attachmentIds.indexOf(attachId);
                    if (index >= 0) {
                        index++;
                        title = AJS.I18n.getText("attach.cat.file-viewer.revision.title", index);
                        ids = this.attachmentIds;
                        return false;
                    }
                });

                if (ids.length) {
                    $(context).after(FVITemplates.buttonRevisions({'title': title}));
                    FVI.initRevisionsDialog(ids, attachId);
                }
            }, 200);
        },

        addButtonOpenInBrowser: function () {
            waitForElement('#cp-error-message a.download-button', function (selector, context) {
                $('#sa-open-in-browser').remove();
                context.after(FVITemplates.buttonOpenInBrowser({'url': context.attr('href')}));
            }, 200);
        },

        resetControlsFlag: function () {
            FVI.isControlsInit = false;
        },

        addFVLinkParams: function (link, id, name, type) {
            if (!link.is("a")) {
                link = link.find('a');
            }
            link.attr('file-preview-id', id);
            link.attr('file-preview-title', name);
            link.attr('file-preview-type', type);
        },

        appendComments: function (data, isActiveLastThread) {
            $('#sa-comments').data('showNotificationInfoIcon', data.showNotificationInfoIcon);
            var threadsContainer = $('#sa-threads-container');
            threadsContainer.find('.sa-no-comments').remove();

            $.each(data.comments, function (i, commentEntity) {
                var thread = threadsContainer.find('#sa-comment-thread-' + commentEntity.threadId + ' .sa-thread');
                if (!thread.length) {
                    commentEntity["canDeleteAll"] = data.canDeleteAll;
                    FVI.appendThread(threadsContainer, commentEntity);
                }
                FVI.appendComment(commentEntity);
            });

            if (data.comments.length !== 0) {
                if (isActiveLastThread) {
                    threadsContainer.find('.sa-thread-wrapper').first().trigger('click.sa-make-active');
                }
                if (threadsContainer[0] !== undefined) {
                    threadsContainer.scrollTop(0);
                }
            } else {
                threadsContainer.append(FVITemplates.commentsNotFound());
            }
            FVI.drawThreadPreviews();
            FVI.redrawThreadsNavigation();

            var createThreadPanel = $('#sa-create-thread-wrapper');
            if (createThreadPanel.is(":visible")){
                FVI.initCommentHelpIcon(createThreadPanel.find('.sa-fv-controls'));
            }
        },

        appendComment: function (commentEntity) {
            var thread = $('#sa-comment-thread-' + commentEntity.threadId + ' .sa-thread');
            var comment = thread.find("[data-comment-id='" + commentEntity.id + "']");
            if (!comment.length) {
                commentEntity = FVI.removeFVLinkFromRenderedComment(commentEntity);
                var threadComments = thread.find('.sa-comment');
                var newComment = FVITemplates.comment(commentEntity);

                var isInserted = false;
                threadComments.each(function (i, elem) {
                    var currentComment = $(elem);
                    if (currentComment.data('commentId') > commentEntity.id) {
                        currentComment.before(newComment);
                        isInserted = true;
                        return false;
                    }
                });
                if (!isInserted) {
                    thread.append(newComment);
                }
            }
        },

        removeFVLinkFromRenderedComment: function (commentEntity) {
            var commentBody = $(commentEntity.renderedContent);
            commentBody.find('a[file-preview-id]').each(function (i, link) {
                commentBody.find(link).replaceWith(link.innerHTML);
            });
            commentEntity.renderedContent = $('<div>').append(commentBody.clone()).html();
            return commentEntity;
        },

        appendThread: function (threadsContainer, commentEntity) {
            var threadAppended = false;
            commentEntity["canAddComments"] = FVI.isCanAddComments();

            threadsContainer.find('.sa-thread-wrapper').each(function () {
                if ($(this).data('threadId') < commentEntity.threadId) {
                    $(this).before(FVITemplates.thread(commentEntity));
                    threadAppended = true;
                    return false;
                }
            });

            if (!threadAppended) {
                threadsContainer.append(FVITemplates.thread(commentEntity));
            }
        },

        loadComments: function () {
            $.ajax({
                type: "GET",
                data: {
                    'attachId': FVI.getOpenedAttachId()
                },
                url: AJS.params.baseURL + '/rest/attach-cat/1.0/comment',
                success: FVI.appendComments
            });
        },

        reloadComments: function () {
            $('#sa-threads-container').empty();
            FVI.loadComments();
        },

        clearAndHideCreateThread: function () {
            $('#sa-create-thread-wrapper').find('textarea').val('').blur();
            FVI.hideCreateThread();
            return false;
        },

        submitThread: function (e) {
            if (e.ctrlKey && e.keyCode == 13) {
                return FVI.createThread();
            }
        },

        submitEditComment: function (e) {
            if (e.ctrlKey && e.keyCode == 13) {
                return FVI.editCommentSave(e);
            }
        },

        ctrlEnterSubmitReply: function (e) {
            if (e.ctrlKey && e.keyCode == 13) {
                return FVI.addReply(e);
            }
        },

        catchFVKeydownLRArrowEvents: function (e) {
            if ((e.keyCode === 37 || e.keyCode === 39)) {
                if ($('#sa-comments').find('textarea:focus')) {
                    e.stopPropagation();
                    e.stopImmediatePropagation();
                } else {
                    FVI.resetControlsFlag();
                }
            }
        },

        createThread: function () {
            var createThreadPanel = $('#sa-create-thread-wrapper');
            var textarea = createThreadPanel.find('textarea');
            var content = textarea.val().trim();
            var attachId = FVI.getOpenedAttachId();

            SmartAttachAnalytics.eventCreateCommentThread(FVI.getAttachURL(attachId));

            if (content !== '') {
                FVI.showSpinner(createThreadPanel, textarea);
                $.ajax({
                    type: "POST",
                    data: {
                        'attachId': attachId,
                        'content': content
                    },
                    url: AJS.params.baseURL + '/rest/attach-cat/1.0/comment/thread',
                    success: function (data) {
                        FVI.hideSpinner(createThreadPanel, textarea);
                        FVI.clearAndHideCreateThread();
                        FVI.appendComments(data, true);
                        FVI.redrawThreadsNavigation();
                        FVI.makeIconCommentForAttachment();
                    },
                    error: function () {
                        FVI.hideSpinner(createThreadPanel, textarea);
                    }
                });
            } else {
                textarea.focus();
            }

            return false;
        },

        showDeleteCommentDialog: function (e) {
            var comment = $(e.currentTarget).closest('.sa-comment');

            var dialog = new AJS.Dialog({
                width: 500,
                height: 170,
                id: "sa-comment-delete-dialog",
                closeOnOutsideClick: true
            });

            dialog.addHeader(AJS.I18n.getText("attach.cat.file-viewer.dialog.delete-comment.title"));
            dialog.addPanel('panel', FVITemplates.deleteCommentDialog(), 'sa-comment-delete-dialog-panel');

            dialog.addButton(AJS.I18n.getText("attach.cat.action.delete"), function () {
                FVI.deleteComment(comment, dialog);
            }, "sa-comment-delete");

            dialog.addLink(AJS.I18n.getText("attach.cat.action.cancel"), function (dialog) {
                dialog.remove();
            }, '#');

            dialog.show();
            return false;
        },

        deleteComment: function (comment, dialog) {
            var threadsContainer = $('#sa-threads-container');
            var thread = comment.parents('.sa-thread-wrapper');
            var attachId = thread.data('attachmentId');
            var commentId = comment.data('commentId');

            SmartAttachAnalytics.eventDeleteComment(FVI.getAttachURL(attachId));

            $.ajax({
                type: "DELETE",
                data: {
                    'commentId': commentId
                },
                url: AJS.params.baseURL + '/rest/attach-cat/1.0/comment',
                success: function () {
                    comment.remove();
                    dialog.remove();
                    if (thread.find('.sa-thread').is(':empty')) {
                        thread.remove();
                        FVI.redrawThreadsNavigation();
                        FVI.removeIconCommentForAttachment(attachId);
                    }
                    if (threadsContainer.is(':empty')) {
                        threadsContainer.append(FVITemplates.commentsNotFound());
                    }
                    FVI.drawThreadPreviews();
                },
                error: function () {
                    dialog.remove();
                }
            });
        },

        showDeleteThreadDialog: function () {
            var dialog = new AJS.Dialog({
                width: 500,
                height: 170,
                id: "sa-thread-delete-dialog",
                closeOnOutsideClick: true
            });

            dialog.addHeader(AJS.I18n.getText("attach.cat.file-viewer.dialog.delete-thread.title"));
            dialog.addPanel('panel', FVITemplates.deleteThreadDialog(), 'sa-thread-delete-dialog-panel');

            dialog.addButton(AJS.I18n.getText("attach.cat.action.delete"), function () {
                FVI.deleteThread(dialog);
            }, "sa-thread-delete");

            dialog.addLink(AJS.I18n.getText("attach.cat.action.cancel"), function (dialog) {
                dialog.remove();
            }, '#');

            dialog.show();
            return false;
        },

        deleteThread: function (dialog) {
            var threadsContainer = $('#sa-threads-container');
            var thread = threadsContainer.find('.sa-active-thread');
            var attachId = thread.data('attachmentId');

            SmartAttachAnalytics.eventDeleteCommentThread(FVI.getAttachURL(attachId));

            $.ajax({
                type: "DELETE",
                data: {
                    'threadId': thread.data('threadId'),
                    'attachId': FVI.getOpenedAttachId()
                },
                url: AJS.params.baseURL + '/rest/attach-cat/1.0/comment/thread',
                success: function () {
                    thread.remove();
                    dialog.remove();
                    if (threadsContainer.is(':empty')) {
                        threadsContainer.append(FVITemplates.commentsNotFound());
                    }
                    FVI.redrawThreadsNavigation();
                    FVI.removeIconCommentForAttachment(attachId);
                },
                error: function () {
                    dialog.remove();
                }
            });
        },

        editComment: function (e) {
            var comment = $(e.currentTarget).closest('.sa-comment');
            var textarea = comment.find('.sa-edit-comment-textarea');
            comment.find('.sa-comment-edit-body').show();
            comment.find('.sa-comment-body').hide();
            textarea.autosize();
            textarea.focus();
            FVI.initCommentHelpIcon(comment.find('.sa-fv-controls'));
            return false;
        },

        editCommentSave: function (e) {
            var attachId = FVI.getOpenedAttachId();
            var comment = $(e.currentTarget).closest('.sa-comment');
            var textarea = comment.find('.sa-edit-comment-textarea');
            FVI.showSpinner(comment, textarea);

            SmartAttachAnalytics.eventEditComment(FVI.getAttachURL(attachId));

            $.ajax({
                type: 'PUT',
                data: {
                    'commentId': comment.data('commentId'),
                    'content': textarea.val()
                },
                url: AJS.params.baseURL + '/rest/attach-cat/1.0/comment',
                success: function (updatedComment) {
                    FVI.hideSpinner(comment, textarea);
                    if (updatedComment) {
                        updatedComment = FVI.removeFVLinkFromRenderedComment(updatedComment);
                        comment.replaceWith(FVITemplates.comment(updatedComment));
                        FVI.drawThreadPreviews();
                    }
                },
                error: function (data) {
                    console.error('Can not save the comment ' + data);
                    FVI.editCommentCancel(e);
                    FVI.hideSpinner(comment, textarea);
                }
            });
            return false;
        },

        editCommentCancel: function (e) {
            var comment = $(e.currentTarget).closest('.sa-comment');
            var textarea = comment.find('textarea');
            comment.find('.sa-comment-body').show();
            comment.find('.sa-comment-edit-body').hide();
            comment.find('.sa-info-icon-wrapper').remove();
            textarea.val(textarea.data('originalContent'));
            textarea.parents('form.aui').find('button.aui-button-primary').attr('disabled', true);
            return false;
        },

        showCreateThread: function () {
            var createThreadPanel = $('#sa-create-thread-wrapper');
            var threadsContainer = $('#sa-threads-container');
            var shortPanelHeight = createThreadPanel.height();
            var containerScrollTop = threadsContainer.scrollTop();

            $(this).hide();
            createThreadPanel.addClass('input-wrapper-full').find('form').show();
            createThreadPanel.find('textarea').focus();
            threadsContainer.addClass('threads-container-short');

            var largePanelHeight = createThreadPanel.height();
            threadsContainer.scrollTop(containerScrollTop + (largePanelHeight - shortPanelHeight));

            FVI.initCommentHelpIcon(createThreadPanel.find('.sa-fv-controls'));
        },

        hideCreateThread: function () {
            var createThreadPanel = $('#sa-create-thread-wrapper');
            var threadsContainer = $('#sa-threads-container');
            var largePanelHeight = createThreadPanel.height();
            var containerScrollTop = threadsContainer.scrollTop();
            var textarea = createThreadPanel.find('.sa-textarea-create-thread');

            $('#sa-new-thread-show').show();
            createThreadPanel.removeClass('input-wrapper-full').find('form').hide();
            threadsContainer.removeClass('threads-container-short');
            createThreadPanel.find('.sa-info-icon-wrapper').remove();
            createThreadPanel.find('button.aui-button-primary').attr('disabled', textarea.val() === '');

            var shortPanelHeight = createThreadPanel.height();
            threadsContainer.scrollTop(containerScrollTop - largePanelHeight + shortPanelHeight);
        },

        addReply: function (e) {
            var thread = e ? $(e.currentTarget).closest('.sa-thread-wrapper') : $(this).closest('.sa-thread-wrapper');
            var textarea = thread.find('.sa-reply textarea');
            var threadId = thread.data('threadId');
            var content = textarea.val();
            var form = thread.find('.sa-reply-form');
            var attachId = thread.data('attachmentId');

            SmartAttachAnalytics.eventCreateComment(FVI.getAttachURL(attachId));

            if (content.trim() !== '') {
                FVI.showSpinner(form, textarea);
                $.ajax({
                    type: "POST",
                    data: {
                        'threadId': threadId,
                        'attachId': attachId,
                        'content': content
                    },
                    url: AJS.params.baseURL + '/rest/attach-cat/1.0/comment/reply',
                    success: function (data) {
                        FVI.appendComment(data);
                        FVI.hideReplyControlsByThread(thread, true);
                        FVI.hideSpinner(form, textarea);
                        FVI.drawThreadPreviews();
                    },
                    error: function () {
                        FVI.hideSpinner(form, textarea);
                    }
                });
            } else {
                textarea.focus();
            }
            return false;
        },

        showReplyControls: function () {
            var thread = $(this).closest('.sa-thread-wrapper');
            var textarea = thread.find('.sa-reply-textarea');
            textarea.autosize();
            thread.find('.sa-reply-short').removeClass('sa-reply-short');
            textarea.css('height', '70px').show();
            thread.find('.sa-reply-controls').show();
            thread.find('.sa-reply-input').hide();
            FVI.initCommentHelpIcon(thread.find('.sa-reply-form .sa-fv-controls'));
            FVI.scrollReplyControls(thread);
            textarea.focus();
        },

        hideReplyControls: function () {
            var thread = $(this).closest('.sa-thread-wrapper');
            FVI.hideReplyControlsByThread(thread, true);
            return false;
        },

        hideReplyControlsByThread: function (thread, isErase) {
            var textarea = thread.find('.sa-reply-textarea');
            var value = isErase ? '' : textarea.val();

            if (value !== undefined && value.trim() === '') {
                textarea.val('').hide();
                thread.find('.sa-reply').addClass('sa-reply-short');
                thread.find('.sa-reply-controls').hide();
                thread.find('.sa-reply-input').show();
                thread.find('.sa-info-icon-wrapper').remove();
                textarea.parents('form.aui').find('button.aui-button-primary').attr('disabled', true);
            }
        },

        scrollReplyControls: function (thread) {
            var container = thread.parent();
            var replyForm = thread.find('.sa-reply-form');

            var createThreadPositionTop = $('#sa-create-thread-wrapper').position().top;
            var replyFormScrollHeight = replyForm.get(0).scrollHeight;
            var distance = createThreadPositionTop - replyForm.position().top;

            if (distance < replyFormScrollHeight) {
                container.animate({scrollTop: replyFormScrollHeight - distance + container.scrollTop()}, "fast");
            }
        },

        makeThreadActiveByEvent: function (e) {
            var thread = $(e.currentTarget);
            if (!thread.hasClass('sa-active-thread')) {
                $('#sa-threads-container').trigger('click.sa-unmake-active');
                $('.sa-active-thread').removeClass('sa-active-thread');
                var threadWrap = thread.closest('.sa-thread-wrapper');
                threadWrap.addClass('sa-active-thread');
            }
        },

        makeThreadActive: function (thread) {
            FVI.unmakeThreadActive();
            thread.closest('.sa-thread-wrapper').addClass('sa-active-thread');
        },

        unmakeThreadActiveByEvent: function (e) {
            if ($(e.target).attr("id") === 'sa-threads-container') {
                FVI.unmakeThreadActive();
            }
        },

        unmakeThreadActive: function () {
            var thread = $('.sa-active-thread');
            FVI.hideReplyControlsByThread(thread, false);
            thread.removeClass('sa-active-thread');
        },

        drawThreadPreviews: function () {
            $('#sa-threads-container').find('.sa-thread-wrapper').each(function () {
                var allComments = $(this).find('.sa-comment');
                var firstComment = allComments.first();

                var previewContainer = $(this).find('.sa-thread-preview');
                var authorHtml = firstComment.find('.sa-comment-author').html();
                var content = escapeHtml(firstComment.find('.sa-comment-body').text());
                content = content.trim().length === 0 ? "..." : content;

                previewContainer.find('.sa-thread-preview-avatar').empty().append(authorHtml);
                previewContainer.find('.sa-thread-preview-content').empty().append(content);
                previewContainer.find('.sa-thread-preview-badge .aui-badge').empty().append(allComments.length);
            });
        },

        initRevisionsDialog: function (ids, currentAttachId) {
            var container = $('#sa-file-revisions-container');
            $('#inline-dialog-sa-file-revision-dialog').remove();

            var attachments = [];

            $.each(ids, function (i, elem) {
                var attachment = FVI.getAttachmentWithClassRevision(AttachCat.getItemById(elem));
                var attachmentLink = attachment.find('a[file-preview-id]');
                var $aLinkTitle = $(attachment.find('.attachment-title a, a.attachment-title').get(0));

                attachments.unshift({
                    'id': attachmentLink.length ? attachmentLink.attr('file-preview-id') : elem,
                    'title': attachmentLink.length ? attachmentLink.attr('file-preview-title') : $aLinkTitle.text(),
                    'version': i + 1,
                    'active': currentAttachId === elem,
                    'fakeFV': attachmentLink.length === 0
                });
            });

            container.empty().append(FVITemplates.listRevisions({'attachments': attachments}));

            AJS.InlineDialog($("#sa-file-revisions"), "sa-file-revision-dialog",
                function (content, trigger, showPopup) {
                    content.html($(container).html());
                    showPopup();
                    return false;
                },
                {
                    offsetX: -1,
                    offsetY: 0
                }
            );
        },

        changeFileRevision: function () {
            var isFakeFV = $(this).attr('is-fake-fv') === "true";
            FVI.closeAllFileViewers();

            if (FVI.isFVEnabled() && !isFakeFV) {
                try {
                    var fileViewer = require('jira/fileviewer');
                    if (fileViewer !== undefined) {
                        fileViewer.addDocumentSelector('.sa-revision-item');
                        fileViewer.attachToElements($('#sa-file-revisions-container'));
                        $(this).trigger('simpleClick.fvOpen');
                        FVI.initButtons();
                    }
                } catch (err) {
                    FakeFileViewer.changeFileRevision(this);
                }
            } else {
                FakeFileViewer.changeFileRevision(this);
            }
        },

        isCanAddComments: function () {
            if (FVI.isCanAddComment === undefined) {
                FVI.isCanAddComment = $.ajax({
                        type: "GET",
                        async: false,
                        url: AJS.params.baseURL + '/rest/attach-cat/1.0/comment/canAdd',
                        data: {
                            'issueKey': AttachCat.getIssueKey()
                        },
                        success: function (data) {
                            return data;
                        }
                    }).responseText === "true";
            }
            return FVI.isCanAddComment;
        },

        redrawThreadsNavigation: function () {
            var threads = $('.sa-thread-wrapper');
            $('.sa-thread-nav-count').each(function (i) {
                $(this).empty().append(AJS.I18n.getText('attach.cat.file-viewer.thread.navigation', i + 1, threads.length));
            });
        },

        bindThreadArrowsNavigation: function () {
            $(window).on('click.sa-next-thread', '.sa-next-thread', function () {
                var activeThread = $('.sa-active-thread');
                var allThreads = $('.sa-thread-wrapper');
                var index = allThreads.index(activeThread);
                if (index < allThreads.length - 1) {
                    index++;
                    var positionTopPrevious = activeThread.position().top;
                    FVI.makeThreadActive($(allThreads.get(index)));
                    FVI.scrollToActiveThread(positionTopPrevious);
                } else {
                    FVI.makeThreadActive($(allThreads.get(0)));
                    FVI.scrollToActiveThread(0);
                }
                return false;
            });

            $(window).on('click.sa-prev-thread', '.sa-previous-thread', function () {
                var activeThread = $('.sa-active-thread');
                var allThreads = $('.sa-thread-wrapper');
                var index = allThreads.index(activeThread);
                if (index > 0) {
                    index--;
                    var positionTop = activeThread.position().top;
                    FVI.makeThreadActive($(allThreads.get(index)));
                    FVI.scrollToActiveThread(positionTop);
                } else {
                    FVI.makeThreadActive($(allThreads.get(allThreads.length - 1)));
                    FVI.scrollToActiveThread(allThreads[0].scrollHeight);
                }
                return false;
            });
        },

        scrollToActiveThread: function (positionTopPrevious) {
            var thread = $('.sa-active-thread');
            var container = thread.parent();

            var containerST = container.scrollTop();
            var positionTopNew = thread.position().top;

            container.animate({scrollTop: positionTopNew - positionTopPrevious + containerST}, "fast");
        },

        showFVAndCreateThread: function (id, attachment) {
            var attachTitle = FVI.getLinkElemFromAttachTitle(attachment.find('.attachment-title'));
            var fvparams = attachTitle.is('[file-preview-id]');

            FVI.isActiveSidebar = true;
            FVI.initButtons();

            if (FVI && fvparams) {
                $(attachment).find('.attachment-thumb a').trigger('click');
            } else {
                FakeFileViewer.open(attachment);
            }

            FVI.waitForCreateThreadButton();
        },

        showFVAndSidebar: function (attachment) {
            var attachTitle = FVI.getLinkElemFromAttachTitle(attachment.find('.attachment-title'));
            var fvparams = attachTitle.is('[file-preview-id]');

            FVI.isActiveSidebar = true;
            if (FVI && fvparams) {
                FVI.initButtons();
                $(attachment).closest('.attachment-content').find('.attachment-thumb a').trigger('click');
            } else {
                FakeFileViewer.open(attachment);
            }
        },

        openCommentByURLParams: function () {
            var vars = $.getUrlVars();
            if (vars.attachId && vars.commentId) {
                var attach = AttachCat.getItemById(vars.attachId);
                if (attach) {
                    FVI.showFVAndSidebar(attach);
                    waitForElement('#sa-threads-container .sa-comment[data-comment-id="' + vars.commentId + '"]', function (selector, context) {
                        $(context).parents('.sa-thread-wrapper').trigger('click.sa-make-active');
                        if (vars.addReply) {
                            $('.sa-active-thread .sa-reply-input').trigger('focus.sa-reply-show');
                        }
                    }, 200);
                }
            }
        },

        showSpinner: function (container, textarea) {
            textarea.prop("disabled", true);
            container.find('button').prop("disabled", true);
            container.find('.sa-comment-spinner').css('display','inline-block');
        },

        hideSpinner: function (container, textarea) {
            textarea.prop("disabled", false);
            container.find('button').prop("disabled", false);
            container.find('.sa-comment-spinner').hide();
        },

        makeIconCommentForAttachment: function () {
            var attachId = FVI.getOpenedAttachId();
            var attach = AttachCat.getItemById(attachId);

            if ($('#sa-threads-container').find('.sa-thread-wrapper').length !== 0 && attach.length) {
                attach = FVI.getAttachmentWithClassRevision(attach);
                if (attach.find('dd.attach-cat-comment.hasComment').length === 0){
                    AttachCat.markAttachmentAsHasComment(attach);
                    var revision = FVI.getAttachmentRevisionIdsContainsId(attachId);

                    if (revision.length) {
                        var latestRevisionId = revision[revision.length - 1];
                        var latestRevision = AttachCat.getItemById(latestRevisionId);
                        AttachCat.markAttachmentAsHasComment(latestRevision.not('.revision'));
                    }
                }
            }
        },

        removeIconCommentForAttachment: function (attachmentId) {
            var threadsByAttach = $('#sa-threads-container').find(".sa-thread-wrapper[data-attachment-id='"+ attachmentId+"']");

            if (threadsByAttach.length === 0) {
                var attachment = AttachCat.getItemById(attachmentId);
                var revisionIds = FVI.getAttachmentRevisionIdsExcludeCurrentId(attachmentId);

                if (revisionIds.length) {
                    if (!FVI.isAttachmentsContainComments(revisionIds)) {
                        $.each(FVI.getAttachmentRevisionIdsContainsId(attachmentId), function(i, revisionId){
                            AttachCat.unmarkAttachmentAsHasComment(AttachCat.getItemById(revisionId));
                        });
                    } else {
                        AttachCat.unmarkAttachmentAsHasComment(AttachCat.getItemById(FVI.getAttachmentWithClassRevision(attachment)));
                    }
                } else {
                    AttachCat.unmarkAttachmentAsHasComment(attachment);
                }
            }
        },

        initCommentHelpIcon: function ($commentControls) {
            var isShowNotificationInfoIcon = $('#sa-comments').data('showNotificationInfoIcon');
            var isNotificationInfoIconAbsent = $commentControls.find('.sa-info-icon-wrapper').length === 0;
            if (isShowNotificationInfoIcon && isNotificationInfoIconAbsent) {
                $commentControls.append(FVITemplates.commentNotificationHelpIcon());
                var icon = $commentControls.find('.sa-comment-info-icon');

                AJS.InlineDialog(icon, "sa-comment-info-dialog",
                    function (content, trigger, showPopup) {
                        content.html(FVITemplates.commentNotificationHelpPanel());
                        showPopup();
                        return false;
                    }, {
                        offsetX: -10
                    }
                );

                icon.on('hover', function () {
                    $(this).removeClass('animation-blink');
                });
            }
        },

        initMentionHelpIcon: function ($commentControls) {
            if ($commentControls.find('.sa-comment-mentioning-info').length === 0) {
                $commentControls.append(FVITemplates.commentMentionHelpIcon());
                var icon = $commentControls.find('.sa-comment-mentioning-info');

                AJS.InlineDialog(icon, "sa-comment-info-dialog",
                    function (content, trigger, showPopup) {
                        content.html(FVITemplates.commentMentionHelpPanel());
                        showPopup();
                        return false;
                    }, {
                        offsetX: -10
                    }
                );
            }
        },

        checkOnEmptyTextareaAndEnableButtons: function (){
            var textarea = $(this);
            var controls = textarea.parents('form.aui').find('button.aui-button-primary');
            var value = textarea.val().trim();
            controls.attr('disabled', value === '');
        },

        isAttachmentsContainComments : function (attachmentIds) {
            var isContains = false;

            $.each(attachmentIds, function (i, attachmentId) {
                var attachment = AttachCat.getItemById(attachmentId);

                if (attachment.length > 1) {
                    attachment = FVI.getAttachmentWithClassRevision(attachment);
                }

                if (attachment.find('dd.attach-cat-comment.hasComment').length) {
                    isContains = true;
                    return false;
                }
            });

            return isContains;
        },

        getAttachmentRevisionIdsContainsId: function (attachmentId) {
            var attachmentsIds = [];
            $.each(FVI.documents, function (i, revision) {
                if ($.inArray(attachmentId, revision.attachmentIds) > -1) {
                    attachmentsIds = revision.attachmentIds;
                    return false;
                }
            });
            return attachmentsIds;
        },

        getAttachmentRevisionIdsExcludeCurrentId: function (attachmentId) {
            var attachmentsIds = FVI.getAttachmentRevisionIdsContainsId(attachmentId);

            return $.grep(attachmentsIds, function (id) {
                return id !== attachmentId;
            });
        },

        getAttachmentWithClassRevision: function (attachment) {
            if (attachment.length > 1) {
                attachment = $.grep(attachment, function(elem) {
                    return $(elem).hasClass('revision');
                });
                return $(attachment);
            }
            return attachment;
        },

        isFVEnabled: function () {
            if (FVI.isFVEnabledFlag === undefined) {
                FVI.isFVEnabledFlag = $.ajax({
                        type: "GET",
                        async: false,
                        url: AJS.params.baseURL + '/rest/attach-cat/1.0/settings/isActiveFileViewer',
                        success: function (data) {
                            return data;
                        }
                    }).responseText === "true";
            }
            return FVI.isFVEnabledFlag;
        },

        getLoggedInUser: function () {
            var user = $.parseJSON($.ajax({
                type: "GET",
                async: false,
                url: AJS.params.baseURL + '/rest/api/2/myself',
                success: function (data) {
                    return data;
                }
            }).responseText);

            return {
                'avatarUrl': user.avatarUrls['24x24'],
                'fullName': user.displayName,
                'name': user.name,
                'baseUrl': AJS.params.baseURL,
                'canAddComments': FVI.isCanAddComments()
            };
        },

        closeAllFileViewers: function() {
            try {
                var fileViewer = require('jira/fileviewer');
                fileViewer.closeViewer();
            } catch (e) {}
            FakeFileViewer.hide();
        },

        getOpenedAttachId: function () {
            return parseInt($('#fv-comments').attr('attach-id'));
        },

        getAttachURL: function(attachId) {
            var attachTitle = AttachCat.getItemById(attachId).find('.attachment-title');
            return FVI.getUrlFromAttachTitle(attachTitle);
        },

        getUrlFromAttachTitle: function (attachTitle) {
            return attachTitle.attr('href') !== undefined ? attachTitle.attr('href') : attachTitle.find('a').attr('href');
        },

        getLinkElemFromAttachTitle: function (attachTitle) {
            return attachTitle.attr('href') !== undefined ? attachTitle : attachTitle.find('a');
        },

        waitForCreateThreadButton: function () {
            waitForElement('#sa-new-thread-show', function (selector, context) {
                $(context).trigger('click.sa-create-thread-show');
            }, 200);
        }
    };

    function observeDOMTitleUpdated (obj, callback) {
        var MutationObserver = window.MutationObserver || window.WebKitMutationObserver,
            eventListenerSupported = window.addEventListener;

        if (MutationObserver) {
            if (FVI.mutationObs) {FVI.mutationObs.disconnect();}

            FVI.mutationObs = new MutationObserver(function (mutations, observer) {
                if (mutations[0].addedNodes.length || mutations[0].removedNodes.length)
                    callback();
            });
            // have the observer observe foo for changes in children
            FVI.mutationObs.observe(obj, {childList: true, subtree: true});
        }
        else if (eventListenerSupported) {
            obj.addEventListener('DOMNodeInserted', callback, false);
            obj.addEventListener('DOMNodeRemoved', callback, false);
        }
    }

    function waitForElement(elementPath, callBack, iterations) {
        if ($(elementPath).length) {
            callBack(elementPath, $(elementPath));
        } else {
            iterations--;
            if (iterations > 0) {
                window.setTimeout(function () {
                    if ($(elementPath).length) {
                        callBack(elementPath, $(elementPath));
                    } else {
                        waitForElement(elementPath, callBack, iterations);
                    }
                }, 50)
            }
        }
    }

    function escapeHtml(string) {

        var entityMap = {
            "&": "&amp;",
            "<": "&lt;",
            ">": "&gt;",
            '"': '&quot;',
            "'": '&#39;',
            "/": '&#x2F;'
        };
        return String(string).replace(/[&<>"'\/]/g, function (s) {
            return entityMap[s];
        });
    }

    function resizeImageContainer() {
        if ($('.fake-file-viewer').length) {
            FakeFileViewer.resizeImageContainer();
        } else {
            FileViewerFunctions.zoomPageFit();
        }
    }

    window.FileViewerFunctions = {

        getImageDimensions: function (imageNode) {
            var source = imageNode.src;
            var imgClone = document.createElement("img");
            imgClone.src = source;
            return {imageWidth: imgClone.width, imageHeight: imgClone.height};
        },

        zoomPageFit: function () {
            var $el = $('#cp-image-preview');
            var $image = $('#cp-img');

            if ($image.length) {
                var imageDimensions = FileViewerFunctions.getImageDimensions($image.get(0));

                var viewportWidth = $el.width();
                var viewportHeight = $el.height();

                if ((viewportWidth / imageDimensions.imageWidth) < (viewportHeight / imageDimensions.imageHeight)
                    && (imageDimensions.imageWidth > viewportWidth)) {
                    FileViewerFunctions.zoomWidth($el, $image, imageDimensions);
                }
            }
        },

        zoomWidth: function ($el, $image, imageDimensions) {
            var viewportWidth = $el.width();
            var scaleFactor = viewportWidth / imageDimensions.imageWidth;
            FileViewerFunctions.changeScale(scaleFactor, $el, $image, imageDimensions);
        },

        changeScale: function (scale, $el, $image, imageDimensions) {
            var viewportWidth = $el.width();
            var viewportHeight = $el.height();
            var oldWidth = $image.width();
            var oldHeight = $image.height();
            var containerPosition = $el.find(".cp-image-container").position();

            //find the position of the pixel in the centre of the viewport
            var oldPixelCentreWidth = (viewportWidth / 2) + Math.abs(containerPosition.left);
            var oldPixelCentreHeight = (viewportHeight / 2) + Math.abs(containerPosition.top);

            $image.css("width", imageDimensions.imageWidth * scale);
            $image.css("height", imageDimensions.imageHeight * scale);

            //calculate the new pixel centre after the image has been scaled
            var newPixelCentreWidth = (oldPixelCentreWidth / oldWidth) * $image.width();
            var newPixelCentreHeight = (oldPixelCentreHeight / oldHeight) * $image.height();

            //move the scrollbar to the new pixel and then center the viewport on it
            $el.scrollLeft(newPixelCentreWidth - (viewportWidth / 2));
            $el.scrollTop(newPixelCentreHeight - (viewportHeight / 2));

            if (($el.width() < $image.width()) || ($el.height() < $image.height())) {
                $image.addClass("pannable");
            } else {
                $image.removeClass("pannable");
            }

            $image.toggleClass('pixelate', true);
            var $container = $el.find(".cp-image-container");
            $image = $el.find("#cp-img");
            $container.width($image.width());
            $container.height($image.height());

            var scalePercentage = Math.round(parseInt(scale * 100, 10));
            var $scaleInfo = $el.find('.cp-scale-info');
            $scaleInfo.text(scalePercentage + '%');
            $scaleInfo
                .stop(true, true)
                .fadeIn(50)
                .delay(400)
                .fadeOut(100);
        }
    };

})(AJS.$);