require(['jquery', 'underscore', 'smart-attachments/dialog/CreateIssueDecorator'], function ($, _, CreateIssueDialogDecorator) {
    var EXPECTED_IDS = ['prefillable-create-issue-dialog', 'create-issue-dialog'];
    var TARGET_ELEMENT_SELECTOR = '.create-issue, div.iic-widget button.iic-widget__more, .s-editor-dialog, button[aria-controls="p1-toolbar-menu"]';

    function isCreateIssueDialog(dialog) {
        return EXPECTED_IDS.indexOf(dialog.options.id) > -1;
    }

    function onDialogReady(event, dialog) {
        if (!isCreateIssueDialog(dialog)) {
            return;
        }
        var decorator = new CreateIssueDialogDecorator({dialog: dialog});
    }

    function isStructureSection($document) {
        var structureBlock = $document.find('#structure');
        return structureBlock.size() > 0;
    }

    var observer;
    var MutationConfig = {
        childList: true,
        attributes: false,
        characterData: false,
        subtree: true
    };

    function watchFormCreation(setDialogReadyCallback) {
        observer = new MutationObserver(function (mutations) {
            var subtreeMutation = _.find(mutations, function (mutation) {
                var stop = false;
                var size = mutation.addedNodes.length;
                for (var i = 0; i < size && !stop; i++) {
                    var node = mutation.addedNodes.item(i);
                    if (node.classList.contains('s-editorpanel')) {
                        console.log("Editor panel");
                        var button = node.querySelector('a.s-editor-dialog');
                        if (button) {
                            console.log('add handler');
                            button.addEventListener('click', setDialogReadyCallback)
                        }
                        stop = true;
                    }
                }
                return stop;
            });
        });
        observer.observe(document.getElementById('structure'), MutationConfig);
    }

    var attachHandler = false;

    function init() {
        var $document = $(document);
        if (document.querySelector('#issue-create')) {
            console.log('direct mode');
            var dialogStub = {
                $popupContent: $(document.querySelector('section.aui-page-panel-content')),
                options: {
                    id: 'create-issue-dialog'
                }
            };
            onDialogReady(null, dialogStub);
        } else {

            var setDialogReadyCallback = function () {
                $document.off('dialogContentReady.smart');
                $document.one('dialogContentReady.smart', onDialogReady);
                if (!attachHandler) {
                    if (isStructure) {
                        document.getElementById('create_link').addEventListener('click', setDialogReadyCallback);
                        watchFormCreation(setDialogReadyCallback);
                        attachHandler = true;
                    }
                }
            };

            var isStructure = isStructureSection($document);

            $document.on('click', TARGET_ELEMENT_SELECTOR, setDialogReadyCallback);

            var dialog = JIRA.Dialog.current;

            if (dialog && isCreateIssueDialog(dialog)) {
                setDialogReadyCallback();
            }
        }
    }

    if ($.isReady) {
        init();
    } else {
        AJS.toInit(init);
    }
});