require(['jquery', 'underscore', 'smart-attachments/dialog/CreateIssueDecorator'], function ($, _, CreateIssueDialogDecorator) {
    var EXPECTED_IDS = ['prefillable-create-issue-dialog', 'create-issue-dialog'];
    var TARGET_ELEMENT_SELECTOR = '.create-issue, div.iic-widget button.iic-widget__more, .s-editor-dialog, button[aria-controls="p1-toolbar-menu"]';

    function isCreateIssueDialog(dialog) {
        return EXPECTED_IDS.indexOf(dialog.options.id) > -1;
    }

    function onDialogReady(event, dialog) {
        if (!isCreateIssueDialog(dialog)) {
            return;
        }
        var decorator = new CreateIssueDialogDecorator({dialog: dialog});
    }

    function isStructureSection($document) {
        var structureBlock = $document.find('#structure');
        return structureBlock.size() > 0;
    }

    var bodyObserver;

    var BodyObserverMutationConfig = {
        childList: true,
        attributes: false,
        characterData: false,
        subtree: false
    };

    function watchDialogCreation(setDialogReadyCallback) {
        bodyObserver = new MutationObserver(function (mutations) {
            _.find(mutations, function (mutation) {
                var stop = false;
                var size = mutation.addedNodes.length;
                for (var i = 0; i < size && !stop; i++) {
                    var node = mutation.addedNodes[i];
                    if (node.classList) {
                        if (node.classList.contains('jira-dialog') && (node.getAttribute('id') === 'create-issue-dialog')) {
                            console.log("Pre dialog");
                            setDialogReadyCallback.apply(this);
                        }
                    }
                }
                return stop;
            });
        });
        bodyObserver.observe(document.getElementById('jira'), BodyObserverMutationConfig);
    }

    var attachHandler = false;

    function init() {
        var $document = $(document);
        if (document.querySelector('#issue-create')) {
            console.log('direct mode');
            var dialogStub = {
                $popupContent: $(document.querySelector('section.aui-page-panel-content')),
                options: {
                    id: 'create-issue-dialog'
                }
            };
            onDialogReady(null, dialogStub);
        } else {

            var setDialogReadyCallback = function () {
                $document.off('dialogContentReady.smart');
                $document.one('dialogContentReady.smart', onDialogReady);
                if (!attachHandler) {
                    if (isStructure) {
                        watchDialogCreation(setDialogReadyCallback);
                        attachHandler = true;
                    }
                }
            };

            var isStructure = isStructureSection($document);

            $document.on('click', TARGET_ELEMENT_SELECTOR, setDialogReadyCallback);

            var dialog = JIRA.Dialog.current;

            if (dialog && isCreateIssueDialog(dialog)) {
                setDialogReadyCallback();
            }
        }
    }

    if ($.isReady) {
        init();
    } else {
        AJS.toInit(init);
    }
});