define('smart-attachments/AttachmentPanel',
    [
        'jquery',
        'underscore',
        'jira/lib/class',
        'jira/util/data/meta',
        'jira/util/events',
        'jira/util/events/types',
        'jira/util/events/reasons',
        'jira/dialog/dialog-register',
        'jira/dialog/form-dialog',
        'smart-attachments/base/CommonClasses',
        'smart-attachments/base/Operations',
        'smart-attachments/AsyncModule',
        'smart-attachments/toolbox',
        'smart-attachments/dialog/BulkForm',
        'smart-attachments/dialog/VersionDecorator',
        'smart-attachments/dialog/AttachFileDecorator',
        'smart-attachments/dialog/AttachFilesDecorator',
        'smart-attachments/dialog/CtrlvDialogDecorator',
        'smart-attachments/dialog/service-desk/AttachFileDecorator',
        'smart-attachments/dialog/SendToEmail',
        'smart-attachments/model/AttachmentCategory',
        'smart-attachments/dnd/upload/WikiExecutor',
        'smart-attachments/dnd/upload/Revisions'
    ],
    function ($, _,
              Class,
              Meta,
              Events,
              Types,
              Reasons,
              Dialogs,
              FormDialog,
              CommonClasses,
              Operations,
              AsyncModule,
              Toolbox,
              BulkForm,
              VersionDialogDecorator,
              AttachFileDecorator,
              AttachFilesDecorator,
              CtrlVDialogDecorator,
              SDAttachFileDecorator,
              SendToEmailDialog,
              AttachmentCategory,
              WikiExecutor,
              Revisions) {

        function getItemId(item) {
            var link = item.find('a[href*="/secure/attachment/"]');
            if (link.length > 0) {
                return parseInt(link.attr('href').match(/\/(\d+)\//)[1]);
            }
            return void 0;
        }

        function findCategoryContainer(categoryId) {
            return function (container) {
                return container.category.id === categoryId;
            }
        }

        function findNotCategoryContainer(categoryId) {
            return function (container) {
                return container.category.id !== categoryId;
            }
        }

        function findCategoryContainerWithAttchmentId(attachmentId) {
            return function (container) {
                return container.isContainsAttachmentId(attachmentId);
            }
        }

        function mapToCategoryReducer(categories, hasComments) {
            return function (result, item) {
                var $item = $(item);
                var id = getItemId($item);
                if (id) {
                    var category = _.find(categories, function (category) {
                        return category.attachmentIds.indexOf(id) > -1
                    });
                    if (category) {
                        item.dataset['attachmentId'] = id;
                        if (hasComments.indexOf(id) > -1) {
                            item.dataset['hasComments'] = true;
                        }

                        var itemsSet = result[category.id];
                        if (typeof itemsSet === 'undefined') {
                            itemsSet = [];
                            result[category.id] = itemsSet;
                        }

                        itemsSet.push(item);
                    }
                }
                return result;
            };
        }

        var skipDialogs = ['prefillable-create-issue-dialog', 'create-issue-dialog', 'attach-file-dialog', 'upload-attach-verison-dialog'];
        var nonDropZoneDialog = ['attach-screenshot-dialog'];

        function getAttachmentId(attachmentItem) {
            return attachmentItem.getItemId();
        }

        function moveInfosBySelectedAttachments(result, source) {
            var selected = source.getSelectedItems();
            if (selected.length) {
                return result.concat(selected.map(function (attachmentItem) {
                    var moveInfo = attachmentItem.getMoveInfo();
                    moveInfo.source = source;
                    return moveInfo;
                }));
            } else {
                return result;
            }
        }

        var AttachCat = Class.extend({

            init: function ($element, issue, isAgile) {
                window.AttachCat = this;
                this.$node = $element;
                this.isServiceDesk = Toolbox.isSDProject();
                this.$document = $(document);
                this.issue = issue;
                this.getProjectKey = Toolbox.getProjectKeyFactory(this.issue.getIssueKey());
                this.isAgile = isAgile;
                if (isAgile) {
                    if (!Meta.get('issue-key')) {
                        Meta.set('issue-key', this.issue.getIssueKey());
                    }
                }
                var attachmentsSection = this.getAttachmentsSection();
                attachmentsSection.hide();
                this.categorySection = $('#attach-cat-content');
                this.categoriesContainerNode = this.categorySection.children('#attach-cat-categories');
                this.categoriesContainerNode.empty();

                if (this.isAgile) {
                    this.initGHXBoardNavigation();
                }

                this.initAttachMenuLink();

                this.$document.off('dialogContentReady.smart');
                this.$document.on('dialogContentReady.smart', this.onDialogContentReadyDefault.bind(this));
            },

            getAttachmentsSection: function () {
                return $('#attachmentmodule, #ghx-tab-attachments');
            },

            getCategoryContainer: function (categoryId) {
                return _.find(this.categoryContainers, findCategoryContainer(categoryId));
            },

            addProgressBarToCategory: function (event, progressBar, categoryId) {
                console.log('Add progress bar to category: ' + categoryId);
                var categoryContainer = _.find(this.categoryContainers, function (container) {
                    return container.category.id === categoryId;
                }, this);
                if (categoryContainer) {
                    categoryContainer.addProgressBar(progressBar);
                }
            },

            onDialogContentReadyDefault: function (event, dialog) {
                var dialogId = dialog.options.id;
                if (skipDialogs.indexOf(dialogId) < 0) {
                    console.log("Need decorate " + dialog.options.id);
                    var uploadInfo = {
                        issue: this.issue
                    };
                    var decorator = void 0;
                    if (nonDropZoneDialog.indexOf(dialogId) > -1) {
                        uploadInfo.categories = this.categories;
                        decorator = new CtrlVDialogDecorator({
                            dialog: dialog,
                            uploadInfo: uploadInfo
                        });
                    } else {
                        uploadInfo.revisionMatcher = Revisions.createFilesMatcher(this.revisionMatcher.bind(this))
                        decorator = new AttachFilesDecorator({
                            dialog: dialog,
                            uploadInfo: uploadInfo
                        });
                    }
                    if (decorator) {
                        this.onDecorateSuccess(decorator);
                    }
                }
            },

            onDialogReady: function (callback, isFake) {
                var result = $.Deferred();
                var handler = function (event, dialog) {
                    var decorator = callback.call(this, dialog);
                    if (decorator) {
                        result.resolve(decorator);
                    }
                };
                this.$document.one('dialogContentReady' + (isFake ? "-fake" : ""), handler.bind(this));
                return result;
            },

            prepareForRevision: function (categoryContainer) {
                return function (event, versionInfo) {
                    console.log('prepare for version upload');
                    var attachmentInfos = this.categoryContainers.reduce(function (collector, container) {
                        var infos = _.filter(container.getAttachmentInfo(), function (info) {
                            return info.id !== versionInfo.callerAttachId;
                        });

                        if (infos.length) {
                            collector.push({
                                category: container.category,
                                files: infos
                            });
                        }
                        return collector;
                    }, []);

                    var versioning = {
                        info: versionInfo,
                        container: categoryContainer,
                        attachmentInfos: attachmentInfos
                    };

                    var ctx = this;

                    this.onDialogReady(function (dialog) {
                        console.log('Version dialog content ready');
                        return new VersionDialogDecorator({
                            dialog: dialog,
                            issue: ctx.issue,
                            versioning: versioning,
                            isAgile: ctx.isAgile
                        })
                    }).then(this.onDecorateSuccess.bind(this));
                }
            },

            prepareForUploadToCategory: function (categoryContainer) {
                return function (event, category, issue) {
                    console.log('prepare for file upload');
                    var uploadInfo = {
                        container: categoryContainer,
                        category: category,
                        issue: issue,
                        revisionMatcher: Revisions.createFilesMatcher(this.revisionMatcher.bind(this))
                    };
                    if (this.isServiceDesk) {
                        sessionStorage.setItem('invokedFromCategory', category.id);
                        this.onDialogReady(function (dialog) {
                            var decorator = new SDAttachFileDecorator({
                                dialog: dialog,
                                uploadInfo: uploadInfo
                            });
                        }, true)
                    } else {
                        this.onDialogReady(function (dialog) {
                            return new AttachFileDecorator({
                                dialog: dialog,
                                uploadInfo: uploadInfo
                            });
                        }).then(this.onDecorateSuccess.bind(this));
                    }
                }.bind(this);
            },

            prepareForUploadFiles: function () {
                console.log('prepare for files upload');
                var uploadInfo = {
                    categories: this.categories,
                    issue: this.issue,
                    revisionMatcher: Revisions.createFilesMatcher(this.revisionMatcher.bind(this))
                };

                this.onDialogReady(function (dialog) {
                    return new AttachFilesDecorator({
                        dialog: dialog,
                        uploadInfo: uploadInfo
                    });
                }).then(this.onDecorateSuccess.bind(this));
            },

            onDecorateSuccess: function (decorator) {
                decorator.bind('submit:success', function () {
                    this.reloadView();
                }.bind(this))
            },

            revisionMatcher: function (categoryId, fileName) {
                console.log('Try match file to another attachment in category ' + categoryId);
                var categoryContainer = this.getCategoryContainer(categoryId);
                return categoryContainer.findAttachmentForRevision(fileName);
            },

            reloadView: function () {
                Toolbox.invalidateDataAndView(this.issue);
            },

            onAttachmentRemove: function (event, container, itemView) {
                this.trigger('attachment:change');
                if (itemView.isRevision) {
                    if (itemView.owner && itemView.owner.revisions.length < 2) {
                        this.reloadView();
                        return;
                    }
                }
                Events.trigger('change:attachments', [this.issue]);
            },

            onAttachmentMove: function (event, attachmentId, sourceCategoryId, targetCategoryId) {
                var source = this.getCategoryContainer(sourceCategoryId);
                var target = this.getCategoryContainer(targetCategoryId);
                if (!source || !target) {
                    return;
                }

                var attachmentView = source.getAttachmentView(attachmentId);
                if (!attachmentView) {
                    return;
                }

                var moveInfo = attachmentView.getMoveInfo();

                SmartAttachmentsAnalytics.event('attachmentToCategoryMoved', [moveInfo.attachmentItem.attachmentName]);

                var result = Operations.moveAttachment(this.issue.getIssueKey(), targetCategoryId, moveInfo.attachmentIds);
                result.then(function () {
                    source.releaseElement(moveInfo.attachmentItem);
                    target.addAttachmentElement(moveInfo.attachmentItem.element, moveInfo);
                });
            },

            onCategoryMove: function (event, position, sourceCategoryId) {
                console.log("Put category " + sourceCategoryId + " at position " + position);
                var source = this.getCategoryContainer(sourceCategoryId);
                var left = this.categoryContainers.slice(0, position);
                var right = this.categoryContainers.slice(position);
                var isCategoryNotMatch = function (container) {
                    return container.category.id !== sourceCategoryId;
                };
                if (source.position < position) {
                    left = _.filter(left, isCategoryNotMatch);
                } else {
                    right = _.filter(right, isCategoryNotMatch);
                }

                left.push(source);
                var result = left.concat(right);
                _.each(result, function (container, position) {
                    container.position = position;
                });
                if (left.length > 1) {
                    source.$node.insertAfter(left[left.length - 2].$node);
                } else {
                    right[0].$node.before(source.$node);
                }
                this.categoryContainers = result;
                Operations.saveCategoriesPosition(this.getProjectKey(), this.getIdsPositionsOfCategories());
            },

            onNewContentAdded: function (event, $ctx, reason) {
                if (reason === Reasons.panelRefreshed) {
                    var categories = this.categories;
                    var isKnownAttachment = function (attachmentId) {
                        return _.some(categories, function (category) {
                            return category.attachmentIds.indexOf(attachmentId) > -1;
                        })
                    };

                    if ($ctx) {
                        if ($ctx.attr('id') === 'attachmentmodule') {
                            // Agile board требует особого внимания =)
                            if (this.isAgile) {
                                $ctx.hide();
                            }
                        }

                        if ($ctx.hasClass('item-attachments')) {
                            console.log('Panel refreshed');
                            var attachmentElements = this.getAttachmentsSection().find('.item-attachments .attachment-content');
                            var attachmentChanges = false;
                            _.each(attachmentElements, function (item) {
                                var attachmentId = getItemId($(item));
                                if (attachmentId && !isKnownAttachment(attachmentId)) {
                                    item.dataset['attachmentId'] = attachmentId;
                                    var categoryContainer = _.find(this.categoryContainers, findCategoryContainerWithAttchmentId(attachmentId));
                                    if (categoryContainer) {
                                        categoryContainer.addAttachmentElement(item);
                                        attachmentChanges = true;
                                        this.trackAnalyticsAttachmentUpload(categoryContainer, item);
                                    }
                                }
                            }, this);
                            if (attachmentChanges) {
                                this.trigger('attachment:change');
                            }
                        }
                    }
                }

                if (reason === Reasons.inlineEditStarted) {
                    if ($ctx) {
                        var fieldId = $ctx.attr('id');
                        if (fieldId === 'type-val' || fieldId === 'assignee-val') {
                            var inlineEditForm = $ctx.find('form');
                            if (fieldId === 'assignee-val' && this.isAgile) {
                                var destroyView = function () {
                                    this.$node.find('.ops').empty();
                                    this.$node.find('.attach-cat-bulk-container').remove();
                                    this.$node.find('#attach-cat-categories').empty().append('<div id="not-initialized"></div>');
                                    Events.trigger('change:sectionView', [this.issue]);
                                }.bind(this);
                                inlineEditForm.one('before-submit', destroyView);
                            }

                            inlineEditForm.one('before-submit', this.resetDropzone.bind(this));
                        }
                    }
                }
            },

            resetDropzone: function () {
                console.log('Reset dropzone');
                var attachmentDropzone = this.getAttachmentsSection();
                var content = attachmentDropzone.find('.mod-content');
                if (content.length) {
                    content.removeClass('issue-drop-zone');
                    content.find('div.issue-drop-zone').remove();
                }
                Events.trigger('change:attachments', [this.issue]);
                return true;
            },

            attachListeners: function (pluginConfig) {
                WikiExecutor.init();
                this.addProgressBarToCategoryHandler = this.addProgressBarToCategory.bind(this);
                this.onNewContentAddedHandler = this.onNewContentAdded.bind(this);

                Events.unbind('progressBar:add.smart');
                Events.unbind(Types.NEW_CONTENT_ADDED + '.smart');
                Events.bind("progressBar:add.smart", this.addProgressBarToCategoryHandler);
                Events.bind(Types.NEW_CONTENT_ADDED + '.smart', this.onNewContentAddedHandler);

                this.categoryContainers = [];
                this.categories = pluginConfig.categories;
                this.hasCreatePermission = pluginConfig.hasCreatePermission;
                if ($('#attach-cat-categories .attach-cat-container').length === 0) {
                    if (this.isAgile) {
                        var ops = AttachCategory.agileOps({issueId: pluginConfig.issueId});
                        this.sectionOps = this.$node.find('.ops');
                        this.sectionOps.append(ops);
                    } else {
                        this.sectionOps = $('#attachment-cat-issue-section_heading .ops');
                        this.sectionOps.append(this.getAttachmentsSection().find('.ops .drop').removeClass('drop'));
                    }

                    if (pluginConfig.projectAdmin) {
                        this.sectionOps.append(AttachCategory.configButton({projectKey: this.getProjectKey()}));
                    }

                    this.draw(pluginConfig);
                    this.setBulkControls();

                    this.bindEvents();

                }
                this.initDialogs();
                this.onAssignToMeClick();

                var JIRAEvents = require("jira/util/events");
                JIRAEvents.trigger('smart-attachment.init');
                Revisions.registerMatcher(this.revisionMatcher.bind(this));
            },

            initDialogs: function () {
                var versionDialogOptions = _.clone(Dialogs.attachFile.options);
                versionDialogOptions.id = "upload-attach-verison-dialog";
                versionDialogOptions.trigger = "a.load-version";

                this.uploadRevisionDialog = new FormDialog(versionDialogOptions);
            },

            draw: function (data) {
                var itemAttachmentsSection = this.getAttachmentsSection().find('.item-attachments');
                var containerId = itemAttachmentsSection.attr('id');
                console.log('Container id: ' + containerId);
                var originalAttachmentItems = itemAttachmentsSection.find('.attachment-content');

                this.addDialogDeleteClass(originalAttachmentItems);

                _.each(this.categories, function (category) {
                    var ids = this.getDocumentAttachmentIdsFromDataCategory(category);
                    _.each(ids, function (id) {
                        if (category.attachmentIds.indexOf(id) === -1) {
                            category.attachmentIds.push(id);
                        }
                    });
                }, this);
                //sort items by category
                var mapToCategory = [];
                var reducer = mapToCategoryReducer(this.categories, data.attachmentsWithComments);

                originalAttachmentItems.toArray().reduce(reducer, mapToCategory);

                var needCollapse = function (category) {
                    return data.collapsedCategories.indexOf(category.id) > -1;
                };

                _.each(this.categories, function (category, position) {
                    var categoryId = category.id;
                    var attachmentElements = mapToCategory[categoryId];
                    var catContainer = new AttachmentCategory({
                        position: position,
                        issue: this.issue,
                        category: category,
                        categories: this.categories,
                        pluginConfig: data,
                        baseUrl: AJS.params.baseURL,
                        isJSDForm: this.isServiceDesk,
                        isAgile: this.isAgile,
                        attachmentElements: attachmentElements,
                        containerId: containerId
                    });

                    catContainer.bind('attachment:prepareForRevision', this.prepareForRevision(catContainer).bind(this));
                    catContainer.bind('attachment:prepareForUpload', this.prepareForUploadToCategory(catContainer).bind(this));
                    catContainer.bind('attachment:remove', this.onAttachmentRemove.bind(this));
                    catContainer.bind('attachment:move', this.onAttachmentMove.bind(this));
                    catContainer.bind('category:position', this.onCategoryMove.bind(this));

                    if (needCollapse(category)) {
                        catContainer.collapse();
                    }

                    this.categoryContainers.push(catContainer);
                    this.categoriesContainerNode.append(catContainer.render());
                    catContainer.attachHandlers();
                }, this);

                if (!data.hasCreatePermission) {
                    this.categorySection.addClass('not-editable')
                }

                this.$node.find('#attach-cat-help').remove();
                var $help = $(AttachCategory.help());
                this.$node.find('.mod-header').append($help);

                this.hideRestrictedAttachments(originalAttachmentItems);

                var classes;
                if (data.previewSize > 200) {

                    var str = data.previewSize.toString();
                    var width = parseInt(str.slice(0, str.length / 2));
                    var height = parseInt(str.slice(str.length / 2, str.length));

                    classes = {
                        containerHeight: (height + 42) + "px",
                        height: (height) + "px",
                        width: (width) + "px",
                        top: (-51 - height) + "px",
                        iconMargin: height > 80 ? "-40px" : (-height / 2) + "px",
                        fontSize: height > 80 ? "80px" : height + "px"
                    };
                } else {
                    classes = {
                        containerHeight: (data.previewSize + 42) + "px",
                        height: (data.previewSize) + "px",
                        width: (data.previewSize) + "px",
                        top: (-51 - data.previewSize) + "px",
                        iconMargin: data.previewSize > 80 ? "-40px" : (-data.previewSize / 2) + "px",
                        fontSize: data.previewSize > 80 ? "80px" : data.previewSize + "px"
                    };
                }
                this.categorySection.prepend(AttachCategory.classes(classes));
                console.log('Draw finished');
            },

            addDialogDeleteClass: function (originalAttachmentItems) {
                var customDeleteFormClass = "sa-form-delete-attachment-link";
                var links = "a.icon-delete, a.aui-iconfont-delete, .attachment-delete > a, a:has( > span.aui-iconfont-delete)";

                originalAttachmentItems.find(links).each(function (i, link) {
                    $(link).addClass(customDeleteFormClass);
                });
            },

            setBulkControls: function () {
                var bulkMenuItem = $(AttachCategory.bulkMenuOption());
                bulkMenuItem.find('a').on('click', this.onEnterBulkMode.bind(this));
                this.sectionOps.find('#attachment-manage-options').append(bulkMenuItem);

                if (!this.isEmpty()) {
                    bulkMenuItem.removeClass('hidden');
                }

                this.bind('attachment:change', function () {
                    console.log('change bulk status');
                    if (this.isEmpty()) {
                        bulkMenuItem.addClass('hidden');
                    } else {
                        bulkMenuItem.removeClass('hidden');
                    }
                }.bind(this));

                var bulk = AttachCategory.bulkOperations({
                    categories: this.categories,
                    attachmentsDeletable: this.categorySection.find('.icon-delete, .aui-iconfont-delete').length > 0,
                    categoriesAvailable: this.categoryContainers.length > 1,
                    hasCreatePermission: this.hasCreatePermission,
                    user: AJS.params.loggedInUser,
                    isAgile: this.isAgile
                });

                this.categorySection.append(bulk);
                if (this.categorySection.height() > 0.7 * $(window).height()) {
                    this.categorySection.prepend(bulk);
                }

                this.bulkForm = new BulkForm({
                    element: this.categorySection
                });

                this.bulkForm.bind('bulk:hide', function () {
                    bulkMenuItem.removeClass('hidden');
                    _.invoke(this.categoryContainers, 'enableDND');
                }.bind(this));

                this.bulkForm.bind('bulk:show', function () {
                    bulkMenuItem.addClass('hidden');
                    _.invoke(this.categoryContainers, 'disableDND');
                }.bind(this));

                this.bulkForm.bind('bulk:delete', this.onBulkDelete.bind(this));
                this.bulkForm.bind('bulk:download', this.onBulkDownload.bind(this));
                this.bulkForm.bind('bulk:move', this.onBulkMove.bind(this));
                this.bulkForm.bind('bulk:send', this.onBulkSend.bind(this));
                this.bulkForm.bind('bulk:archive', this.onBulkArchiveToCategory.bind(this));
            },

            onEnterBulkMode: function (event) {
                event.preventDefault();
                event.stopPropagation();
                $(document).click();
                this.bulkForm.show();
                return false;
            },

            getSelectedAttachments: function () {
                return this.categoryContainers.reduce(function (result, container) {
                    return result.concat(container.getSelectedItems());
                }, []);
            },

            onBulkDelete: function () {
                console.log('Bulk delete');
                var ids = this.getSelectedAttachments().map(getAttachmentId);
                SmartAttachmentsAnalytics.event('bulkOperationAttachmentsDeleted', [ids.length]);
                Operations.deleteDocuments(this.issue.getIssueKey(), ids)
                    .then(function () {
                        _.each(this.categoryContainers, function (conatiner) {
                            _.each(conatiner.getSelectedItems(), function (selectedItem) {
                                conatiner.releaseElement(selectedItem, true);
                            });
                        })
                    }.bind(this))
                    .always(function () {
                        this.bulkForm.hide();
                    }.bind(this));
            },

            onBulkDownload: function () {
                var ids = this.getSelectedAttachments().map(getAttachmentId);
                SmartAttachmentsAnalytics.event('bulkOperationAttachmentsDownloaded', [ids.length]);
                Operations.downloadAttachments(this.issue.getIssueKey(), ids);
                this.bulkForm.hide();
            },

            onBulkMove: function (event, targetCategoryId) {
                console.log('Bulk move to category: ' + targetCategoryId);
                var containers = _.filter(this.categoryContainers, findNotCategoryContainer(targetCategoryId));
                var moveInfos = containers.reduce(moveInfosBySelectedAttachments, []);
                var ids = moveInfos.reduce(function (result, moveInfo) {
                    return result.concat(moveInfo.attachmentIds);
                }, []);
                var result;
                if (ids.length) {
                    SmartAttachmentsAnalytics.event('bulkOperationAttachmentsToCategoryMoved', [ids.length]);
                    var target = this.getCategoryContainer(targetCategoryId);
                    var issue = this.issue;
                    result = Operations.moveAttachment(this.issue.getIssueKey(), targetCategoryId, ids);
                    result.then(function () {
                        _.each(moveInfos, function (moveInfo) {
                            var source = moveInfo.source;
                            source.releaseElement(moveInfo.attachmentItem);
                            target.addAttachmentElement(moveInfo.attachmentItem.element, moveInfo);
                        });
                        Events.trigger('change:attachments', [issue]);
                    });
                } else {
                    result = $.Deferred();
                    result.resolve({});
                }
                result.always(this.bulkForm.hide.bind(this.bulkForm));
            },

            onBulkSend: function () {
                var selected = this.getSelectedAttachments();
                var sendToEmailDialog = new SendToEmailDialog({
                    issue: this.issue,
                    items: selected
                });
                this.bulkForm.hide();
                sendToEmailDialog.show();
            },

            onBulkArchiveToCategory: function (event, targetCategoryId, fileName) {
                console.log("Archive to category " + targetCategoryId + " with fileName: " + fileName);
                var moveInfos = this.categoryContainers.reduce(moveInfosBySelectedAttachments, []);
                var idGroup = moveInfos.reduce(function (result, moveInfo) {
                    if (moveInfo.isDocument) {
                        result.documents.push(moveInfo.documentId);
                    }
                    result.attachments = result.attachments.concat(moveInfo.attachmentIds);
                    return result
                }, {documents: [], attachments: []});

                SmartAttachmentsAnalytics.event('bulkOperationAttachmentsArchived', [idGroup.attachments.length]);
                this.bulkForm.startSpinner();
                Operations.archiveToCategory(this.issue.getIssueKey(), fileName, targetCategoryId, idGroup.attachments, idGroup.documents)
                    .then(function () {
                        this.reloadView();
                        JIRA.Messages.showMsg(AJS.I18n.getText("attach.cat.archive.success"), {
                            type: JIRA.Messages.Types.SUCCESS,
                            closeable: false,
                            timeout: 3
                        });
                    }.bind(this))
                    .fail(function () {
                        JIRA.Messages.showMsg(AJS.I18n.getText("attach.cat.archive.error"), {
                            type: JIRA.Messages.Types.ERROR,
                            closeable: false,
                            timeout: 3
                        });
                    })
                    .always(function () {
                        this.bulkForm.stopSpinner();
                    }.bind(this));
            },

            onAssignToMeClick: function () {
                var hanlder = function () {
                    this.resetDropzone();
                    if (this.isAgile) {
                        GH.DetailsView.reload();
                    }
                }.bind(this);
                $(this.isAgile ? '#gh' : '#issue-content').off('click.sa-reset').on('click.sa-reset', 'a.issueaction-assign-to-me', hanlder);
            },

            bindFeedbackClickHandler: function () {
                console.info('bindFeedbackClickHandler');

                var handler = function (e) {
                    if (!window.CCFeedbackWidget) {
                        return;
                    }

                    e.preventDefault();

                    var feedbackWidgetEvent;
                    if (window.CustomEvent) {
                        feedbackWidgetEvent = new CustomEvent('toggleFeedbackWidget');
                    } else {
                        feedbackWidgetEvent = document.createEvent('CustomEvent');
                        feedbackWidgetEvent.initCustomEvent('toggleFeedbackWidget', true, true);
                    }
                    document.dispatchEvent(feedbackWidgetEvent);
                };

                $(this.isAgile ? '#gh' : '#issue-content').off('click.sa-feedback').on('click.sa-feedback', '#attach-cat-feedback-button', handler);
            },

            isEmpty: function () {
                return !_.some(this.categoryContainers, function (container) {
                    return !container.isEmpty();
                })
            },

            hideRestrictedAttachments: function (originalAttachmentItems) {
                var hrefs = [];
                var histVal = [];
                var ids = [];

                _.each(originalAttachmentItems, function (element) {
                    var item = $(element);
                    if (!item.data('attachmentId')) {
                        var id = getItemId(item);
                        if (id) {
                            ids.push();
                            item.remove();
                        }
                    }
                }, this);

                _.each(ids, function (id) {
                    hrefs.push('a[href*="/secure/attachment/' + id + '/"]');
                    histVal.push('span.hist-value:contains(' + id + ')');
                });

                if (hrefs.length) {
                    $('<style type="text/css">' + hrefs.join() + '{display: none;}</style>').appendTo("head");
                    $(hrefs.join()).parent('.activity-new-val').parent().remove();
                    $(histVal.join()).parent('.activity-old-val').parent().remove();
                    $(document).bind('DOMNodeInserted', function () {
                        $(hrefs.join()).parent('.activity-new-val').parent().remove();
                        $(histVal.join()).parent('.activity-old-val').parent().remove();
                    });
                }
            },

            bindEvents: function () {
                this.bindRemoveDocumentsFromDropdownWikiEditor();
                if (Toolbox.isFVIModuleEnabled()) {
                    FVI.init();
                }

                AsyncModule.onReady('goedit', this.bindGoEdit.bind(this));

                this.bindFeedbackClickHandler();
            },

            bindGoEdit: function () {
                console.log('Set handler for GoEdit file save');
                this.$document.off('goedit:file-saved').on('goedit:file-saved', function (e, params) {
                    var attachmentId = params[0].fileID;
                    var fileName = params[0].fileName;
                    var attachmentView;
                    _.find(this.categoryContainers, function (container) {
                        attachmentView = container.getAttachmentView(attachmentId);
                        return !!attachmentView;
                    });
                    if (attachmentView) {
                        var categoryId = attachmentView.category.id;
                        var documentId;

                        if (attachmentView.isDocument) {
                            documentId = attachmentView.getDocumentId();
                        }

                        var issue = this.issue;

                        JIRA.SmartAjax.makeRequest({
                            type: 'POST',
                            async: true,
                            url: AJS.params.baseURL + '/rest/attach-cat/1.0/attachments/goedit-integration/replace-move',
                            data: {
                                oldAttachId: attachmentId,
                                oldAttachName: fileName,
                                oldAttachCategoryId: categoryId,
                                isReplace: params[0].replace,
                                documentId: documentId,
                                parent: attachmentId,
                                issueKey: issue.getIssueKey()
                            },
                            success: _.partial(Toolbox.invalidateDataAndView, issue)
                        });
                        SmartAttachmentsAnalytics.event('attachmentWithGoEditEdited', [fileName]);
                    }
                }.bind(this));
            },

            bindRemoveDocumentsFromDropdownWikiEditor: function () {
                this.$document.off('aui-dropdown2-show.smart');
                this.$document.on('aui-dropdown2-show.smart', '.wiki-edit-dropdown-attachment', this.extendWikiEditor.bind(this));
            },

            extendWikiEditor: function (e) {
                this.removeDuplicatesFromWikiEditList(e);
            },

            removeDuplicatesFromWikiEditList: function (e) {
                var attachmentNames = this.categoryContainers.reduce(function (collector, container) {
                    container.getAttachmentInfo({withAllRevisions: true})
                        .map(function (info) {
                            return info.name;
                        })
                        .forEach(function (name) {
                            if (collector.indexOf(name) === -1) {
                                collector.push(name);
                            }
                        });
                    return collector;
                }, []);

                var dropZoneAttachmentNames = this.getFormDropZoneAttachmentNames($(e.currentTarget));
                attachmentNames = attachmentNames.concat(dropZoneAttachmentNames);

                $(e.target).find('.wiki-edit-attachment-name').each(function () {
                    var currentName = this.textContent;
                    var documentIndex = attachmentNames.indexOf(currentName);
                    if (documentIndex === -1) {
                        $(this).closest('li').remove();
                    } else {
                        attachmentNames = _.filter(attachmentNames, function (name) {
                            return name !== currentName;
                        })
                    }
                });
            },

            getFormDropZoneAttachmentNames: function ($currentTarget) {
                var formDropZone = $currentTarget.closest('form').prop('_decoratedDropZone');
                var JSDDropZone = $currentTarget.closest('.dialog2-content-container').prop('_decoratedDropZone');
                var formDropZoneAttachmentNames = this.getDropZoneTempAttachmentNames(formDropZone);
                var jsdDropZoneAttachmentNames = this.getDropZoneTempAttachmentNames(JSDDropZone);
                return formDropZoneAttachmentNames.concat(jsdDropZoneAttachmentNames);
            },

            getDropZoneTempAttachmentNames: function (dropZone) {
                var attachmentNames = [];
                if (dropZone) {
                    dropZone.getTempAttachments().forEach(function (tempAttachment) {
                        attachmentNames.push(tempAttachment.name);
                    });
                }
                return attachmentNames;
            },

            convertImageToFile: function (item) {
                var dt = item.find('dt');
                dt.find('.blender').remove();
                dt.after(item.find('dd.attachment-date'));

                if (item.find('.icon-delete, .aui-iconfont-delete').length) {
                    dt.after("<dd class='attachment-delete'></dd>");
                    item.find('dd.attachment-delete').append(dt.find('.attachment-delete').removeClass('attachment-delete'));
                } else {
                    var deleteItem = dt.find('dd.attachment-delete');
                    if (!this.isAgile) {
                        deleteItem.append('<span class="icon"></span>')
                    }
                    dt.after(deleteItem);
                }

                item.find('dd.attachment-size').after(item.find('dd.attach-cat-comment'));
            },

            initAttachMenuLink: function () {
                if (this.isServiceDesk) {
                    return;
                }

                var targetId = '#opsbar-operations_more';

                if (this.isAgile) {
                    targetId = '#gh .ghx-actions';
                }

                function getMenuItem(isAgile) {
                    var menuItem;
                    if (isAgile) {
                        menuItem = $('.js-issueaction-container-cog').find('#attach-file');
                    } else {
                        menuItem = $('#opsbar-operations_more_drop').find('#attach-file');
                    }
                    return menuItem;
                }

                var setMenuItemHandler = function (menuItem) {
                    if (menuItem.length > 0) {
                        menuItem.off('click.sa');
                        menuItem.on('click.sa', this.prepareForUploadFiles.bind(this));
                        menuItem.removeClass('unified-attach-file').addClass('issueaction-attach-file');
                    }
                }.bind(this);

                var menuButtonClickHandler = function () {
                    console.log('we wait');
                    var menuItem = getMenuItem(this.isAgile);
                    setMenuItemHandler(menuItem);
                }.bind(this);

                $(targetId).on('click', menuButtonClickHandler.bind(this));

                var menuStructureClickHandler = function (e, $layer) {
                    var menuItem = $layer.find('.unified-attach-file');
                    setMenuItemHandler(menuItem);
                }.bind(this);

                this.$document.off('click.sa-menu');
                this.$document.on('click.sa-menu', '.s-issue-actions-trigger', function () {
                    this.$document.one('InlineLayer.show', menuStructureClickHandler);
                }.bind(this));

                var $separateButton = $('.pluggable-ops a.unified-attach-file');
                if ($separateButton.length > 0) {
                    setMenuItemHandler($separateButton);
                }
            },

            initGHXBoardNavigation: function () {
                var agileIcon = $('a[href="#ghx-tab-com-stiltsoft-jira-smart-attachments-attachment-cat-issue-section-agile"]');
                if (agileIcon.length) {
                    var agileIconHref = agileIcon.attr('href');
                    var agileTabItem = $('a[href="#ghx-tab-attachments"], a[href="#attachmentmodule"]');
                    agileTabItem.attr('href', agileIconHref);
                    var section = agileTabItem.parent();
                    agileIcon.parent().before(section);
                    agileIcon.parent().remove();
                }
            },

            trackAnalyticsAttachmentUpload: function (categoryContainer, attachmentElement) {
                var $attachmentElement = $(attachmentElement);
                var attachmentName = this.getItemNameElement($attachmentElement).text();
                var tempAttachment = new CommonClasses.TempAttachment(attachmentName, categoryContainer.category.id);
                SmartAttachmentsAnalytics.event('attachmentUpload', [tempAttachment]);
            },

            markAttachmentAsHasComment: function ($attachment) {
                $attachment.find('dd.attach-cat-comment').addClass('hasComment');
            },

            unmarkAttachmentAsHasComment: function ($attachment) {
                $attachment.find('dd.attach-cat-comment.hasComment').removeClass('hasComment');
            },

            getItemId: function (item) {
                return item.find('a[href*="/secure/attachment/"]').attr('href').match(/\/(\d+)\//)[1];
            },

            getItemById: function (id) {
                return this.categorySection.find('a[href*="/secure/attachment/' + id + '/"]').parents('.attachment-content');
            },

            getItemNameElement: function (item) {
                return item.find('dl > dt > a').first();
            },

            getIssueKey: function () {
                return this.issue.getIssueKey();
            },

            getIdsPositionsOfCategories: function () {
                return this.categoryContainers.map(function (container) {
                    return container.category.id;
                });
            },

            getDocumentAttachmentIdsFromDataCategory: function (category) {
                return category.documents.reduce(function (collector, document) {
                    return collector.concat(document.attachmentIds);
                }, []);
            },

            destroy: function () {
                console.log('Destroy: ' + this.issue.getIssueKey());
                Revisions.reset();
                if (this.addProgressBarToCategoryHandler) {
                    Events.unbind("progressBar:add.smart", this.addProgressBarToCategoryHandler);
                }
                if (this.onNewContentAddedHandler) {
                    Events.unbind(Types.NEW_CONTENT_ADDED + '.smart');
                }
                this.$document.off('dialogContentReady.smart');
            }
        });


        return AttachCat;
    });