(function ($) {

    window.AttachCat = {
        body: null,
        attachmentsSection: null,
        categorySection: null,
        agileSection: null,
        categoriesContainer: null,
        categories: null,
        hasCreatePermission: null,
        issueId: null,
        catId: 0,
        sectionOps: null,
        versioning: null,
        fileSizeLimit: 5000,
        isAgile: false,
        isStructure: false,
        viewer: "",

        reinit: function() {               //JIRA 6.1 fix
            if ($('#file_attachments .attachment-content').length) {
                this.categorySection.empty();
                this.sectionOps.empty();
                $('#attach-cat-help').remove();
                this.init();
            }
        },

        init: function() {
            var t = this;
            t.body = $(document.body);
            t.attachmentsSection = $('#attachmentmodule, #ghx-tab-attachments');
            t.categorySection = $('#attach-cat-content');
            t.categoriesContainer = t.categorySection.children('#attach-cat-categories');
            t.isAgile = t.attachmentsSection.attr('id') == 'ghx-tab-attachments' || t.attachmentsSection.hasClass('ghx-detail-section');
            t.isStructure = t.categorySection.parents('#st-issue-details').length > 0;
            if (t.isAgile) {
                t.sectionOps = $('<ul class="ops attach-cat-ops"></ul>');
                t.agileSection = $('#ghx-tab-com-stiltsoft-jira-smart-attachments-attachment-cat-issue-section-agile');
                t.agileSection.find('.ghx-header').append(t.sectionOps);
            } else {
                t.sectionOps = $('#attachment-cat-issue-section_heading .ops');
            }

            JIRA.SmartAjax.makeRequest({
                type: 'GET',
                url: AJS.params.baseURL + '/rest/attach-cat/1.0/attachments',
                data: {
                    issueKey: t.getIssueKey(),
                    extraAttributes: true
                },
                success: function(data, statusText, xhr) {
                    if (xhr.status === 204) {
                        return;
                    }
                    t.issueId = data.issueId;
                    t.categories = data.categories;
                    t.hasCreatePermission = data.hasCreatePermission;
                    if ($('#attach-cat-categories .attach-cat-container').length == 0) {
                        if (t.isAgile) {
                            t.sectionOps.append(AttachCategory.agileOps({issueId: JIRA.Issue.getIssueId()}));
                        } else {
                            t.sectionOps.append(t.attachmentsSection.find('.ops .drop').removeClass('drop'));
                        }
                        t.draw(data);
                        t.customizeJiraDnD();
                        t.bindEvents();
                    }
                },
                error: function () {
                    JIRA.Messages.showMsg(AJS.I18n.getText("attach.cat.get.error"), {
                        type: JIRA.Messages.Types.ERROR,
                        closeable: false,
                        timeout: 3
                    });
                }
            });

            t.attachmentsSection.hide();
            t.initGHXBoardNavigation();

            JIRA.SmartAjax.makeRequest({
                    type: 'GET',
                    url: AJS.params.baseURL + '/rest/api/2/attachment/meta',
                    success: function(data) {
                        t.fileSizeLimit = data.uploadLimit;
                    }
            });
            t.initAttachMenuLink();
        },

        customizeJiraDnD: function () {

            if (this.isServiceDeskAttachmentModuleExist()) {
                return;
            }

            var t = this;

            if (t.categories === null) {
                setTimeout(function () {
                    AttachCat.customizeJiraDnD();
                }, 100);
                return;
            }

            $('.wrap-categories-container').remove();

            if (t.isAgile) {
                try {
                    var AttachmentsDropZone = require('dndattachment/dropzones/AttachmentsDropZone');
                    AttachmentsDropZone.prototype.connectContainer = function () {};
                    AttachmentsDropZone.prototype.disconnectContainer = function () {};
                } catch (err) {
                }

                var dropZone = $('.attachments-drop-zone__dragover-border');

                if (!dropZone.length) {
                    $('#gh').append(AttachCategory.dragAndDropOverlay());
                }
            }

            var $context = $('.attachments-drop-zone__dragover-border');
            $context.append(AttachCategory.container());
            var catContainer = $context.find('#categories-container');

            AttachDropSections.createSections(window, catContainer, t.categories, AttachCategory.categoryInDnD);
        },

        draw: function(data) {
            var t = this;
            var attachFiles = t.attachmentsSection.find('#file_attachments .attachment-content');
            var attachImages = t.attachmentsSection.find('#attachment_thumbnails .attachment-content');
            var isJSDForm = t.isServiceDeskAttachmentModuleExist();

            AttachCat.addDialogDeleteClass(attachFiles, attachImages);

            $.each(t.categories, function(i, category) {
                var catContainer = $(AttachCategory.category({
                    category: category,
                    issueId : t.issueId,
                    issueKey: t.getIssueKey(),
                    hasCreatePermission: data.hasCreatePermission,
                    baseUrl: AJS.params.baseURL,
                    isJSD: isJSDForm
                }));
                t.categoriesContainer.append(catContainer);

                category.attachmentIds = $.merge(category.attachmentIds, t.getDocumentAttachmentIdsFromDataCategory(category));

                if (category.attachmentIds.length) {
                    var categories = $.extend(true, [], t.categories);
                    categories.splice(i, 1);

                    var catFiles = catContainer.find('.attach-files');
                    attachFiles = t.addCatItems(attachFiles, category, catFiles, catContainer, data, categories);

                    var catImages = catContainer.find('.attach-images');
                    attachImages = t.addCatItems(attachImages, category, catImages, catContainer, data, categories);
                }

                if (catContainer.find('.attachment-content').length === 0) {
                    t.hideCategory(catContainer);
                }
            });

            t.applyUserSettings(data.collapsedCategories);

            if (!data.hasCreatePermission) {
                t.categorySection.addClass('not-editable')
            }

            if (data.projectAdmin) {
                t.sectionOps.append(AttachCategory.cog({projectKey: t.getProjectKey()}));
            }

            $('#attach-cat-help').remove();
            var $help = $(AttachCategory.help());
            $('#attachment-cat-issue-section_heading, #ghx-tab-com-stiltsoft-jira-smart-attachments-attachment-cat-issue-section-agile .ghx-header').append($help);

            if (data.showFD) {
                var popup = AJS.InlineDialog($help, "attach-cat-feature-discovery", function(contents, trigger, showPopup) {
                    contents.html(AttachCategory.featureDiscovery()) ;
                    showPopup();
                }, {noBind: true, width: 400, initCallback: t.bindFeatureDiscoveryEvents, hideCallback: function() {
                    JIRA.SmartAjax.makeRequest({
                        type: 'POST',
                        url: AJS.params.baseURL + '/rest/attach-cat/1.0/attachments/fd'
                    });
                }});

                popup.show();
            }

            t.setComments(data.attachmentsWithComments);
            t.setDocuments(data.categories);
            t.setBulkControls();

            if (t.isAgile) {
                t.attachmentsSection.remove();
            }

            t.hideRestrictedAttachments(attachFiles, attachImages);

            var classes;
            if (data.previewSize > 200) {

                var str = data.previewSize.toString();
                var width = parseInt(str.slice(0, str.length / 2));
                var height = parseInt(str.slice(str.length / 2, str.length));

                classes = {
                    containerHeight: (height + 42) + "px",
                    height: (height) + "px",
                    width: (width) + "px",
                    top: (-51 - height) + "px",
                    iconMargin: height > 80 ? "-40px" : (-height / 2) + "px",
                    fontSize: height > 80 ? "80px" : height + "px"
                };
            } else {
                classes = {
                    containerHeight: (data.previewSize + 42) + "px",
                    height: (data.previewSize) + "px",
                    width: (data.previewSize) + "px",
                    top: (-51 - data.previewSize) + "px",
                    iconMargin: data.previewSize > 80 ? "-40px" : (-data.previewSize / 2) + "px",
                    fontSize: data.previewSize > 80 ? "80px" : data.previewSize + "px"
                };
            }
            t.categorySection.prepend(AttachCategory.classes(classes));


            var galleryImg = t.categoriesContainer.find('.gallery img');
            if (galleryImg.length) {
                galleryImg.attr('height', '');
                galleryImg.attr('width', '');
            }

            t.drawBadges();
        },

        drawBadges: function () {
            $('div.attach-cat-container').each(function () {
                var t = $(this);
                var length = t.find('.attachment-content').not('.revision').length;
                t.find('.aui-badge').empty().append(length);
            });
        },

        addDialogDeleteClass: function (attachFiles, attachImages) {
            var customDeleteFormClass = "sa-form-delete-attachment-link";
            var attachments = attachFiles.add(attachImages);
            var links = "a.icon-delete, a.aui-iconfont-delete, .attachment-delete > a, a:has( > span.aui-iconfont-delete)";

            attachments.find(links).each(function (i, link) {
                $(link).addClass(customDeleteFormClass);
            });
        },

        addCatItems: function(items, category, itemsList, catContainer, data, categories) {
            var t = this;
            var lastAttaches = [];

            $.each(items, function(i, item) {
                var $item = $(item);
                var id = parseInt(t.getItemId($item));

                if ($.inArray(id, category.attachmentIds) > -1) {
                    var nameElement = t.getItemNameElement($item);
                    var href = nameElement.attr('href');
                    var attachmentName = t.getAttachmentNameForEditInOffice(nameElement);
                    var optionsMenu = t.makeAttachmentOptionsMenu(data, categories, href, id, attachmentName, catContainer);
                    t.changeHoverAttachmentTitle(item);
                    t.addItem(itemsList, catContainer, $item, optionsMenu);
                } else {
                    lastAttaches.push(item);
                }
            });

            return lastAttaches;
        },

        makeAttachmentOptionsMenu: function(data, categories, href, id, attachmentName, catContainer) {
            var t = this;
            return data.hasCreatePermission ?
                AttachCategory.attachmentOptionsMenu({
                    categories: categories,
                    issueId: t.issueId,
                    canAddComment: isFVIModuleEnabled() && data.canAddComments,
                    officeType: EditInOffice.getType(href),
                    id: id,
                    name: attachmentName,
                    cat: catContainer.attr('id'),
                    issueKey: t.getIssueKey(),
                    isSupported: EditInOffice.isSupportedBrowser(),
                    user: AJS.params.loggedInUser
                }) : null;
        },

        addItem: function(catList, catContainer, item, attachmentOptionsMenu) {
            var t = this;

            catList.append(item);
            t.decorateAttachmentOptionsMenu(item, attachmentOptionsMenu);
            catContainer.find('.attach-cat-empty').hide();
            catContainer.find('.attach-cat-download-all').show();
        },

        decorateAttachmentOptionsMenu: function (item, attachmentOptionsMenu) {
            var deleteActions = item.find('.attachment-delete');
            if (!deleteActions.length) {
                deleteActions = $('<dd class="attachment-delete"/>');
                item.find('.attachment-title').before(deleteActions);
            }

            deleteActions = item.find('.attachment-delete').clone();

            AttachCat.decorateAttachmentOptionsMenuForListView(item, attachmentOptionsMenu);
            AttachCat.decorateAttachmentOptionsMenuForGridView(item, attachmentOptionsMenu, deleteActions);
        },

        decorateAttachmentOptionsMenuForGridView: function(item, attachmentOptionsMenu, deleteActions) {
            var attachHeaderLine = $('<div class="sa-attach-header-line"></div>');
            item.find('dl').append(attachHeaderLine);
            attachHeaderLine.append(deleteActions);

            if (attachmentOptionsMenu) {
                attachHeaderLine.append($(attachmentOptionsMenu));
            }
        },

        decorateAttachmentOptionsMenuForListView: function(item, attachmentOptionsMenu) {
            if (!attachmentOptionsMenu) {
                return;
            }
            item.find('.attachment-delete').prepend($(attachmentOptionsMenu));
        },

        applyUserSettings: function(collapsedCategories) {
            var t = this;
            $.each(collapsedCategories, function(i, collapsedCategoryId) {
                t.hideCategory(t.categorySection.find('#' + collapsedCategoryId));
            });
        },

        setComments: function(attachmentsWithComments) {
            var t = this;

            if (isFVIModuleDisabled()) {
                return;
            }

            $(t.categoriesContainer).find('li.attachment-content').each(function(i, attachmentElemLi){
                t.addCommentIcon($(attachmentElemLi));
            });

            $.each(attachmentsWithComments, function(i, attachId) {
                var attachment = t.getItemById(attachId);
                if (attachment.length) {
                    t.markAttachmentAsHasComment(attachment);
                }
            });
        },

        setBulkControls: function() {
            var t = this;

            if (t.categorySection.find('.attachment-content').length > 1) {
                var catTitles = t.categorySection.find('.attach-cat-title');
                if (catTitles.length > 1) {
                    catTitles.each(function() {
                        $(this).prepend(AttachCategory.bulkCheckbox());
                    });
                }

                var bulk = AttachCategory.bulkOperations({
                    categories: t.categories,
                    attachmentsDeletable: t.categorySection.find('.icon-delete, .aui-iconfont-delete').length > 0,
                    categoriesAvailable: $('.attach-cat-container').length > 1,
                    hasCreatePermission: t.hasCreatePermission,
                    user: AJS.params.loggedInUser,
                    isAgile: t.isAgile
                });

                t.categorySection.append(bulk);
                if ( t.categorySection.height() > 0.7 * $(window).height()) {
                    t.categorySection.prepend(bulk);
                }

                $('#attachment-manage-options').append(AttachCategory.bulkMenuOption())
            }
        },

        setDocuments: function (categories) {
            var t = this;
            var documents = [];

            $.each(categories, function (i, category) {
                documents = $.merge(documents, category.documents);
            });

            if (!isFVIModuleDisabled()) {
                FVI.documents = documents;
            }

            $.each(documents, function(n, document) {
                var latest;
                var latestId;

                for (var i = document.attachmentIds.length - 1; i >= 0; i--) {
                    latestId = document.attachmentIds[i];
                    latest = t.getItemById(document.attachmentIds[i]);
                    if (latest.length) {
                        break;
                    }
                }

                if (latest.length == 0) {
                    return true;
                }

                var isLatestFile = t.isItemFile(latest);

                if (!isLatestFile) {
                    latest.parents('.attach-cat-container').children('.item-attachments.attach-files').append(latest);
                    t.convertImageToFile(latest);
                }

                var hasComments = false;
                var deletable = latest.find('.icon-delete, .aui-iconfont-delete').length > 0;

                $.each(document.attachmentIds, function(i, id) {
                    if (latestId != id) {
                        var item = t.getItemById(id);
                        if (item.length) {
                            t.decorateRevision(item, document);
                            deletable = deletable && item.find('.icon-delete, .aui-iconfont-delete').length > 0;

                            if (!t.isItemFile(item)) {
                                t.convertImageToFile(item);
                            }

                            latest.after(item);

                            if (item.find('dd.attach-cat-comment.hasComment').length) {
                                hasComments = true;
                            }
                        }
                    }
                });

                var documentNode = latest.clone();
                var nameElement = t.getItemNameElement(documentNode);
                nameElement.html(document.name);
                documentNode.data("document", true).data('documentId', document.id);
                documentNode.prepend(AttachCategory.versionExpander({id: document.id}));
                t.removeArchiveExpandIconForRevision(documentNode);

                if (documentNode.find('.attach-cat-options').length == 0) {
                    t.decorateAttachmentOptionsMenuForListView(documentNode, AttachCategory.docDownloadOpts());
                    documentNode.find('.attachment-delete span.icon').remove();
                }

                documentNode.find('.attach-cat-options .aui-dropdown-content.aui-list').data('document', true)
                    .find('.download-doc').parent().show();
                if (!deletable) {
                    documentNode.find('.icon-delete, .aui-iconfont-delete').replaceWith('<span class="icon"></span>');
                }
                latest.before(documentNode);
                t.decorateRevision(latest, document);

                if (!isLatestFile && documentNode.data('attachmentType') == 'image' && isFVIModuleDisabled()) {
                    var linkTitle = documentNode.find('a.gallery, a.attachment-title');
                    if (linkTitle.first().attr("file-preview-id") === undefined) {
                        linkTitle.fancybox({
                            type: "image",
                            centerOnScroll: true,
                            transitionIn: 'elastic',
                            onStart : AJS.disableKeyboardShortcuts,
                            onClosed : AJS.enableKeyboardShortcuts
                        });
                    }
                }

                t.bindDocumentDelete(documentNode);
                t.bindDownLoadDoc(documentNode);

                if (hasComments) {
                    t.markAttachmentAsHasComment($(documentNode));
                }

                t.goeditPopoverInit(latest);
                t.goeditPopoverInit(documentNode);
            });
        },

        decorateRevision: function(item, document) {
            item.addClass('revision ' + document.id);
            item.data('revisionId', document.id);
            item.find('.attach-cat-bulk-checkbox-container').remove();
            var opts = item.find('.attach-cat-options .aui-dropdown-content.aui-list');
            opts.find('.aui-list-section.move-to-cat').remove();
            opts.find('.load-version').parent().remove();
            opts.find('.edit-office').parent().remove();
        },

        hideRestrictedAttachments: function(attachFiles, attachImages) {
            var t = this;
            var hrefs = [];
            var histVal = [];
            var ids = [];

            $.each(attachFiles, function() {
                var item = $(this);
                ids.push(t.getItemId(item));
                item.remove();
            });
            $.each(attachImages, function() {
                var item = $(this);
                ids.push(t.getItemId($(this)));
                item.remove();
            });

            $.each(ids, function(i, id) {
                hrefs.push('a[href*="/secure/attachment/' + id + '/"]');
                histVal.push('span.hist-value:contains(' + id + ')');
            });

            if (hrefs.length) {
                $('<style type="text/css">' + hrefs.join() + '{display: none;}</style>').appendTo("head");
                $(hrefs.join()).parent('.activity-new-val').parent().remove();
                $(histVal.join()).parent('.activity-old-val').parent().remove();
                $(document).bind('DOMNodeInserted', function() {
                    $(hrefs.join()).parent('.activity-new-val').parent().remove();
                    $(histVal.join()).parent('.activity-old-val').parent().remove();
                });
            }
        },

        changeHoverAttachmentTitle: function (item) {
            var titleLink = $(item).find('dt.attachment-title a');
            if (titleLink.length) {
                var currentTitle = titleLink.attr('title');
                var attachmentName = titleLink.text();
                if (currentTitle.indexOf(attachmentName) < 0) {
                    titleLink.attr('title', attachmentName + ' - ' + currentTitle);
                }
            }
        },

        bindEvents: function() {
            var t = this;

            JIRA.Dropdowns.bindGenericDropdowns($('#attachment-cat-issue-section, #ghx-tab-com-stiltsoft-jira-smart-attachments-attachment-cat-issue-section-agile'));
            t.bindAttachFile();
            t.bindRenameFile();
            t.bindLoadVersion();
            t.bindEditInOffice();
            t.bindMoveCats();
            t.bindHideShowCats();
            t.bindCommentEvents();
            t.bindBulkOperations();
            SADraggable.bind();
            t.bindFixEvents();
            t.bindDocumentExpandEvents();
            t.bindDialogExtendEvents();
            t.bindGoEdit();
            t.bindRemoveDocumentsFromDropdownWikiEditor();
            if (isFVIModuleEnabled()) {
                FVI.init();
            }
        },

        bindAttachFile: function() {
            var t = this;

            if (t.categories.length < 1) {
                return;
            }

            $('.attach-cat-add').click(function() {
                t.catId = $(this).data('cat');
            });
        },

        bindRenameFile: function() {
            var t = this;

            $('.attach-cat-options .rename-attach').click(function() {
                var a = $(this);
                var opts = a.parents('.aui-list');
                var isDoc = opts.data('document');
                var id = opts.data('attachId');
                var item = t.getItemById(id);
                var documentId;

                if (isDoc) {
                    item = item.not('.revision');
                    documentId = item.data('documentId');
                } else if (item.length > 1) {
                    item = item.filter('.revision');
                }

                var nameElement = t.getItemNameElement(item);
                var oldName = nameElement.html().trim();

                $('#rename-attach-dialog').remove();

                var dialog = new AJS.Dialog({
                    width: 600,
                    height: isDoc ? 180 : 230,
                    id: "rename-attach-dialog",
                    closeOnOutsideClick: false
                });

                dialog.addHeader(isDoc ? AJS.I18n.getText("attach.cat.rename.document.dialog.title") :
                    AJS.I18n.getText("attach.cat.rename.dialog.title"));

                var body = AttachCategory.renameDialog({name: $('<div/>').html(oldName).text(), isDoc: isDoc});

                dialog.addPanel('panel', body, 'rename-attach-dialog-panel');

                dialog.addButton(AJS.I18n.getText("attach.cat.rename.dialog.save"), function() {
                    SmartAttachAnalytics.eventAttachmentRename();

                    var name = $('#attach-new-name').val();

                    JIRA.SmartAjax.makeRequest({
                        type: 'POST',
                        url: AJS.params.baseURL + '/rest/attach-cat/1.0/attachments/rename',
                        data: {
                            issueKey: t.getIssueKey(),
                            id: id,
                            catId: item.parents('.attach-cat-container').attr('id'),
                            name: name,
                            documentId: documentId
                        },
                        success: function() {
                            isDoc ? JIRA.trigger(JIRA.Events.REFRESH_ISSUE_PAGE, [JIRA.Issue.getIssueId()]) :  location.reload();
                            dialog.remove();
                        }
                    });
                }, "save-attach-rename");

                dialog.addLink(AJS.I18n.getText("attach.cat.rename.dialog.cancel"), function (dialog) {
                    $('#inline-dialog-attach-rename-error').remove();
                    dialog.remove();
                }, '#');

                $(document).click();
                dialog.show();

                $('#attach-new-name')
                    .bind('input', AttachCat.bindRenameValidation)
                    .bind('keydown', function(e) {
                        if (e.keyCode == 13) {
                            if (!$(".save-attach-rename").prop("disabled")) {
                                $('.save-attach-rename').click();
                            }
                            return false;
                        }
                    });

                AttachCat.bindRenameValidationErrorDialog();

                return false;
            });
        },

        bindRenameValidation: function () {
            var notPermittedCharacters = ['#', '%', '&', '{', '}', '\\', '<', '>', '*', '?', '/', '$', '!', '‘', '“', ':', '@', '+', '`', '|', '='];
            var input = $('#attach-new-name');
            var value = input.val().trim();
            var buttonSave = $(".save-attach-rename");
            var iconOk = $('#attach-rename-ok');
            var iconError = $('#attach-rename-error');

            if (value.length !== 0) {
                var error = false;
                $.each(notPermittedCharacters, function () {
                    if (value.indexOf(this) !== -1) {
                        error = true;
                        return false;
                    }
                });
                if (error) {
                    iconOk.hide();
                    iconError.css('display', 'inline-block');
                    input.css('border-color', '#d04437');
                } else {
                    iconOk.css('display', 'inline-block');
                    iconError.hide();
                    input.css('border-color', '#cccccc');
                }
                buttonSave.prop("disabled", error);
            } else {
                buttonSave.prop("disabled", true);
                iconOk.hide();
                iconError.css('display', 'inline-block');
                input.css('border-color', '#d04437');
            }
        },

        bindRenameValidationErrorDialog: function () {
            AJS.InlineDialog($("#attach-rename-error"), "attach-rename-error",
                function (content, trigger, showPopup) {
                    var data = {
                        symbols: '# % & { } \\ < > * ? / $ ! ‘ “ : @ + ` | =',
                        isEmpty: $('#attach-new-name').val().trim().length === 0
                    };
                    content.html(AttachCategory.renameDialogError(data));
                    showPopup();
                    return false;
                }, {
                    offsetX: -10,
                    onHover: true,
                    gravity: 's'
                }
            );
        },

        bindDownLoadDoc: function(item) {
            var t = this;

           item.find('.attach-cat-options .download-doc').click(function() {
                var a = $(this);
                var opts = a.parents('.aui-list');
                var item = t.getItemById(opts.data('attachId')).not('.revision');

                downloadFileFromPath(AJS.params.baseURL + '/rest/attach-cat/1.0/attachments/download-doc?' +
                    'issueKey=' + t.getIssueKey() +
                    "&catId=" + item.parents('.attach-cat-container').attr('id') +
                    "&docId=" + item.data('documentId'));

                $(document).click();
                return false;
            });
        },

        bindLoadVersion: function() {
            var t = this;

            $('.attach-cat-options .load-version').click(function(e) {
                SmartAttachAnalytics.eventUploadNewRevision();

                var a = $(this);
                var id = a.parents('.aui-list').data('attachId');
                var item = t.getItemById(id).not('.revision');
                var nameElement = t.getItemNameElement(item);
                var name = nameElement.text().trim();
                var catId = item.parents('.attach-cat-container').attr('id');
                t.versioning = {
                    parent: id,
                    name: name,
                    catId: catId,
                    isDoc: item.data('document'),
                    documentId: item.data('documentId'),
                    callerAttachId: id
                };
            });
        },

        bindEditInOffice: function() {
            var t = this;

            $('.attach-cat-options .edit-office').click(EditInOffice.clickHandler);

            $('.edit-in-office-refrefh').live('click', function() {
                if (t.isAgile) {
                    GH.DetailsView.reload();
                } else {
                    JIRA.trigger(JIRA.Events.REFRESH_ISSUE_PAGE, [JIRA.Issue.getIssueId()]);
                }
                $(this).parent().children('.icon-close').click();
                return false;
            });

            if (EditInOffice.isSupportedBrowser()) {
                $('.attach-cat-options').click(function() {
                    var link = $('.edit-office:visible');
                    if (link.length) {
                        JIRA.SmartAjax.makeRequest({
                            type: 'GET',
                            url: AJS.params.baseURL + '/rest/attach-cat/1.0/attachments/lock',
                            data: {
                                id: link.data('id'),
                                catId: t.getItemById(link.data('id')).parents('.attach-cat-container').attr('id'),
                                issueKey: t.getIssueKey()
                            },
                            success: function(data) {
                                if (data) {
                                    link.addClass('disabled').attr('title', AJS.I18n.getText("attach.cat.edit.office.locked"));
                                } else {
                                    link.removeClass('disabled').attr('title', AJS.I18n.getText("attach.cat.edit.office"));
                                }
                            }
                        });
                    }
                });
            }
        },

        bindMoveCats: function() {
            var t = this;

            $('.attach-cat-options .move-to-cat').click(function() {
                var a = $(this);
                var opts = a.parents('.aui-list');
                var isDoc = opts.data('document');
                var newId = a.attr('id');
                var item = t.getItemById(opts.data('attachId'));
                if (isDoc) {
                    item = item.not('.revision');
                }
                var oldContainer = item.parents('.attach-cat-container');
                var oldId = oldContainer.attr('id');
                var target = t.isItemFile(item) ? "files" : "images";

                t.move(oldId, newId, t.categorySection.find('#' + newId), oldContainer, item, target);
                $(document).click();
                return false;
            });
        },

        bindHideShowCats: function() {
            var t = this;

            $('.attach-cat-title').click(function() {
                var parent = $(this).parent();
                t.toggleHideCategory(parent);

                if (parent.find('.attachment-content').length) {
                    JIRA.SmartAjax.makeRequest({
                        type: 'POST',
                        url: AJS.params.baseURL + '/rest/attach-cat/1.0/attachments/settings',
                        data: {
                            projectKey: t.getProjectKey(),
                            id: parent.attr('id'),
                            value: parent.hasClass('collapsed')
                        }
                    });
                }

            });
        },

        bindCommentEvents: function () {
            if (isFVIModuleDisabled()) {
                return;
            }

            $(window)
                .off('click.sa-icon-view-comment')
                .on('click.sa-icon-view-comment', '.attach-cat-comment a.aui-iconfont-comment', function () {
                    FVI.showFVAndSidebar($(this).parents('.attachment-content'));
                })
                .off('click.sa-icon-add-comment')
                .on('click.sa-icon-add-comment', '.attach-cat-comment a.aui-iconfont-add-comment', function () {
                    var attachment = $(this).parents('.attachment-content');
                    var id = AttachCat.getItemId(attachment);
                    FVI.showFVAndCreateThread(id, attachment);
                });

            $('a.add-attach-comment')
                .off('click.sa-menu-add-comment')
                .on('click.sa-menu-add-comment', function (e) {
                e.preventDefault();

                var id = $(this).parents('.aui-list').data('attachId');
                var attachment = AttachCat.getItemById(id);
                FVI.showFVAndCreateThread(id, attachment);
            });
        },

        dropFileToCategory: function (e, $targetCategory) {
            var t = AttachCat;

            if (AttachCat.categories.length === 0) {
                showAutocloseableWarningMessage(AJS.I18n.getText("attach.cat.files.uploading.categories.restricted"));
            }

            if (t.hasCreatePermission) {

                var files = e.originalEvent !== undefined ? e.originalEvent.dataTransfer.files : e.dataTransfer.files;
                if (files && files.length) {
                    SmartAttachAnalytics.eventAttachDND();

                    var category = t.checkOnExistTargetCat($targetCategory);

                    if (!category.length && !t.categories) {
                        var warnMessage = AJS.I18n.getText("attach.cat.files.uploading.default.category.restricted");
                        showAutocloseableWarningMessage(warnMessage);
                    }

                    var uploadingHint = JIRA.Messages.showMsg(
                        AJS.I18n.getText("attach.cat.files.uploading", category.find('.attach-cat-title').text()),
                        {
                            type: JIRA.Messages.Types.WARNING,
                            timeout: -1
                        }
                    );

                    var categoryId = category.attr('id');

                    var filesName = [];
                    var fileIDs = [];
                    var count = files.length;
                    var progressFinished = $('#attach-cat-upload-progress-finished');
                    var progressLeft = $('#attach-cat-upload-progress-left');
                    var total = files.length + 2;
                    var finished = 1;

                    function updateProgress() {
                        var width = Math.round(100 * finished++ / total);
                        progressFinished.width(width + "%");
                        progressLeft.width((100-width) + "%");
                    }

                    $.each(files, function(i, file) {
                        updateProgress();

                        if (file.size > t.fileSizeLimit) {
                            count--;
                            $('#attach-cat-msg-container').append(
                                JIRA.Messages.buildMsg(
                                    AJS.I18n.getText("upload.too.big", file.name, AJS.InlineAttach.Text.fileSize(file.size),
                                        AJS.InlineAttach.Text.fileSize(t.fileSizeLimit)), {
                                        type: JIRA.Messages.Types.ERROR,
                                        closeable: true,
                                        timeout: 5
                                    }));
                            if (count == 0) {
                                uploadingHint.remove();
                            }

                            return true;
                        }

                        SmartAttachAnalytics.eventUploadFile(file.name);

                        new AJS.InlineAttach.AjaxUpload({
                            file: file,
                            params: {
                                filename: file.name,
                                size: file.size,
                                atl_token: atl_token(),
                                issueId: JIRA.Issue.getIssueId(),
                                formToken: ""
                            },
                            url: AJS.InlineAttach.AjaxPresenter.DEFAULT_URL,
                            success: function(val) {
                                if (val.id) {
                                    fileIDs.push(val.id);
                                    filesName.push(val.name);
                                } else {
                                    count--;
                                    if (val.errorMessage) {
                                        $('#attach-cat-msg-container').append(
                                            JIRA.Messages.showMsg(val.errorMessage, {
                                                type: JIRA.Messages.Types.ERROR,
                                                closeable: true,
                                                timeout: 5
                                            }));
                                    }
                                }

                                if (fileIDs.length == count && count > 0) {
                                    commit();
                                } else if (count == 0) {
                                    uploadingHint.remove();
                                }
                            }
                        }).upload();
                    });

                    function commit() {
                        updateProgress();

                        if (categoryId && categoryId > 0) {
                            var tempAttachments = AttachCat.convertAttachNamesToTempAttachments(filesName, categoryId);

                            JIRA.SmartAjax.makeRequest({
                                type: 'POST',
                                async: false,
                                url: AJS.params.baseURL + '/rest/attach-cat/1.0/attachments',
                                data: {
                                    issueKey: t.getIssueKey(),
                                    tempAttachments: JSON.stringify(tempAttachments)
                                }
                            });
                        }

                        updateProgress();

                        JIRA.SmartAjax.makeRequest({
                            type: "POST",
                            url: AJS.params.baseURL + '/secure/AttachFile.jspa',
                            data: {
                                inline: true,
                                id: JIRA.Issue.getIssueId(),
                                filetoconvert: fileIDs,
                                atl_token: atl_token()
                            },
                            success: function() {
                                if (t.isAgile) {
                                    if (!$targetCategory.hasClass('ghx-detail-contents')) {
                                        GH.DetailsView.reload();
                                    }
                                } else {
                                    JIRA.trigger(JIRA.Events.REFRESH_ISSUE_PAGE, [JIRA.Issue.getIssueId()]);
                                }
                                uploadingHint.remove();

                                var targetCategoryName = category.find('.attach-cat-title').text();

                                if (targetCategoryName !== "") {
                                    var message = AJS.I18n.getText("attach.cat.files.uploaded", targetCategoryName);
                                    JIRA.Messages.showMsg(message, {
                                        type: JIRA.Messages.Types.SUCCESS,
                                        closeable: true,
                                        timeout: 5
                                    });
                                }
                            }
                        });
                    }
                }
            } else {
                JIRA.Messages.showMsg(AJS.I18n.getText("attach.cat.files.uploading.restricted"), {
                    type: JIRA.Messages.Types.WARNING,
                    timeout: 5,
                    closeable: true
                });
            }
        },

        bindBulkOperations: function() {
            var t = this;
            var bulkContainer = $('.attach-cat-bulk-container');
            var startBtn = $('#attach-cat-start-bulk');
            var opSelect = $('.attach-cat-bulk-op');
            var catSelect = $('.attach-cat-bulk-move');
            var saveBtn = $('.attach-cat-save-bulk');
            var cancelBtn = $('.attach-cat-cancel-bulk');
            var catChkBox = $('.attach-cat-title .attach-cat-bulk-checkbox');
            var zipName = $('.attach-cat-zip-name');

            zipName.bind('input', function() {
                zipName.val($(this).val());
            });

            var enableConfirm = function() {
                if ($('.selected-attach').length && opSelect.val() >= 0) {
                    saveBtn.enable();
                } else {
                    saveBtn.enable(false);
                }
            };

            startBtn.click(function() {
                bulkContainer.fadeIn();
                if (!t.isAgile) {bulkContainer.addClass('attach-cat-bulk-container-table');}
                catChkBox.removeAttr('checked');
                opSelect.enable();
                catSelect.enable();
                showBoxes();
                hideAttachmentsActions();
                $(document).click();
                t.categoriesContainer.sortable("option", "disabled", true);
                return false;
            });

            opSelect.change(function() {
                opSelect.find('option[value="' + $(this).val() + '"]').attr('selected', 'selected');
                if ($(this).val() == 2 || $(this).val() == 4) {
                    catSelect.show();
                } else {
                    catSelect.hide();
                }

                if ($(this).val() == 4) {
                    zipName.show().val('');
                } else {
                    zipName.hide();
                }

                if (saveBtn.is(':visible')) {
                    showBoxes();
                }

                enableConfirm();
            });

            catSelect.change(function() {
                catSelect.find('option[value="' + $(this).val() + '"]').attr('selected', 'selected');
            });

            catChkBox.click(function(e) {
                e.stopPropagation();
                var box = $(this);
                var catContainer = box.parents('.attach-cat-container');
                if (box.is(':checked')) {
                    catContainer.find('.attachment-content').not('.revision').addClass('selected-attach');
                } else {
                    catContainer.find('.selected-attach').removeClass('selected-attach');
                }
                enableConfirm();
            });

            saveBtn.click(function() {
                var $selectedAttach = $('.selected-attach');
                var ids = [];
                var docIds = [];
                var froms = [];
                var items = [];
                $selectedAttach.each(function() {
                    var item = $(this);
                    items.push(item);
                    ids.push(t.getItemId(item));
                    froms.push($(this).parents('.attach-cat-container').attr('id'));

                    if (item.data('document')) {
                        docIds.push(item.data('documentId'));
                    }
                });

                switch (opSelect.val()) {
                    case "0": {
                        SmartAttachAnalytics.eventBulkOpDelete(items.length);

                        JIRA.SmartAjax.makeRequest({
                            type: 'POST',
                            url: AJS.params.baseURL + '/rest/attach-cat/1.0/attachments/remove',
                            data: {
                                issueKey: t.getIssueKey(),
                                ids: ids
                            },
                            success: function () {
                                $.each(docIds, function(i, id) {
                                    $('.revision.' + id).remove();
                                });
                                $selectedAttach.remove();
                                t.checkCategoriesEmptiness();
                                hideBulk();
                                t.drawBadges();
                            }
                        });

                        break;
                    }
                    case "1": {
                        SmartAttachAnalytics.eventBulkOpDownload(items.length);
                        downloadFileFromPath(AJS.params.baseURL + '/rest/attach-cat/1.0/attachments/download-bulk?' +
                            'issueKey=' + t.getIssueKey() +
                            "&ids=" + ids.join("&ids="));
                        hideBulk();
                        break;
                    }
                    case "2": {
                        SmartAttachAnalytics.eventBulkOpMove(items.length);
                        for (var i = 0; i < ids.length; i++) {
                            var item = items[i];
                            var target = t.isItemFile(item) ? "files" : "images";
                            t.move(froms[i], catSelect.val(), t.categorySection.find('#' + catSelect.val()),
                                item.parents('.attach-cat-container'), item, target);
                        }
                        hideBulk();
                        break;
                    }
                    case "3": {
                        hideBulk();
                        var issueAttachmentsForSend = SADialogEmail.getIssueAttachmentsForSend(ids, items);
                        SADialogEmail.createAndShowEmailDialog(issueAttachmentsForSend);
                        break;
                    }
                    case "4": {
                        SmartAttachAnalytics.eventBulkOpArchive(items.length);
                        $.each(docIds, function(i, id) {
                            $('.revision.' + id).each(function() {
                                ids.push(t.getItemId($(this)));
                            });

                        });

                        JIRA.SmartAjax.makeRequest({
                            type: 'POST',
                            url: AJS.params.baseURL + '/rest/attach-cat/1.0/attachments/archive',
                            data: {
                                issueKey: t.getIssueKey(),
                                ids: ids,
                                docIds: docIds,
                                cat: catSelect.val() || 0,
                                name: zipName.val() || AJS.I18n.getText("attach.cat.archive.def.name")
                            },
                            success: function() {
                                if (t.isAgile) {
                                    GH.DetailsView.reload();
                                } else {
                                    JIRA.trigger(JIRA.Events.REFRESH_ISSUE_PAGE, [JIRA.Issue.getIssueId()]);
                                }
                                JIRA.Messages.showMsg(AJS.I18n.getText("attach.cat.archive.success"), {
                                    type: JIRA.Messages.Types.SUCCESS,
                                    closeable: false,
                                    timeout: 3
                                });
                            },
                            error: function () {
                                JIRA.Messages.showMsg(AJS.I18n.getText("attach.cat.archive.error"), {
                                    type: JIRA.Messages.Types.ERROR,
                                    closeable: false,
                                    timeout: 3
                                });
                            }
                        });
                        break;
                    }
                    default : {}
                }

                return false;
            });

            cancelBtn.click(function() {
                hideBulk();
                return false;
            });

            function showBoxes() {
                if (opSelect.val() != 0) {
                    catChkBox.show();
                    catChkBox.parents('.attach-cat-title').addClass('attach-cat-title-with-checkbox');
                }
            }

            function hideBulk() {
                opSelect.find('option').first().attr('selected', 'selected');
                catSelect.find('option').first().attr('selected', 'selected');
                opSelect.change().enable(false);
                catSelect.enable(false);
                saveBtn.enable(false);
                bulkContainer.fadeOut().promise().done(function(){
                    bulkContainer.removeClass('attach-cat-bulk-container-table');
                });
                catChkBox.hide();
                catChkBox.parents('.attach-cat-title').removeClass('attach-cat-title-with-checkbox');
                t.categoriesContainer.sortable("option", "disabled", false);
                showAttachmentsActions();
            }

            function hideAttachmentsActions() {
                var attachments = $('.attach-files, .attach-images');
                attachments.find('.blender, .attachment-delete').each(function () {
                    $(this).addClass('inactive-item');
                });
                attachments.find('.attach-cat-comment, .sa-attach-header-line').addClass('inactive-item');

                var attContent = $('.attachment-content');

                attContent.each(function(){
                    var t = $(this);
                    t.prepend("<div class='sa-invisible-panel-wrap'><div class='sa-invisible-panel'></div></div>");
                    t.find('.sa-invisible-panel').css({
                        width: $(this).outerWidth()  + "px",
                        height: $(this).outerHeight() + "px"
                    });
                });

                attContent.not('.revision').on('click.bulkOp', function (e) {
                    var li = $(e.target).closest('li.attachment-content');
                    if (!li.hasClass('selected-attach')) {
                        li.addClass('selected-attach');
                    } else {
                        li.removeClass('selected-attach');
                    }

                    var catContainer = li.parents('.attach-cat-container');
                    var allAttachments = catContainer.find('.attachment-content').not('.revision');
                    var selectedAttachments = catContainer.find('.selected-attach');
                    catContainer.find('.attach-cat-bulk-checkbox').prop('checked', allAttachments.length == selectedAttachments.length);

                    enableConfirm();
                    return false;
                });
            }

            function showAttachmentsActions() {
                $('.sa-invisible-panel').remove();
                $('.attachment-content').off('click.bulkOp');
                $('.selected-attach').removeClass('selected-attach');
                $('.inactive-item').removeClass('inactive-item');
                AttachCat.bindDocumentExpandEvents();
            }
        },

        bindDocumentDelete: function(document) {
            var t = this;

            document.find('a.icon-delete, .aui-iconfont-delete').click(function() {
                var dialog = new AJS.Dialog({
                    width: 600,
                    height: 180,
                    id: "delete-attach-document-dialog",
                    closeOnOutsideClick: false
                });

                var docName = t.getItemNameElement(document).eq(0).text();
                dialog.addHeader(AJS.I18n.getText("attach.cat.delete.document.dialog.title", docName));

                var body = AttachCategory.deleteDocumentDialog();

                dialog.addPanel('panel', body, 'delete-attach-document-dialog-panel');

                dialog.addButton(AJS.I18n.getText("attach.cat.action.delete"), function() {

                    JIRA.SmartAjax.makeRequest({
                        type: 'POST',
                        url: AJS.params.baseURL + '/rest/attach-cat/1.0/attachments/remove',
                        data: {
                            issueKey: t.getIssueKey(),
                            ids: t.getItemId(document)
                        },
                        success: function() {
                            dialog.remove();
                            if (t.isAgile) {
                                GH.DetailsView.reload();
                            } else {
                                JIRA.trigger(JIRA.Events.REFRESH_ISSUE_PAGE, [JIRA.Issue.getIssueId()]);
                            }
                        }
                    });
                }, "save-attach-rename");

                dialog.addLink(AJS.I18n.getText("attach.cat.rename.dialog.cancel"), function (dialog) {
                    dialog.remove();
                }, '#');

                dialog.show();

                return false;
            });
        },

        bindFixEvents: function() {
            var t = this;
            //JIRA 6.1 fix
            $('#attachment-sorting-options, #attachment-sorting-order-options').click(function() {
                $(document).click().one('ajaxComplete', function() {
                    if (t.categorySection.is(":visible")) {
                        t.reinit();
                    }
                });
            });
        },

        bindDocumentExpandEvents: function() {
            var t = this;

            $('.attach-cat-version-shower').click(function() {
                var btn = $(this);
                var id = btn.data('id');
                AttachCatVersioning.issueId = t.issueId;
                AttachCatVersioning.expandedDocs[id] = true;
                btn.hide();
                btn.parent().children('.attach-cat-version-hider').show();
                t.categoriesContainer.find('.revision.' + id).show();
            });

            $('.attach-cat-version-hider').click(function() {
                var btn = $(this);
                var id = btn.data('id');
                AttachCatVersioning.expandedDocs[id] = false;
                btn.hide();
                btn.parent().children('.attach-cat-version-shower').show();
                t.categoriesContainer.find('.revision.' + id).hide();
            });

            if (AttachCatVersioning.issueId == t.issueId) {
                $.each(AttachCatVersioning.expandedDocs, function(id, shouldShow) {
                    if (shouldShow) {
                        $('.attach-cat-version-shower[data-id="' + id + '"]').click();
                    }
                });
            }
        },

        bindFeatureDiscoveryEvents: function() {
            var popup = this;
            $('#attach-cat-feature-close').click(function() {
                popup.hide();
                return false;
            });
        },

        bindDialogExtendEvents: function () {
            var t = this;
            var jiraDialogHideEventKey = 'Dialog.hide.sa-attach-dialog';

            var onHideDialog = function () {
                t.catId = 0;
                t.versioning = null;
            };

            $(JIRA.Dialog)
                .off(jiraDialogHideEventKey)
                .on(jiraDialogHideEventKey, onHideDialog);
        },

        bindGoEdit: function() {
            $(document).off('goedit:file-saved').on('goedit:file-saved', function(e, params) {
                var category = AttachCat.getItemById(params[0].fileID).parents('.attach-cat-container').attr('id');
                var documentId;
                AttachCat.getItemById(params[0].fileID).each(function(){
                    var $elem = $(this);
                    if ($elem.hasClass('revision')) {
                        documentId = $elem.data('revisionId');
                    }
                });

                JIRA.SmartAjax.makeRequest({
                    type: 'POST',
                    async: false,
                    url: AJS.params.baseURL + '/rest/attach-cat/1.0/attachments/goedit-integration/replace-move',
                    data: {
                        oldAttachId: params[0].fileID,
                        oldAttachName: params[0].fileName,
                        oldAttachCategoryId: category,
                        isReplace: params[0].replace,
                        documentId: documentId,
                        parent: params[0].fileID,
                        issueKey: AttachCat.getIssueKey()
                    },
                    success: function() {
                        isDoc || !params[0].replace ? JIRA.trigger(JIRA.Events.REFRESH_ISSUE_PAGE, [JIRA.Issue.getIssueId()]) :  location.reload();
                    }
                });
            });

            $('.sa-goedit-integration a').off('click').on('click', function (e) {
                e.preventDefault();
                var id = $(e.currentTarget).parents('.aui-list').data('attachId');
                var attachName = AttachCat.getItemNameElement(AttachCat.getItemById(id));
                SmartAttachAnalytics.eventEditWithGoEdit(attachName.attr('href'));
                $(document).trigger('goedit:open-file', [{"fileID": id, "fileName": attachName.text()}]);
            });

            $('.attachment-content').not('.revision').each(function () {
                AttachCat.goeditPopoverInit($(this));
            });
        },

        bindRemoveDocumentsFromDropdownWikiEditor: function () {
            $(document).on('aui-dropdown2-show', '.wiki-edit-dropdown-attachment', function (e) {
                var documentNames = [];
                $("#attachment_thumbnails li.attachment-content, #file_attachments li.attachment-content").each(function () {
                    var attachment = $(this);
                    if (attachment.data('document')) {
                        documentNames.push(attachment.find('a.attachment-title, .attachment-title a').first().text());
                    }
                });

                $(e.target).find('.wiki-edit-attachment-name').each(function () {
                    var attachmentDropdownItem = $(this);
                    if (documentNames.indexOf(attachmentDropdownItem.text()) > -1) {
                        attachmentDropdownItem.closest('li').remove();
                    }
                });
            });
        },

        move: function(oldId, newId, newContainer, oldContainer, item, target) {
            if (oldId === newId) return;
            SmartAttachAnalytics.eventAttachmentMoveBetweenCat();
            var t = this;
            var isDoc = item.data('document');
            var attachments = [];
            var revisions;
            var a = $('.aui-list[data-attach-id="' + t.getItemId(item) + '"]').find('#' + newId);
            var oldCatName = oldContainer.find('.attach-cat-title').data('title');

            if (a.length) {
                a.attr('id', oldId);
                a.attr('title', oldCatName);
                a.find('strong').html(oldCatName);
            }

            if (isDoc) {
                revisions = t.categoriesContainer.find('.revision.' + item.data('documentId'));
                revisions.each(function() {
                    attachments.push(t.getItemId($(this)));
                });
            } else {
                attachments.push(t.getItemId(item));
            }

            JIRA.SmartAjax.makeRequest({
                type: 'POST',
                url: AJS.params.baseURL + '/rest/attach-cat/1.0/attachments/move',
                data: {
                    issueKey: t.getIssueKey(),
                    toCatId: newId,
                    attachments: attachments
                },
                success: function() {
                    var newContainerTarget = newContainer.find('.attach-' + target);
                    newContainerTarget.append(item);
                    if (revisions) {
                        newContainerTarget.append(revisions);
                    }

                    t.checkCategoryEmptiness(newContainer);
                    t.checkCategoryEmptiness(oldContainer);
                    t.drawBadges();
                }
            });
        },

        checkCategoriesEmptiness: function() {
            var t = this;

            $('.attach-cat-container').each(function() {
                t.checkCategoryEmptiness($(this))
            });
        },

        checkCategoryEmptiness: function(cat) {
            var t = this;

            if (cat.find('.attachment-content').length == 0) {
                cat.find('.attach-cat-download-all').hide();
                t.hideCategory(cat);
            } else {
                cat.find('.attach-cat-download-all').show();
                t.showCategory(cat);
            }
        },

        toggleHideCategory: function(category) {
            category.toggleClass("collapsed");
            category.children('.item-attachments').toggle();

            if (!category.find('.attachment-content').length) {
                category.children('.attach-cat-empty').toggle();
            }
        },

        hideCategory: function(category) {
            category.addClass("collapsed");
            category.children('.item-attachments').hide();
            category.children('.attach-cat-empty').hide();
        },

        showCategory: function(category) {
            category.removeClass("collapsed");
            category.children('.item-attachments').show();

            if (!category.find('.attachment-content').length) {
                category.children('.attach-cat-empty').show();
            } else {
                category.children('.attach-cat-empty').hide();
            }
        },

        convertImageToFile: function(item) {
            var dt = item.find('dt');
            dt.find('.blender').remove();
            dt.after(item.find('dd.attachment-date'));

            if (item.find('.icon-delete, .aui-iconfont-delete').length) {
                dt.after("<dd class='attachment-delete'></dd>");
                item.find('dd.attachment-delete').append(dt.find('.attachment-delete').removeClass('attachment-delete'));
            } else {
                var deleteItem = dt.find('dd.attachment-delete');
                if (!this.isAgile) {
                    deleteItem.append('<span class="icon"></span>')
                }
                dt.after(deleteItem);
            }

            item.find('dd.attachment-size').after(item.find('dd.attach-cat-comment'));
        },

        initAttachMenuLink: function() {
            if (this.isServiceDeskAttachmentModuleExist()) {
                return;
            }

            var li = $('#issue-content').find('#attach-file');
            if (li.length > 0) li.removeClass('unified-attach-file').addClass('issueaction-attach-file');
        },

        checkOnExistTargetCat: function(category) {
            if (category.hasClass('attachments-drop-zone__dragover-fade') || category.hasClass('issue-container') || category.find('.attach-cat-title').text() == '') {
                var id = Number(category.attr('id'));
                if (!isNaN(id)) {
                    category = $('#attach-cat-categories').find('.attach-cat-container[id="' + category.attr('id') + '"]');
                } else {
                    category = $('#attach-cat-categories').find('.attach-cat-container[id="0"]');
                }
            }
            return(category);
        },

        addCommentIcon: function (attachment) {
            if (attachment.find('dd.attach-cat-comment').length === 0) {
                if (attachment.parent().hasClass('attach-images')) {
                    attachment.find('.attachment-size').before(AttachCategory.comment());
                } else {
                    attachment.find('.attachment-size').after(AttachCategory.comment());
                }
            }
        },

        initGHXBoardNavigation: function () {
            var agileIcon = $('a[href="#ghx-tab-com-stiltsoft-jira-smart-attachments-attachment-cat-issue-section-agile"]');
            if (agileIcon.length) {
                var agileIconHref = agileIcon.attr('href');
                var agileTabItem = $('a[href="#ghx-tab-attachments"], a[href="#attachmentmodule"]');
                agileTabItem.attr('href', agileIconHref);
                var section = agileTabItem.parent();
                agileIcon.parent().before(section);
                agileIcon.parent().remove();
            }
        },

        removeArchiveExpandIconForRevision: function (document) {
            document.find('.twixi-wrap.concise a.twixi').hide();
        },

        markAttachmentAsHasComment: function ($attachment) {
            $attachment.find('dd.attach-cat-comment').addClass('hasComment');
        },

        unmarkAttachmentAsHasComment: function ($attachment) {
            $attachment.find('dd.attach-cat-comment.hasComment').removeClass('hasComment');
        },

        getItemId: function(item) {
            return item.find('a[href*="/secure/attachment/"]').attr('href').match(/\/(\d+)\//)[1];
        },

        getItemById: function(id) {
            return this.categorySection.find('a[href*="/secure/attachment/' + id + '/"]').parents('.attachment-content');
        },

        reselectAttachment: function (attachment) {
            return AttachCat.getItemById(AttachCat.getItemId(attachment));
        },

        getItemNameElement: function(item) {
            return item.find('dl > dt > a').first();
        },

        getAttachmentNameForEditInOffice: function(nameElement) {
            var attachmentName = nameElement.text().trim().replace('#', '');
            return encodeURIComponent(EditInOffice.makeTempAttachmentNameForURL(attachmentName));
        },

        isItemFile: function(item) {
            return item.parents('.item-attachments').hasClass('attach-files');
        },

        getIssueKey: function() {
            return AttachCatDialogHandler.getIssueKey();
        },

        getProjectKey: function() {
            var issueKey = this.getIssueKey();
            return issueKey.substr(0, issueKey.lastIndexOf("-"));
        },

        getIdsPositionsOfCategories: function () {
            var ids = [];
            AttachCat.categoriesContainer.find('.attach-cat-container').each(function () {
                ids.push($(this).attr('id'));
            });
            return ids;
        },

        getAttachmentsFromSection: function () {
            return this.categorySection.find('.attachment-content');
        },

        goeditPopoverInit: function ($attachment) {
            $(document).trigger('goedit:open-popover', {
                element: $attachment.find('.attachment-title a, a.attachment-title').get(0),
                fileID: AttachCat.getItemId($attachment),
                fileName: AttachCat.getItemNameElement($attachment).text()
            });

            if (AttachCat.isGoEditPresent()) {
                $attachment.find('.sa-goedit-integration').show();
            }
        },

        isGoEditPresent: function () {
            if (AttachCat.isGoEditPresentFlag === undefined) {
                AttachCat.isGoEditPresentFlag = $(document).find("#goedit-popover-wrapper").length > 0;
            }
            return AttachCat.isGoEditPresentFlag;
        },

        convertAttachNamesToTempAttachments: function (attachmentNames, categoryId) {
            var tempAttachments = [];

            $.each(attachmentNames, function (i, attachmentName) {
                tempAttachments.push(new AttachCatClasses.TempAttachment(attachmentName, categoryId));
            });

            return tempAttachments;
        },

        getDocumentAttachmentIdsFromDataCategory: function (category) {
            var attachmentIds = [];
            $.each(category.documents, function (i, document) {
                attachmentIds = $.merge(attachmentIds, document.attachmentIds);
            });
            return attachmentIds;
        },

        isServiceDeskAttachmentModuleExist: function () {
            var metadata = $('#dnd-metadata-webpanel');
            return metadata.length && metadata.data('projectType') === 'service_desk' && !this.agileSection;
        }
    };

    AJS.toInit(function () {
        waitForInit();
    });

    function waitForInit() {
        setTimeout(function() {
            var sections = $('#attachment-cat-issue-section, #ghx-tab-com-stiltsoft-jira-smart-attachments-attachment-cat-issue-section-agile');

            if (sections.length) {
                if (JIRA.Dialogs.attachFile.options.trigger != "a.load-version") {
                    JIRA.Dialogs.deleteIssueLink.options.trigger += ", .sa-form-delete-attachment-link";
                    new JIRA.FormDialog(JIRA.Dialogs.deleteIssueLink.options);

                    initAttachFileDialog(sections);

                    JIRA.Dialogs.editComment.options.trigger = "a.edit-attach-comment";
                    JIRA.Dialogs.editComment.options.id = "edit-attach-comment-dialog";
                    new JIRA.FormDialog(JIRA.Dialogs.editComment.options);
                }

                AttachCat.init();
            } else {
                waitForInit();
            }
        }, 100);
    }

    function isGreenHopperSection(sections) {
        return sections.attr('id').indexOf('ghx') !== -1;
    }

    function initAttachFileDialog(sections) {
        if (isGreenHopperSection(sections)) {
            initGHFormDialog();
        } else {
            initJIRAFormDialog();
        }
    }

    function initJIRAFormDialog() {
        JIRA.Dialogs.attachFile.options.trigger = "a.load-version";
        new JIRA.FormDialog(JIRA.Dialogs.attachFile.options);
    }

    function initGHFormDialog() {
        var flagAlreadyBoundDialog = 'sa-load-version-bind';

        if (isBoundUploadNewRevFormDialog(flagAlreadyBoundDialog)) {
            return;
        }

        if (GH && GH.FormDialog) {

            new GH.FormDialog({
                id: "attach-file-dialog",
                trigger: "a.load-version",
                handleRedirect: true,
                issueMsg: 'thanks_issue_attached',
                onContentRefresh: function () {
                    $(".overflow-ellipsis").textOverflow();
                }
            });

            markUploadNewRevFormDialogBound(flagAlreadyBoundDialog);
        }
    }

    function isBoundUploadNewRevFormDialog(flagAlreadyBoundDialog) {
        return $(document.body).hasClass(flagAlreadyBoundDialog);
    }

    function markUploadNewRevFormDialogBound(flagAlreadyBoundDialog) {
        $(document.body).addClass(flagAlreadyBoundDialog);
    }

    function showAutocloseableWarningMessage(message) {
        var flag = JIRA.Messages.showMsg(message, {
            type: JIRA.Messages.Types.WARNING,
            timeout: 0,
            closeable: true
        });
        setTimeout(function () {
            flag[0].close();
        }, 20000);
    }

    function isFVIModuleEnabled() {
        return !isFVIModuleDisabled();
    }

    function isFVIModuleDisabled() {
        return typeof FVI === 'undefined';
    }

    function downloadFileFromPath(path) {
        var a = document.createElement('A');
        a.href = path;
        a.download = path.substr(path.lastIndexOf('/') + 1);
        document.body.appendChild(a);
        a.click();
        document.body.removeChild(a);
    }

})(AJS.$);