(function ($) {

    window.SAProjectCategoryAutoDistribution = {

        bindActions: function () {
            this.bindButtonClick();
            this.bindButtonHover();
        },

        bindButtonClick: function () {
            $(document.body).on('click', 'span.auto-distribution-button', function () {
                var autoDistributionButton = $(this);
                if (!autoDistributionButton.parent().hasClass('locked')) {
                    autoDistributionButton.addClass('active');

                    var ruleTypeId = SAProjectCategoryAutoDistribution.getRuleTypeId(autoDistributionButton);
                    var ruleValue = SAProjectCategoryAutoDistribution.getRuleValueFromButtonData(autoDistributionButton);
                    SAProjectCategoryAutoDistribution.showEditDialog(ruleTypeId, ruleValue);
                }
                return false;
            });
        },

        bindButtonHover: function () {
            $(document.body).on('mouseover', 'span.auto-distribution-button', function () {
                var buttonIcon = $(this);
                var tableRow = buttonIcon.closest('tr');
                var autoDistributionRule = SAProjectCategoryAutoDistribution.getSelectedAutoDistributionRule(tableRow);
                SAProjectCategoryAutoDistribution.setTitleForButton(buttonIcon, autoDistributionRule);
            });
        },

        showEditDialog: function (ruleTypeId, ruleValue) {
            var dialog = new AJS.Dialog({
                width: 480,
                height: 280,
                id: "auto-distribution-dialog",
                closeOnOutsideClick: false
            });

            dialog.addHeader(AJS.I18n.getText("attach.cat.project.dialog.auto-distribution.title"));

            var dialogBody = AttachCat.dialogBodyOfConfigureAutoDistribution({
                ruleTypeId: ruleTypeId,
                ruleValue: ruleValue
            });

            dialog.addPanel('panel', dialogBody, 'auto-distribution-dialog-panel');

            dialog.addButton(AJS.I18n.getText("attach.cat.action.save"), function () {
                SAProjectCategoryAutoDistribution.storeDataInMarkup();
                dialog.remove();
            }, "auto-distribution-save-button");

            dialog.addLink(AJS.I18n.getText("attach.cat.action.cancel"), function (dialog) {
                SAProjectCategoryAutoDistribution.removeClassActive();
                dialog.remove();
            }, '#');

            dialog.show();
        },

        storeDataInMarkup: function () {
            var autoDistributionButton = $('.auto-distribution-button.active');
            var container = autoDistributionButton.parent();
            var ruleTypeId = $('#auto-distribution-rule-type-dropdown').val();
            var ruleValue = $('#auto-distribution-textarea-value').val();

            if (ruleValue.trim().replace(/\r\n|\r|\n/g, '').length > 0) {
                SAProjectCategoryAutoDistribution.setRuleTypeIdAndValue(autoDistributionButton, ruleTypeId, ruleValue);
                container.addClass('filled');
            } else {
                SAProjectCategoryAutoDistribution.setRuleTypeIdAndValue(autoDistributionButton, "0", "");
                container.removeClass('filled');
            }
            SAProjectCategoryAutoDistribution.removeClassActive();
        },

        getRuleTypeId: function (autoDistributionButton) {
            var ruleType = autoDistributionButton.data('ruleType');
            switch (ruleType) {
                case "pattern":
                    return 0;
                case "regexp":
                    return 1;
                default:
                    return 0
            }
        },

        getRuleValueFromButtonData: function (autoDistributionButton) {
            var ruleValue = autoDistributionButton.data('ruleValue');
            return ruleValue ? ruleValue : '';
        },

        getRuleTypeFromButtonData: function (autoDistributionButton) {
            var ruleType = autoDistributionButton.data('ruleType');
            return ruleType ? ruleType : 'pattern';
        },

        getSelectedAutoDistributionRule: function (tr) {
            var autoDistributionButton = tr.find('.aui-iconfont-configure');
            return {
                type: SAProjectCategoryAutoDistribution.getRuleTypeFromButtonData(autoDistributionButton),
                value: SAProjectCategoryAutoDistribution.getRuleValueFromButtonData(autoDistributionButton)
            }
        },

        setRuleTypeIdAndValue: function (autoDistributionButton, ruleTypeId, ruleValue) {
            var ruleType = ruleTypeId === "0" ? "pattern" : "regexp";
            SAProjectCategoryAutoDistribution.setRuleTypeAndValue(autoDistributionButton, ruleType, ruleValue);
        },

        setRuleTypeAndValue: function (autoDistributionButton, type, value) {
            if (value.length !== 0) {
                autoDistributionButton.data('ruleType', type);
                autoDistributionButton.data('ruleValue', value);
            } else {
                autoDistributionButton.data('ruleType', 'pattern');
                autoDistributionButton.data('ruleValue', '');
            }
        },

        setTitleForButton: function (button, rule) {
            if (rule.value) {
                button.attr('title', AJS.I18n.getText('attach.cat.project.table.auto-distribution.button.hover.title', rule.type, rule.value));
            } else {
                button.removeAttr('title')
            }
        },

        restoreStoredValues: function (cancelButton) {
            var $container = $(cancelButton.closest('tr').find('.auto-distribution-container'));
            var autoDistributionButton = $container.find('.aui-iconfont-configure');
            var storedValues = $container.find('.auto-distribution-stored-values');
            var ruleType = storedValues.data('ruleType');
            var ruleValue = storedValues.data('ruleValue');
            ruleValue = ruleValue ? ruleValue : "";
            SAProjectCategoryAutoDistribution.setRuleTypeAndValue(autoDistributionButton, ruleType, ruleValue);

            if (ruleValue) {
                $container.addClass('filled');
            } else {
                $container.removeClass('filled');
            }
        },

        clearCreateRowData: function (row) {
            var container = row.find('.auto-distribution-container');
            var button = $('#attach-cat-add-auto-distribution');
            button.data('ruleType', 'pattern');
            button.data('ruleValue', '');
            container.removeClass('filled');
        },

        persistInMarkupStoredValues: function (tr) {
            var autoDistributionRule = SAProjectCategoryAutoDistribution.getSelectedAutoDistributionRule(tr);
            var storedValuesContainer = tr.find('.auto-distribution-stored-values');
            SAProjectCategoryAutoDistribution.setRuleTypeAndValue(storedValuesContainer, autoDistributionRule.type, autoDistributionRule.value);
        },

        lockButton: function (tr) {
            var container = tr.find('.auto-distribution-container');
            container.addClass('locked');
            container.removeClass('unlocked');
        },

        unlockButton: function (tr) {
            var container = tr.find('.auto-distribution-container');
            container.removeClass('locked');
            container.addClass('unlocked');
        },

        removeClassActive: function () {
            $('.auto-distribution-button.active').removeClass('active');
        }
    }

})(AJS.$);
