(function ($) {

    window.SAProjectCategory = {

        init: function () {
            SmartAttachAnalytics.viewConfigPage();

            this.initToggleButton();
            this.loadCategories();
            this.bindEvents();
            this.initSelect2Inputs();

            SAProjectCategoryDialogs.showHelp();

            window.onbeforeunload = null;
        },

        initToggleButton: function () {
            var toggleAddon = $('#enable-smart-attach-checkbox');
            var wrapper = $('#attach-cat-table-wrapper');
            var table = wrapper.find('table.attach-cat');
            var hidable = wrapper.find('.hidable');

            if (toggleAddon.is(":checked")) {
                wrapper.css('position', 'inherit');
                table.removeClass('sa-hidden-block');
            } else {
                table.addClass('sa-hidden-block');
                hidable.hide();
            }
        },

        loadCategories: function () {
            JIRA.SmartAjax.makeRequest({
                type: 'GET',
                url: AJS.params.baseURL + '/rest/attach-cat/1.0/project/category',
                data: {projectKey: getProjectKey()},
                success: this.loadCategoriesDone,
                error: function () {
                    SAProjectCategory.showErrorMessage("attach.cat.get.error", 3, false);
                }
            });
        },

        loadCategoriesDone: function (data) {
            var tBody = $('#attach-categories');
            S2Wrapper.loadSelectedRestrictions(getProjectKey(), $(S2Wrapper.defSourceRestrictionsContainer));
            SAProjectCategory.appendCategoriesToTable(tBody, data.categories);
            $('#attach-cat-loader').hide();
            $('#sa-attach-cat-table').addClass('sa-categories-data-loaded');
        },

        bindEvents: function () {
            this.bindToggleButton();
            this.bindAddNewTableRow();
            this.bindChangeThumbnailSize();
            this.bindChangeFormFieldType();
            this.bindDeleteEvents();
            this.bindEditEvents();
            this.bindTableRowsSortable();
            this.bindCreteCategoryInputChange();
            this.bindImportButton();
            SAProjectCategoryAutoDistribution.bindActions();
        },

        bindToggleButton: function () {
            $('#enable-smart-attach-checkbox').change(this.toggleButtonChange);
        },

        bindAddNewTableRow: function () {
            $('#attach-cat-add').click(this.addNewRow);
        },

        bindChangeThumbnailSize: function () {
            $('#smart-attach-image-size').change(this.changeThumbnailSize);
        },

        bindChangeFormFieldType: function () {
            $('#sa-type-form-field').change(this.changeFormFieldType);
        },

        bindDeleteEvents: function () {
            $('.attach-cat-delete').live('click', function () {
                $(this).parent().hide();
                $(this).parents('td').children('.attach-cat-delete-container').show();
                return false;
            });

            $('.attach-cat-delete-cancel').live('click', function () {
                $(this).parent().hide();
                $(this).parents('td').children('.attach-cat-first-action-container').show();
                return false;
            });

            $('.attach-cat-delete-save').live('click', function () {
                var tr = $(this).parents('tr');
                SAProjectCategory.deleteCategoryConfirm(tr);
                return false;
            });
        },

        bindEditEvents: function () {
            $('.attach-cat-edit').live('click', function () {
                $(this).parent().hide();
                $(this).parents('td').children('.attach-cat-edit-container').show();

                var tr = $(this).parents('tr');
                var nameSpan = tr.find('.attach-cat-name');
                var input = tr.find('.attach-cat-input');
                input.val(nameSpan.text()).trigger('input');
                nameSpan.hide();
                input.show();

                var selectRestrictions = tr.find('.attach-cat-select-restrictions');
                var selectIssueTypes = tr.find('.attach-cat-select-issue-type');
                selectRestrictions.enable();
                selectIssueTypes.enable();

                SAProjectCategoryAutoDistribution.unlockButton(tr);

                SmartAttachAnalytics.eventEditCategory();
                return false;
            });

            $('.attach-cat-edit-cancel').live('click', function () {
                var cancelButton = $(this);
                cancelButton.parent().hide();
                cancelButton.parents('td').children('.attach-cat-first-action-container').show();

                var tr = cancelButton.parents('tr');
                tr.find('.attach-cat-input').hide();
                tr.find('.attach-cat-name').show();

                var inputRestrict = tr.find('.attach-cat-select-restrictions');
                var inputIssueTypes = tr.find('.attach-cat-select-issue-type');
                inputRestrict.attr('disabled', 'disabled');
                inputIssueTypes.attr('disabled', 'disabled');
                S2Wrapper.restoreSelected(inputIssueTypes);
                S2Wrapper.restoreSelected(inputRestrict);

                SAProjectCategoryAutoDistribution.lockButton(tr);
                SAProjectCategoryAutoDistribution.restoreStoredValues(cancelButton);

                return false;
            });

            $('.attach-cat-edit-save').live('click', function () {
                var btn = $(this);
                SAProjectCategory.editCategoryConfirm(btn);
                return false;
            });
        },

        bindTableRowsSortable: function () {
            var tBody = $('#attach-categories');
            var wrapper = $('#attach-cat-table-wrapper');

            tBody.parents().css('position', 'inherit');
            $('body').css('position', '');
            wrapper.css('position', 'relative');

            tBody.sortable({
                axis: 'y',
                containment: 'parent',
                cancel: "a,button,input,div.select2-container,li.active-result,span.aui-icon",
                tolerance: 'pointer',
                cursor: 'move',
                scroll: true,
                scrollSensitivity: 80,
                scrollSpeed: 3,
                start: function (e, ui) {
                    ui.item.children('td').eq(1).css('width', $('#attach-cat-issue-types').width());
                    ui.item.children('td').eq(2).css('width', $('#attach-cat-groups').width());
                },
                stop: function (e, ui) {
                    ui.item.children('td').eq(1).css('width', '');
                    ui.item.children('td').eq(2).css('width', '');
                    var ids = [];
                    tBody.find('tr').each(function () {
                        ids.push($(this).attr('id'));
                    });
                    JIRA.SmartAjax.makeRequest({
                        type: 'POST',
                        url: AJS.params.baseURL + '/rest/attach-cat/1.0/project/reorder',
                        data: {
                            projectKey: getProjectKey(),
                            ids: ids
                        },
                        success: function () {
                            $(".default-virtual-cat").removeClass('default-virtual-cat');
                        }
                    });
                }
            });
        },

        bindCreteCategoryInputChange: function () {
            $('.attach-cat-input').live('keydown input', this.creteCategoryInputChange);
        },

        bindImportButton: function () {
            $('#import-smart-attach').click(function (e) {
                e.preventDefault();
                SAProjectCategoryDialogs.showImport();
            });
        },

        initSelect2Inputs: function () {
            this.initIssueTypeSelect();
            this.initRestrictionsSelect();
        },

        initIssueTypeSelect: function () {
            S2Wrapper.initIssueTypes($('#attach-cat-issue-type'));
        },

        initRestrictionsSelect: function () {
            S2Wrapper.initRestrictions($('#attach-cat-group'));
        },

        deleteCategoryConfirm: function (tableRow) {
            JIRA.SmartAjax.makeRequest({
                type: 'DELETE',
                url: AJS.params.baseURL + '/rest/attach-cat/1.0/project/category',
                data: {
                    projectKey: getProjectKey(),
                    catId: tableRow.attr('id')
                },
                success: function () {
                    SAProjectCategory.removeTableRow(tableRow);
                },
                error: function () {
                    JIRA.Messages.showMsg(AJS.I18n.getText("attach.cat.change.error"), {
                        type: JIRA.Messages.Types.ERROR,
                        closeable: false,
                        timeout: 3
                    });
                }
            });
        },

        editCategoryConfirm: function (triggeredButton) {
            var tr = triggeredButton.parents('tr');
            var categoryName = tr.find('.attach-cat-input').val().trim();

            if (!SAProjectCategory.checkCatName(categoryName)) {
                return false;
            }

            var loader = triggeredButton.parent().children('.aui-icon-wait');
            triggeredButton.enable(false);
            loader.show();

            var inputRestrict = tr.find('.attach-cat-select-restrictions');
            var inputIssueTypes = tr.find('.attach-cat-select-issue-type');
            var data = SAProjectCategory.getDataForAddOrEditCategory(inputRestrict, inputIssueTypes);
            var autoDistributionRule = SAProjectCategoryAutoDistribution.getSelectedAutoDistributionRule(tr);

            JIRA.SmartAjax.makeRequest({
                type: 'PUT',
                url: AJS.params.baseURL + '/rest/attach-cat/1.0/project/category',
                data: {
                    projectKey: getProjectKey(),
                    catId: tr.attr('id'),
                    name: categoryName,
                    groups: data.groups,
                    roles: data.roles,
                    issueTypes: data.issueTypes,
                    users: data.users,
                    autoDistributionRuleType: autoDistributionRule.type,
                    autoDistributionRuleValue: autoDistributionRule.value
                },
                success: function (id) {
                    SAProjectCategory.editCategorySentSuccess(triggeredButton, categoryName, id);
                },
                error: function () {
                    SAProjectCategory.showErrorMessage("attach.cat.change.error", 3, false);
                },
                complete: function () {
                    triggeredButton.enable();
                    loader.hide();
                }
            });
        },

        editCategorySentSuccess: function (triggeredButton, categoryName, id) {
            var tr = triggeredButton.parents('tr');
            var input = tr.find('.attach-cat-input');
            var keySpan = tr.find('.attach-cat-name');
            var inputRestrict = tr.find('.attach-cat-select-restrictions');
            var inputIssueTypes = tr.find('.attach-cat-select-issue-type');

            tr.removeClass("default-virtual-cat").attr('id', id);
            keySpan.html(categoryName);
            input.hide();
            keySpan.show();
            inputRestrict.attr('disabled', 'disabled');
            inputIssueTypes.attr('disabled', 'disabled');
            S2Wrapper.synchronizeSelectedAndData(inputIssueTypes);
            S2Wrapper.synchronizeSelectedAndData(inputRestrict);
            triggeredButton.parent().hide();
            triggeredButton.parents('td').children('.attach-cat-first-action-container').show();

            SAProjectCategoryAutoDistribution.persistInMarkupStoredValues(tr);
            SAProjectCategoryAutoDistribution.lockButton(tr);

            SmartAttachAnalytics.eventSaveEditedCategory();
            if (input.data('lastValue') !== input.val()) {
                SmartAttachAnalytics.eventRenameCategory();
                input.data('lastValue', input.val());
            }
        },

        getDataForAddOrEditCategory: function (inputRestrict, inputIssueTypes) {
            var data = {
                groups: [],
                roles: [],
                issueTypes: [],
                users: []
            };
            return $.extend(data, S2Wrapper.getDataInputRestrictions(inputRestrict), S2Wrapper.getDataInputIssueTypes(inputIssueTypes));
        },

        removeTableRow: function (tr) {
            var defCatName = $('#attach-categories #0').find('.attach-cat-name').text();
            JIRA.Messages.showMsg(AJS.I18n.getText("attach.cat.delete.success", defCatName), {
                type: JIRA.Messages.Types.SUCCESS,
                closeable: true,
                timeout: 7
            });

            tr.remove();
            SmartAttachAnalytics.eventDeleteCategory();
        },

        appendCategoriesToTable: function (tBody, categories) {
            var defaultCat;

            $.each(categories, function (i, category) {
                SAProjectCategory.addCategoryRowIntoTable(tBody, category);
                if (category.id === 0) {
                    defaultCat = category;
                }
            });
        },

        creteCategoryInputChange: function (e) {
            var input = $(this);
            var btn = input.parents('tr').find('.aui-button-primary:visible');
            if (input.val().trim()) {
                btn.enable();
                if (e.keyCode === 13) {
                    btn.click();
                    return false;
                }
            } else {
                btn.enable(false);
            }
        },

        toggleButtonChange: function () {
            var toggleButton = $(this);

            if (toggleButton.is(":checked")) {
                SAProjectCategory.disable(toggleButton);
            } else {
                SAProjectCategoryDialogs.showDisableConfirmation(toggleButton);
            }
        },

        disable: function (toggleButton) {
            var table = $('#sa-attach-cat-table');
            var wrapper = $('#attach-cat-table-wrapper');
            var hidable = wrapper.find('.hidable');

            JIRA.SmartAjax.makeRequest({
                type: 'POST',
                url: AJS.params.baseURL + '/rest/attach-cat/1.0/project/disable',
                data: {
                    projectKey: getProjectKey(),
                    disable: !toggleButton.is(":checked")
                },
                success: function (data) {
                    if (data.isProjectEnabled) {
                        if (!toggleButton.is(":checked")) {
                            toggleButton.prop('checked', true);
                        }
                        wrapper.css('position', 'inherit');
                        table.removeClass('sa-hidden-block');
                        hidable.show();
                        SmartAttachAnalytics.eventEnableAddOn(data);

                        if (data.showFD === null) {
                            SAProjectCategoryDialogs.showFeatureDiscovery();
                        } else {
                            SAProjectCategoryDialogs.showHelp();
                        }
                    } else {
                        if (toggleButton.is(":checked")) {
                            toggleButton.prop('checked', false);
                        }
                        wrapper.css('position', 'relative');
                        table.addClass('sa-hidden-block');
                        hidable.hide();
                        SmartAttachAnalytics.eventDisableAddOn();
                    }
                }
            });
        },

        addNewRow: function () {
            var tBody = $('#attach-categories');
            var addRow = $('#attach-cat-add-row');
            var addBtn = $('#attach-cat-add');
            var loader = addRow.find('.aui-icon-wait');
            var categoryNameInput = $('#attach-cat-name');
            var categoryName = categoryNameInput.val().trim();

            if (!SAProjectCategory.checkCatName(categoryName)) {
                return false;
            }

            addBtn.enable(false);
            loader.show();

            var inputIssueTypes = $('#attach-cat-issue-type');
            var inputRestrict = $('#attach-cat-group');
            var data = SAProjectCategory.getDataForAddOrEditCategory(inputRestrict, inputIssueTypes);
            var autoDistributionRule = SAProjectCategoryAutoDistribution.getSelectedAutoDistributionRule(addRow);

            JIRA.SmartAjax.makeRequest({
                type: 'POST',
                url: AJS.params.baseURL + '/rest/attach-cat/1.0/project/category',
                data: {
                    projectKey: getProjectKey(),
                    name: categoryName,
                    groups: data.groups,
                    roles: data.roles,
                    issueTypes: data.issueTypes,
                    users: data.users,
                    autoDistributionRuleType: autoDistributionRule.type,
                    autoDistributionRuleValue: autoDistributionRule.value
                },
                success: function (id) {
                    var category = {
                        id: id,
                        name: categoryName,
                        groups: data.groups,
                        roles: data.roles,
                        issueTypes: data.issueTypes,
                        users: data.users,
                        autoDistributionRule: autoDistributionRule
                    };
                    var row = SAProjectCategory.addCategoryRowIntoTable(tBody, category);
                    S2Wrapper.copySelectedDataToNewInput(inputRestrict, row);
                    categoryNameInput.val('');
                    S2Wrapper.clearValue(inputRestrict);
                    S2Wrapper.clearValue(inputIssueTypes);
                    SAProjectCategoryAutoDistribution.clearCreateRowData(addRow);
                    SmartAttachAnalytics.eventCreateCategory();
                },
                error: function () {
                    addBtn.enable();
                    SAProjectCategory.showErrorMessage("attach.cat.change.error", 3, false);
                },
                complete: function () {
                    loader.hide();
                }
            });
            return false;
        },

        changeThumbnailSize: function () {
            var size = $(this).val();
            JIRA.SmartAjax.makeRequest({
                type: 'POST',
                url: AJS.params.baseURL + '/rest/attach-cat/1.0/project/image-size',
                data: {
                    projectKey: getProjectKey(),
                    size: size
                }
            });
        },

        changeFormFieldType: function () {
            var value = $(this).val();
            JIRA.SmartAjax.makeRequest({
                type: 'POST',
                url: AJS.params.baseURL + '/rest/attach-cat/1.0/project/form-field-type',
                data: {
                    projectKey: getProjectKey(),
                    isDropZones: value === "0"
                }
            });
        },

        addCategoryRowIntoTable: function (tBody, category, readOnly) {
            var row = $(AttachCat.category({category: category}));
            var inputRestrictions = row.find('.attach-cat-select-restrictions');
            var inputIssueTypes = row.find('.attach-cat-select-issue-type');

            var defaultVirtualCat = tBody.find(".default-virtual-cat");
            if (defaultVirtualCat.length) {
                defaultVirtualCat.before(row);
            } else {
                tBody.append(row);
            }

            S2Wrapper.initIssueTypes(inputIssueTypes);
            if (readOnly) {
                S2Wrapper.selectPresentedIssueTypeOptions(inputIssueTypes, category.issueTypes);
            } else {
                S2Wrapper.selectAllIssueTypeOptions(inputIssueTypes, category.issueTypes);
            }

            S2Wrapper.initRestrictions(inputRestrictions);
            var restrictSourceOptions = readOnly ? S2Wrapper.getImportRestrictionsOptions() : S2Wrapper.getRestrictionsSourceOptions();
            S2Wrapper.selectOptionsRestrict(inputRestrictions, category, restrictSourceOptions);
            return row;
        },

        checkCatName: function (name) {
            var lcName = name.toLowerCase();
            var unique = true;
            $('.attach-cat-name:visible').each(function () {
                if ($(this).text().toLowerCase() === lcName) {
                    unique = false;
                }
            });

            if (!unique) {
                JIRA.Messages.showMsg(AJS.I18n.getText("attach.cat.change.unique.warn"), {
                    type: JIRA.Messages.Types.WARNING,
                    closeable: true,
                    timeout: 5
                });
            }

            return unique;
        },

        importProjectsUpdateTable: function (projectKey, projects, tBody, isReadOnly) {
            tBody.empty();

            var project = getProjectByKey(projects, projectKey);

            $.each(project.config.categories, function (i, category) {
                var row = SAProjectCategory.addCategoryRowIntoTable(tBody, category, isReadOnly);
                S2Wrapper.initialWidthSelect2(row);
            });
        },

        showErrorMessage: function (i18key, timeout, isClosable) {
            JIRA.Messages.showMsg(AJS.I18n.getText(i18key), {
                type: JIRA.Messages.Types.ERROR,
                closeable: isClosable,
                timeout: timeout
            });
        }
    };

    function getProjectKey() {
        return $('meta[name=projectKey]').attr('content').trim();
    }

    function getProjectByKey(projects, key) {
        for (var i = 0; i < projects.length; i++) {
            if (projects[i].key === key) {
                return projects[i];
            }
        }
    }

    AJS.toInit(function () {
        SAProjectCategory.init();
    });

})(AJS.$);