(function ($) {

    window.S2Wrapper = {

        importSourceRestrictionsContainer: '#pr-import-source-options-select2',
        defSourceRestrictionsContainer: '#selected-restrictions-list',

        initIssueTypes: function (input) {
            input.select2({
                multiple: true,
                data: S2Wrapper.getIssueTypeSourceOptions,
                placeholder: input.data('placeholder'),
                formatResult: S2Wrapper.issueTypesFormat,
                formatSelection: S2Wrapper.issueTypesFormat,
                containerCssClass: 'aui-select2-container',
                dropdownCssClass: 'aui-select2-drop',
                initSelection: function (element, callback) {
                    var selected = input.data('selected');
                    if (selected) callback(selected);
                }
            });
        },

        initRestrictions: function (input) {
            input.select2({
                placeholder: input.data('placeholder'),
                multiple: true,
                ajax: {
                    url: AJS.params.baseURL + '/rest/attach-cat/1.0/project/usergrouprolesearch',
                    dataType: 'json',
                    cache: true,
                    data: function (value) {
                        return {
                            projectKey: S2Wrapper.getProjectKey(),
                            query: value,
                            limit: 10
                        };
                    },
                    results: S2Wrapper.convertToSelect2Format
                },
                minimumInputLength: 3,
                containerCssClass: 'aui-select2-container',
                dropdownCssClass: 'aui-select2-drop',
                initSelection: function (element, callback) {
                    var selected = input.data('selected');
                    if (selected) callback(selected);
                }
            });
        },

        initSelect2ImportProject: function (select, updateTableCallback) {
            select.select2({
                minimumResultsForSearch: Infinity,
                containerCssClass: 'aui-select2-container',
                dropdownCssClass: 'aui-select2-drop'
            }).on('change', function (e) {
                S2Wrapper.loadSelectedRestrictions(e.val, $(S2Wrapper.importSourceRestrictionsContainer));
                updateTableCallback(true);
            });
        },

        loadSelectedRestrictions: function (projectKey, $containerAppend) {
            JIRA.SmartAjax.makeRequest({
                type: 'GET',
                async: false,
                url: AJS.params.baseURL + '/rest/attach-cat/1.0/project/categories-selected-data',
                data: {projectKey: projectKey},
                success: function (data) {
                    $containerAppend.data(data);
                }
            });
        },

        issueTypesFormat: function (data) {
            if (!data.id) {
                return data.text;
            }
            return AttachCat.formatItemWithIcon({iconSrc: data.icon, text: data.text});
        },

        convertToSelect2Format: function (data) {
            var results = [];

            if (data.groups.length) {
                var groups = [];
                $.each(data.groups, function (i, group) {
                    groups.push(S2Wrapper.generateGroup(group));
                });
                results.push({'text': AJS.I18n.getText('common.words.groups'), 'children': groups});
            }

            if (data.roles.length) {
                var roles = [];
                $.each(data.roles, function (i, role) {
                    roles.push(S2Wrapper.generateRole(role));
                });
                results.push({'text': AJS.I18n.getText('common.words.project.roles'), 'children': roles});
            }

            if (data.users.length) {
                var users = [];
                $.each(data.users, function (i, user) {
                    users.push(S2Wrapper.generateUser(user));
                });
                results.push({'text': AJS.I18n.getText('admin.common.words.users'), 'children': users});
            }

            return {
                results: results
            };
        },

        selectPresentedIssueTypeOptions: function (input, issueTypeIds) {
            if (!issueTypeIds.length) {
                return;
            }

            var allOptions = S2Wrapper.getIssueTypeSourceOptions().results;
            var selected = [];

            $.each(issueTypeIds, function (i, id) {
                var sid = id.toString();
                $.each(allOptions, function () {
                    if (this.id === sid) {
                        selected.push(this);
                        return false;
                    }
                });
            });

            input.data('selected', selected);
            input.select2('val', []);
        },

        selectAllIssueTypeOptions: function (input, issueTypeIds) {
            if (!issueTypeIds.length) {
                return;
            }
            var allOptions = S2Wrapper.getIssueTypeSourceOptions().results;
            var selected = [];

            $.each(issueTypeIds, function (i, id) {
                var sid = id.toString();
                var founded = false;
                $.each(allOptions, function () {
                    if (this.id === sid) {
                        founded = true;
                        selected.push(this);
                        return false;
                    }
                });
                if (!founded) {
                    selected.push({id: id, text: id, icon: ''});
                }
            });

            input.data('selected', selected);
            input.select2('val', []);
        },

        selectOptionsRestrict: function (input, category, allOptions) {
            var selected = [];
            if (category.groups)
            $.each(category.groups, function (i, groupName) {
                $.each(allOptions.groups, function () {
                    if (this.name === groupName) selected.push(S2Wrapper.generateGroup(this));
                });
            });
            if (category.users)
            $.each(category.users, function (i, userKey) {
                $.each(allOptions.users, function () {
                    if (this.key === userKey) selected.push(S2Wrapper.generateUser(this));
                });
            });
            if (category.roles)
            $.each(category.roles, function (i, roleId) {
                $.each(allOptions.roles, function () {
                    if (this.id === roleId) selected.push(S2Wrapper.generateRole(this));
                });
            });
            input.data('selected', selected);
            S2Wrapper.restoreSelected(input);
        },

        getIssueTypeSourceOptions: function () {
            var issueTypes = [];
            $('#issue-types-list').find('option').each(function (i, option) {
                var $option = $(option);
                issueTypes.push({id: $option.val(), text: $option.text(), icon: $option.data('icon')});
            });
            return {results: issueTypes};
        },

        copySelectedDataToNewInput: function (currentInput, row) {
            var $row = $(row);
            var newRestrictionsInput = $row.find('.attach-cat-select-restrictions');
            newRestrictionsInput.select2('data', currentInput.select2('data'));
            newRestrictionsInput.data('selected', currentInput.select2('data'));
        },

        getDataInputRestrictions: function (inputRestrict) {
            var data = {};
            $.each(inputRestrict.select2('data'), function (i, item) {
                var value = item.id.replace(item.label + '_', '');
                data[item.label] = data[item.label] !== undefined ? data[item.label] : [];
                data[item.label].push(value);
            });
            return data;
        },

        getDataInputIssueTypes: function (inputIssueTypes) {
            var data = {issueTypes: []};
            $.each(inputIssueTypes.select2('data'), function (i, item) {
                data.issueTypes.push(item.id);
            });
            return data;
        },

        synchronizeSelectedAndData: function (input) {
            input.data('selected', input.select2('data'));
        },

        getRestrictionsSourceOptions: function () {
            return $(S2Wrapper.defSourceRestrictionsContainer).data();
        },

        getImportRestrictionsOptions: function () {
            return $(S2Wrapper.importSourceRestrictionsContainer).data();
        },

        getProjectKey: function () {
            return $('meta[name=projectKey]').attr('content').trim();
        },

        restoreSelected: function (input) {
            input.select2('val', []);
        },

        clearValue: function (input) {
            input.select2('val', '');
        },

        generateGroup: function (group) {
            return {'id': 'groups_' + group.name, 'text': group.name, 'label': 'groups'};
        },

        generateUser: function (user) {
            var text = user.displayName + ' (' + user.name + ')';
            return {'id': 'users_' + user.key, 'text': text, 'label': 'users'};
        },

        generateRole: function (role) {
            return {'id': 'roles_' + role.id, 'text': role.name, 'label': 'roles'};
        },

        initialWidthSelect2: function (row) {
            $(row).find('.restrictions-container, .issue-types-container').find('.select2-input').width('10px');
        }
    }

})(AJS.$);
