define('smart-attachments/service-desk-integration/CommentDNDExecutor',
    [
        'dndattachment/upload/handler',
        'smart-attachments/toolbox',
        'smart-attachments/base/CommonClasses',
        'smart-attachments/base/AutoDistribution',
        'jquery',
        'underscore',
        'exports'
    ],
    function (DnDUploadHandler, Toolbox, CommonClasses, AutoDistribution, $, _, exports) {

        var makeTempAttachment = function (file) {
            return new CommonClasses.TempAttachment(file.name, file.categoryId, file.parentAttachmentId);
        };

        var createCommentDnDExecutor = function () {
            return {
                name: 'Smart Attachment Service Desk Comment DnD Executor',
                weight: 150,

                isCategorized: function (payload) {
                    var files = payload.files;
                    return files && files.length > 0 && (typeof files[0].categoryId !== 'undefined');
                },

                isValid: function (event, args) {
                    if (!!args.isPaste) {
                        return false;
                    }
                    if (!this.isCategorized(args)) {
                        return false;
                    }
                    return (
                        (Toolbox.isSDProject() && args.isWikiTextfieldFocused) ||
                        (!Toolbox.isSDProject())
                    );
                },

                processFiles: function (files, attachmentDropZone) {
                    if (attachmentDropZone.categoryConfig) {
                        var issue = require('jira/issue');
                        var issueId = issue.getIssueId();
                        var Revisions = require('smart-attachments/dnd/upload/Revisions');
                        var deferred = $.Deferred();
                        var addRevisionInfo = Revisions.createFilesMatcher();
                        var addRevisions = addRevisionInfo(files);
                        var tempAttachments = _.map(files, makeTempAttachment);
                        Toolbox.attachFilesToCategories(issueId, tempAttachments)
                            .then(function () {
                                console.log("Attached to category");
                                SmartAttachmentsAnalytics.event('automaticDocumentRevisionUpload', [tempAttachments]);
                                attachmentDropZone
                                    .uploadFiles(files)
                                    .then(deferred.resolve)
                                    .fail(deferred.reject);
                            })
                            .fail(deferred.reject);
                        if (addRevisions) {
                            deferred.done(_.partial(Toolbox.invalidateDataAndView, issue));
                        }
                        return deferred;
                    } else {
                        return attachmentDropZone.uploadFiles(files);
                    }
                }
            };
        };

        exports.init = function () {
            DnDUploadHandler.registerExecutor(createCommentDnDExecutor());
        }
    });