define('smart-attachments/dnd/upload/WikiExecutor',
    [
        'dndattachment/upload/handler',
        'smart-attachments/toolbox',
        'smart-attachments/base/CommonClasses',
        'smart-attachments/base/AutoDistribution',
        'jquery',
        'underscore',
        'exports'
    ],
    function (DnDUploadHandler, Toolbox, CommonClasses, AutoDistribution, $, _, exports) {

        var makeTempAttachment = function (file) {
            return new CommonClasses.TempAttachment(file.name, file.categoryId);
        };

        var assignCategory = function(matcher) {
            return function(file) {
                file.matched = true;
                file.categoryId = matcher.match(file);
                return file;
            }
        };

        var wikiExecutor = function () {
            return {
                name: 'Smart Attachment Wiki Executor',
                weight: 250,

                isCategorized: function (payload) {
                    var files = payload.files;
                    return files && files.length > 0 && (typeof files[0].categoryId !== 'undefined');
                },

                isValid: function (event, args) {
                    if (!!args.isPaste) {
                        return false;
                    }
                    if (this.isCategorized(args)) {
                        return false;
                    }
                    return (args.wikiTextfield && !args.isWikiTextfieldFocused);
                },

                createMatcher: function (categoryId, categories) {
                    return AutoDistribution.createMatcher(categoryId, categories);
                },

                processFiles: function (files, attachmentDropZone) {
                    if (attachmentDropZone.categoryConfig) {
                        var issueId = require('jira/issue').getIssueId();
                        var deferred = $.Deferred();
                        var matcher = this.createMatcher('auto', attachmentDropZone.categoryConfig.categories);
                        var tempFiles = _.chain(files).map(assignCategory(matcher)).map(makeTempAttachment).value();
                        Toolbox.attachFilesToCategories(issueId, tempFiles)
                            .then(function () {
                                console.log("Attached to category");
                                SmartAttachmentsAnalytics.event('automaticDocumentRevisionUpload', [tempFiles]);
                                attachmentDropZone
                                    .uploadFiles(files)
                                    .then(deferred.resolve)
                                    .fail(deferred.reject);
                            })
                            .fail(deferred.reject);
                        return deferred;
                    } else {
                        return attachmentDropZone.uploadFiles(files);
                    }
                }
            };
        };

        exports.init = function () {
            console.log('Register Wiki Executor');
            DnDUploadHandler.registerExecutor(wikiExecutor());
        }
    });